#!/bin/sh -
# $Id: vxadm_lib.sh,v 1.48.34.14 2008/10/17 14:31:10 knarayan Exp $
#ident "$Source: /project/unixvm-cvs/src/linux/cmd/vxvm/voladm/vxadm_lib.sh,v $"

# $Copyright: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $


: ${VOLROOT_DIR:=$__VXVM_ROOT_DIR}
: ${VOL_SCRIPTS_DIR:=$VOLROOT_DIR/usr/lib/vxvm/bin}
: ${VOL_SCRIPTS_LIB:=$VOLROOT_DIR/usr/lib/vxvm/lib}
: ${VOLADM_ROOT:=$VOLROOT_DIR/usr/lib/vxvm/voladm.d}
: ${VOLADM_BIN=$VOLADM_ROOT/bin}
: ${VOLADM_LIB=$VOLADM_ROOT/lib}
: ${VOLADM_HELP=$VOLADM_ROOT/help}
: ${VOL_CONFIG_DIR:=$VOLROOT_DIR/etc/vx}
: ${VOL_LOCK_DIR:=$VROOT_DIR/var/tmp}
: ${VOL_TYPE_DIR:=$VROOT_DIR/usr/lib/vxvm/type}

SYSROOT=$__VXVM_SYSTEM_ROOT_DIR

ECHO='echo -e'; export ECHO		# Need special chars interpreted

export VOL_SCRIPTS_DIR
export VOLADM_ROOT
export VOLADM_BIN
export VOLADM_LIB
export VOLADM_HELP
export VOL_CONFIG_DIR

CNTRL_EXCLUDE_FILE=${VOL_CONFIG_DIR}/cntrls.exclude
DISK_EXCLUDE_FILE=${VOL_CONFIG_DIR}/disks.exclude
ENCLR_EXCLUDE_FILE=${VOL_CONFIG_DIR}/enclr.exclude

SYSDIRS=
[ -n "$VOLROOT_DIR" ] && {
	SYSDIRS=$VOLROOT_DIR/usr/sbin:$VOLROOT_DIR/sbin:
}
SYSDIRS=$SYSDIRS/usr/sbin:/sbin:/usr/bin:/usr/ucb
: ${__VXADM_NEED_PATH:=$VOL_SCRIPTS_DIR:$VOLADM_BIN:$SYSDIRS}
case $PATH in
$__VXADM_NEED_PATH|$__VXADM_NEED_PATH:*) :;;
*)	PATH=$__VXADM_NEED_PATH:$PATH;;
esac
export PATH

voladm_menu_stack=
cleanup_list=cleanup_tempfiles
newline='
'

v_opt=
verbose=
if [ "X$1" = X-v ]
then
	verbose=yes
	v_opt=-v
	shift
fi


#
# We have to be carful how we create tempory files and this can potential be
# a security risk
#
# NOTE: scripts that dot(.) this file (or vxcommon) have to use quit not exit!
#
vxmkdir()
{
	[ "$VXTMPDIR" = "" ] && {
		VXTMPDIR=${TMPDIR-/tmp}
		VXTMPDIR=$VXTMPDIR/vx.$RANDOM.$RANDOM.$RANDOM.$$
		(umask 077 && mkdir -p $VXTMPDIR >/dev/null 2>&1) || {
			# We could not create the tmp directory
			# revert to old method of assigning name
			VXTMPDIR=""
			VXTMPNUM=1
		}
	}
}

vxmktemp()
{
	vxmkdir
	if [ "$VXTMPDIR" = "" ]
	then
		# Could not create directory use old method of assigning name
		VXTMPFILE=${TMPDIR:-/tmp}/vx$VXTMPNUM.$$
		VXTMPNUM=`expr $VXTMPNUM + 1`
	else
		while true ; do
			VXTMPFILE=$VXTMPDIR/vx.$RANDOM.$RANDOM.$RANDOM.$$
			[ ! -f $VXTMPFILE ] && {
				#unique tempfile
				break
			}
		done
	fi
}

vxmktemp; tmpfile1=$VXTMPFILE
vxmktemp; tmpfile2=$VXTMPFILE
vxmktemp; tmpfile3=$VXTMPFILE
vxmktemp; tmpfile4=$VXTMPFILE
vxmktemp; _vd_tmpfile=$VXTMPFILE
vxmktemp; _cef_tmpfile=$VXTMPFILE
vxmktemp; _vdskd_rawmatch=$VXTMPFILE
vxmktemp; _vdskd_cntrlmatch=$VXTMPFILE
vxmktemp; _vdskd_diskmatch=$VXTMPFILE
vxmktemp; _vdskd_enclrmatch=$VXTMPFILE
vxmktemp; _disk_enc=$VXTMPFILE
vxmktemp; _site_enc=$VXTMPFILE
vxmktemp; _disk_list_site=$VXTMPFILE
vxmktemp; _plex_list_site=$VXTMPFILE
vxmktemp; _allsites_list=$VXTMPFILE

cleanup_tempfiles()
{
	if [ "$VXTMPDIR" != "" ] 
	then
		rm -f $VXTMPDIR/*
	else
		if [ "$VXTMPNUM" != "" ]
		then
			rm ${TMPDIR:-/tmp}/vx*.$$
			VXTMPNUM=""
		fi
	fi
}

clean_tempdir()
{
	if [ "$VXTMPDIR" != "" ] 
	then
		rm -rf $VXTMPDIR
		VXTMPDIR=""
	else
		if [ "$VXTMPNUM" != "" ]
		then
			rm ${TMPDIR:-/tmp}/vx*.$$
			VXTMPNUM=""
		fi
	fi
}

quit()
{
	trap "" INT HUP QUIT TERM
	for f in $cleanup_list
	do
		"$f"
	done
	clean_tempdir
	exit $1
}

trap "quit 100" INT HUP QUIT TERM

exec 4<&0 5>&2

# doit - execute a command, printing the command in verbose mode
doit()
{
	[ "$verbose" ] && $ECHO ! "$@" >&5
	"$@"
}

writemsg()
{
	$ECHO "";
	for _wm_line in "$@"
	do
		$ECHO "  $_wm_line"
	done
}

#
# usage: ewritemsg \
#	 [-a] [-M catalog_id] [-f file] \
#	 default_message [shell_parms]
#
# -a: append the list to 'file' instead of overwriting. This
#     flag only has affect when -f is also used.
# -M catalog_id: specify catalog id
# -f file: write the list to 'file' and display the 'file' 
#
ewritemsg()
{
	_ewm_append=no
	_ewm_cat=
	_ewm_file=
	_ewm_name=
	_ewm_sev=
	_ewm_cmd=
	_ewm_opt=

	OPTIND=1 # sh does not re-initialize OPTIND
	while getopts "aM:Nf:n:s:" c
	do
		case $c in
		a) _ewm_append=yes;;
		M) _ewm_cat=$OPTARG;;
		N) _ewm_opt="-N";;
		f) _ewm_file="$OPTARG";;
		n) _ewm_name="-n $OPTARG";;
		s) _ewm_sev="-s $OPTARG";;
		\?) $ECHO "ewritemsg: bad argument list"
		    return;;
		esac
	done
	shift `expr $OPTIND - 1`

	if [ -n "$_ewm_cat" ]
	then
		_ewm_msg="$1"
		shift 1
		if [ -n "$_ewm_sev" -o -n "$_ewm_name" ]
		then
			_ewm_cmd="ugettxt $_ewm_opt $_ewm_name $_ewm_sev"
		else
			_ewm_cmd="egettxt"
		fi
		_ewm_msg="`$_ewm_cmd \"$_ewm_msg\" \"$_ewm_cat\" \"$@\"`"
	else
		_ewm_msg="$*"
	fi

	if [ -z "$_ewm_file" ]
	then
		writemsg "$_ewm_msg"
	else
		if [ "$_ewm_append" = "no" ]
		then
			> $_ewm_file
		fi

		# Save stdout and redirect stdout to $_ewm_file.
		exec 9>&0 >> $_ewm_file
		writemsg "$_ewm_msg"
		# Restore stdout and close $_ewm_file.
		exec >&9 9>&-
	fi
}

read_input()
{
	$ECHO ""
	_once=
	if [ "X$1" = X-m ]
	then
		_once=yes
		shift
	fi
	_prompt="$1 "
	[ -z "$2" ] || _prompt="${_prompt}[$2,q,?] "
	[ -z "$3" ] || _prompt="${_prompt}(default: $3) "
	case $_prompt in
????????????????????????????????????????????????????????????????????*)
		if [ ! -z "$2" ] || [ ! -z "$3" ]
		then
			_prompt="$1$newline"
			[ -z "$2" ] || _prompt="${_prompt}[$2,q,?] "
			[ -z "$3" ] || _prompt="${_prompt}(default: $3) "
		fi
		;;
	esac
	while true
	do
		$ECHO "$_prompt\c"
		read input <&4
		case $input in
		"")	input=$3; return 0;;
		"!")	[ ${VOLADM_SHELL_ESCAPE:-ENABLE} = DISABLE ] &&
							return 0
			${SHELL:-sh} <&4
			$ECHO !
			[ ! -z "$_once" ] && return 1;;
		"!"*)	[ ${VOLADM_SHELL_ESCAPE:-ENABLE} = DISABLE ] &&
							return 0
			${SHELL:-sh} -c "`expr "$input" : '!\(.*\)'`" <&4
			$ECHO !
			[ ! -z "$_once" ] && return 1;;
		'?')	voladm_help; [ ! -z "$_once" ] && return 1;;
		'??')	voladm_help_help; [ ! -z "$_once" ] && return 1;;
		q|quit)	quit 101;;
		x)	quit 100;;
		*)	return 0;;
		esac
	done
}

pop_list()
{
	_list=$1
	eval "set -- \$$_list"
	shift
	eval "$_list=\$*"
}

first_item()
{
	$ECHO $1
}

last_item()
{
	_list=$1
	eval "set -- \$$_list"
	eval "$ECHO \${$#}"
}

list_item()
{
	_item=$1
	shift
	eval "$ECHO \${$_item}"
}


list_count()
{
	$ECHO $#
}

push_list()
{
	_list=$1
	shift
	eval "set -- \$* \$$_list"
	eval "$_list=\$*"
}

append_list()
{
	_list=$1
	shift
	eval "set -- \$$_list \$*"
	eval "$_list=\$*"
}

list_member()
{
	_want=$1
	shift
	for i in $@
	do
		[ "X$_want" = "X$i" ] && return 0
	done
	return 1
}

#
# Set 'out_list' to be the 'elem_num' element of every 'num_elem_in_group'
# elements in the list 'in_list'.
#
# NOTE: The numbering for 'elem_num' starts from 1.
#
# usage: get_list_group num_elem_in_group elem_num in_list out_list
#
get_list_group()
{
	if [ $# -lt 4 ]
	then
		ewritemsg -s ERROR -M vxvmshm:1442 "get_list_group: bad argument list"
		return
	fi

	_glg_num_elem_in_group=$1
	_glg_elem_num=$2
	_glg_in_list=$3
	_glg_out_list=$4

	if [ $_glg_num_elem_in_group -lt $_glg_elem_num ]
	then
		ewritemsg -s ERROR -M vxvmshm:1443 "get_list_group: num_elem_in_group not > than elem_num"
		return
	fi

	if [ $_glg_num_elem_in_group -eq 0 -o $_glg_elem_num -eq 0 ]
	then
		ewritemsg -s ERROR -M vxvmshm:1444 "get_list_group: num_elem_in_group or elem_num are 0"
		return
	fi

	eval $_glg_out_list=

	set -- `eval $ECHO "\\$$_glg_in_list"`
	while [ $# -gt 0 ]
	do
		eval $_glg_out_list=\`eval \
		\"$ECHO \${$_glg_out_list} \${$_glg_elem_num}\"\`
		shift $_glg_num_elem_in_group
	done
}

#
# Remove every 'elem_num' element from every 'num_elem_in_group' in 'list'.
#
# NOTE: The numbering for 'elem_num' starts from 1.
#
# usage: change_list_group num_elem_in_group elem_num list
#
remove_list_group()
{
	if [ $# -lt 3 ]
	then
		ewritemsg -s ERROR -M vxvmshm:1446 "remove_list_group: bad argument list"
		return
	fi

	_rlg_num_elem_in_group=$1
	_rlg_elem_num=$2
	_rlg_list=$3

	if [ $_rlg_num_elem_in_group -lt $_rlg_elem_num ]
	then
		ewritemsg -s ERROR -M vxvmshm:1447 "remove_list_group: num_elem_in_group not > than elem_num"
		return
	fi

	if [ $_rlg_num_elem_in_group -eq 0 -o $_rlg_elem_num -eq 0 ]
	then
		ewritemsg -s ERROR -M vxvmshm:1448 "remove_list_group: num_elem_in_group or elem_num are 0"
		return
	fi

	set -- `eval $ECHO "\\$$_rlg_list"`
	eval $_rlg_list=
	while [ $# -gt 0 ]
	do
		i=1
		while [ $i -le $_rlg_num_elem_in_group ]
		do
			if [ $i -ne $_rlg_elem_num ]
			then
				eval $_rlg_list=\"\$$_rlg_list $1\"
			fi
			inc i
			shift 1
		done
	done
}


#
# list_explode [-t fieldsep] listname var1 var2 ...
#
# Explode a list and store elements into the named variables.  If
# some variables could not be filled (because the named list didn't
# have enough members) return a non-zero status.
#
list_explode()
{
	_le_save=$IFS
	_le_ifs=$IFS
	[ "X$1" = X-t ] && {
		_le_ifs=$2
		shift 2
	}
	_le_list=$1
	shift
	_le_vars="$*"
	IFS="$_le_ifs"
	eval "set \$$_le_list"
	IFS="$_le_save"
	for _le_var in $_le_vars
	do
		[ $# -eq 0 ] && {
			return 1
		}
		eval "$_le_var=\$1"
		shift
	done
	return 0
}

not()
{
	"$@"
	[ $? -eq 0 ] && return 1
	return 0
}

add_cleanup()
{
	push_list cleanup_list $*
}

voladm_menu_push()
{
        if [ "$1" = "-M" ]
        then
                _mnu_def="$2"
                _mnu_cat="$3"
                shift 3
                _mnu_cur="`egettxt \"$_mnu_def\" \"$_mnu_cat\" \"$@\"`"
        else
                _mnu_cur="$*"
        fi

	if [ -z "$VOLADM_CURRENT_MENU" ]
	then
		VOLADM_CURRENT_MENU=$_mnu_cur
	elif [ ! -z "$_mnu_cur" ] 
	then
		VOLADM_CURRENT_MENU=$VOLADM_CURRENT_MENU/$_mnu_cur
	fi
	export VOLADM_CURRENT_MENU
	push_list voladm_menu_stack $VOLADM_CURRENT_MENU
}

voladm_menu_pop()
{
	pop_list voladm_menu_stack
	VOLADM_CURRENT_MENU=`first_item $voladm_menu_stack`
	export VOLADM_CURRENT_MENU
}

voladm_menu_set()
{
        if [ "$1" = "-M" ]
        then
                _mnu_def="$2"
                _mnu_cat="$3"
                shift 3
                _mnu_cur="`egettxt \"$_mnu_def\" \"$_mnu_cat\" \"$@\"`"
        else
                _mnu_cur="$*"
        fi

	voladm_menu_pop
	voladm_menu_push $_mnu_cur
}

voladm_help_push()
{
	VOLADM_CURRENT_HELP=$1
	export VOLADM_CURRENT_HELP
	voladm_help_stack="$VOLADM_CURRENT_HELP $voladm_help_stack"
}

voladm_help_pop()
{
	VOLADM_CURRENT_HELP=`first_item $voladm_help_stack`
	export VOLADM_CURRENT_HELP
	pop_list voladm_help_stack
}

voladm_help_set()
{
	voladm_help_pop
	voladm_help_push $1
}

voladm_help_help()
{
	voladm_help vxadm.info
}

tput_valid()
{
	[ -x /usr/bin/tput ] && [ ! -z "$TERM" ]
}

voladm_help()
{
	help=${1:-$VOLADM_CURRENT_HELP}
	_vx_menu="`egettxt 'Menu:' vxvmshm:883`"
	tput_valid && tput clear
	( $ECHO "$title\n$_vx_menu: $VOLADM_CURRENT_MENU\n"
	  grep -v "^#" $VOLADM_HELP/$help; $ECHO "" ) | voladm_display
}

#
# display a file, using a pager, if needed
#
# usage: voladm_display [-acn] [-f out_file] [in_file]
#
# -a: append the list to 'out_file' instead of overwriting. This
#     flag is only meaningful when -f is used.
# -c: call voladm_continue called after the display is finished if the
#     pager was not used
# -f out_file: write the list to 'out_file' and display 'out_file' 
# -n: add new line after file
#
# If 'in_file' is not specified, read from standard input.
#
voladm_display()
{
	_vd_out_file=
	_vd_append=no
	_vd_pause=no
	_vd_nflag=no

	OPTIND=1 # sh does not re-initialize OPTIND
	while getopts "acnf:" c
	do
		case $c in
		a) _vd_append=yes;;
		c) _vd_pause=yes;;
		n) _vd_nflag=yes;;
		f) _vd_out_file="$OPTARG";;
		\?) ewritemsg -s ERROR -M vxvmshm:1452 "voladm_display: bad argument list"
		    return;;
		esac
	done
	shift `expr $OPTIND - 1`

	if [ -z "$_vd_out_file" ]
	then
		_vd_out_file=$_vd_tmpfile
		cat "$@" > $_vd_out_file
		if [ $_vd_nflag = "yes" ]
		then
			$ECHO >> $_vd_out_file
		fi
	else
		if [ "$_vd_append" = "no" ]
		then
			cat "$@" > $_vd_out_file
		else
			cat "$@" >> $_vd_out_file
		fi
		if [ $_vd_nflag = "yes" ]
		then
			$ECHO >> $_vd_out_file
		fi
	fi

	tput_valid && _vd_lines=`tput lines`
	[ -z "$_vd_lines" ] && _vd_lines=24
	if [ "`wc -l < $_vd_out_file`" -ge $_vd_lines ]
	then
		${PAGER:-more} $_vd_out_file <&4
	else
		cat $_vd_out_file
		if [ "$_vd_pause" = "yes" ]
		then
			voladm_continue
		fi
	fi
}

#
# Send a list to the fmt command and then display the output with a
# pager, if needed. If the number of elements in a group is greater
# than one, then by default (unless '-e elem_num' is used) all elements
# in the list surrounded by brackets will be displayed.
#
# usage: voladm_list_display \
#	[-abcn] [-f file] \
#	[-g num_elem_in_group] [-d elem_num] \
#	list...
# -a: append the list to 'file' instead of overwriting. This
#     flag only has affect when -f is also used.
# -b: put brackets around elements in the list. if there is more than
#     one element to be displayed separate the elements with forward
#     slashes '/'
# -c: call voladm_continue called after the display is finished if the
#     pager was not used
# -f file: write the list to 'file' and display the 'file' 
# -g num_elem_in_group: specify the number of elements in a group (default: 1)
# -d elem_num: specify the group element to display (default: 1)
# -n: add new line after list
#
voladm_list_display()
{
	_vld_append=no
	_vld_pause=no
	_vld_file=
	_vld_num_elem_in_group=1
	_vld_elem_num=1
	_vld_dflag=no
	_vld_use_bracket=no 
	_vld_nflag=no

	OPTIND=1 # sh does not re-initialize OPTIND
	while getopts "abcf:g:d:n" c
	do
		case $c in
		a) _vld_append=yes;;
		b) _vld_use_bracket=yes;;
		c) _vld_pause=yes ;;
		f) _vld_file="$OPTARG";;
		g) _vld_num_elem_in_group="$OPTARG";;
		d) _vld_dflag=yes; _vld_elem_num="$OPTARG";;
		n) _vld_nflag=yes;;
		\?) ewritemsg -s ERROR -M vxvmshm:1455 "voladm_list_display: bad argument list"
		    return;;
		esac
	done
	shift `expr $OPTIND - 1`

	if [ $_vld_num_elem_in_group -eq 0 ]
	then
		ewritemsg -s ERROR -M vxvmshm:1457 "voladm_list_display: num_elem_in_group is 0"
		return
	fi

	if [ $_vld_dflag = "yes" -a $_vld_elem_num != "all" -a \
	     $_vld_elem_num = "0" ]
	then
		ewritemsg -s ERROR -M vxvmshm:1456 "voladm_list_display: num_elem is 0"
		return
	fi

	# If all is specified, then use the default handling.
	if [ $_vld_dflag = "yes" -a $_vld_elem_num = "all" ]
	then
		_vld_dflag=no
	fi

	if [ -z "$_vld_file" ]
	then
		_vld_file=$_vd_tmpfile
		> $_vld_file
	else
		if [ "$_vld_append" = "no" ]
		then
			> $_vld_file
		fi
	fi

	$ECHO >> $_vld_file
	while [ $# -gt 0 ]
	do
		# If '-d elem_num' was specified then display only the
		# element that was selected.
		if [ $_vld_dflag = "yes" ]
		then
			if [ $_vld_use_bracket = "yes" ]
			then
				eval "$ECHO \"  [\${$_vld_elem_num}]\""
			else
				eval "$ECHO \"  \${$_vld_elem_num}\""
			fi
			shift $_vld_num_elem_in_group
		else
			if [ $_vld_use_bracket = "yes" ]
			then
				$ECHO "  [\c"
			fi
			i=$_vld_num_elem_in_group
			_vld_first=yes
			while [ $i -gt 0 ]
			do
				if [ $_vld_use_bracket = "yes" ]
				then
					if [ $_vld_first = "yes" ]
					then
						$ECHO "$1\c"
						_vld_first=no
					else
						$ECHO ",$1\c"
					fi
				else
					$ECHO "  $1"
				fi
				dec i
				shift 1
			done
			if [ $_vld_use_bracket = "yes" ]
			then
				$ECHO "]"
			fi
		fi
	done | fmt >> $_vld_file
	if [ $_vld_nflag = "yes" ]
	then
		$ECHO >> $_vld_file
	fi
	tput_valid && _vld_lines=`tput lines`
	[ -z "$_vld_lines" ] && _vld_lines=24
	if [ "`wc -l < $_vld_file`" -ge $_vld_lines ]
	then
		${PAGER:-more} $_vld_file <&4
	else
		cat $_vld_file
		if [ "$_vld_pause" = "yes" ]
		then
			voladm_continue
		fi
	fi
}

voladm_call()
{
	_util=$1
	shift

	doit "$_util" $v_opt "$@" <&4
	status=$?
	if [ $status -eq 100 ] || [ $status -eq 101 ]
	then
		quit $status
	fi
	return $status
}

voladm_menu_call()
{
	_util=$1
	shift

	doit "$_util" $v_opt "$@" <&4
	status=$?
	if [ $status -eq 100 ]
	then
		quit 100
	fi
	return $status
}

voladm_begin_screen()
{
	if [ "$1" = "-M" ]
	then
		_vbs_cat="$2"
		_vbs_def="$3"
		shift 3
		_vbs_msg="`egettxt \"$_vbs_def\" \"$_vbs_cat\" \"$@\"`"
	else
		_vbs_msg="$1"
	fi
	tput_valid && tput clear
	_vx_menu="`egettxt 'Menu:' vxvmshm:883`"
	$ECHO "\n$_vbs_msg\n$_vx_menu: $VOLADM_CURRENT_MENU"
}

voladm_menu()
{
	_menu_h1="`egettxt \"Display help about menu\" vxvmshm:159`"
	_menu_h2="`egettxt \"Display help about the menuing system\" \
			vxvmshm:160`"
	_menu_h3="`egettxt \"Exit from menus\"  vxvmshm:190`"
	_menu_h4="`egettxt \"Select an operation to perform\"  vxvmshm:297`"
	_menu_h5="`egettxt \"default\"  vxvmshm:540`"

	read _menu_title
	_menu_default=
	_vx_menu="`egettxt 'Menu:' vxvmshm:883`"
	_menu="$_menu_title\n$_vx_menu: $VOLADM_CURRENT_MENU\n"
	_menu_select='case $input in'
	while read _menu_key _menu_item _menu_cat _menu_text
	do
		if [ -z "$_menu_key" ]
		then
			_menu="$_menu\n"
		elif [ "X$_menu_item" = Xdefault ]
		then
			_menu_default=$_menu_key
		else
			_menu_select="$_menu_select '$_menu_key')VOLADM_MENU_SELECT=$_menu_item;break 2;;"
			if [ ! -z "$_menu_cat" ]
			then
				_menu_ctext="`egettxt \"$_menu_text\" \"$_menu_cat\"`"
				_menu="$_menu\n $_menu_key\t$_menu_ctext"
			fi
		fi
	done
	_menu_select="$_menu_select *) :;; esac"
	_menu="$_menu\n ?\t$_menu_h1"
	_menu="$_menu\n ??\t$_menu_h2"
	_menu="\n$_menu\n q\t$_menu_h3"
	_menu_prompt="${1:-$_menu_h4:}"
	if [ ! -z "$_menu_default" ]
	then
		_menu_prompt="$_menu_prompt [$_menu_h5: $_menu_default]"
	fi

	while true
	do
		tput_valid && tput clear
		$ECHO "$_menu"
		while true
		do
			read_input -m "$_menu_prompt" "" $_menu_default
			if [ $? -ne 0 ]
			then
				voladm_continue
				continue 2
			fi
			eval "$_menu_select"

			case $input in
			x)	quit 100;;
			q|quit)	quit 101;;
			'?')	voladm_help; continue 2;;
			h)	voladm_help; continue 2;;
			help)	voladm_help; continue 2;;
			'??')	voladm_help_help; continue 2;;
			esac
			ewritemsg -s ERROR -M vxvmshm:302 "Selection not recognized, enter ?? for help"
		done
	done

	$ECHO ""
}

# voladm_yorn question default
voladm_yorn()
{
	if [ "$1" = "-M" ]
	then
		_vadm_cat="$2"
		_vadm_def="$3"
		_vadm_yon="$4"
		shift 4
		_vadm_msg="`egettxt \"$_vadm_def\" \"$_vadm_cat\" \"$@\"`"
	else
		_vadm_msg="$1"
		_vadm_yon="$2"
	fi
	while true
	do
		read_input "$_vadm_msg" "y,n" $_vadm_yon
		case $input in
		y|yes)	return 0;;
		n|no)	return 1;;
		*)	ewritemsg -s NOTICE -M vxvmshm:788 \
			"Please respond with \\\"y\\\" or \\\"n\\\""
			continue;;
		esac
	done
}

#
# Ask "yes" or "no" for an entire list. Each element of the list may be
# reviewed separately or the same answer may be given for the entire list.
# Separate prompts and defaults are specified for the list and for elements
# of the list.
#
#
# usage: voladm_yorn_batch \
#	[-bh] [-f file] [-M list_catalog_id] [-m elem_catalog_id] \
#	[-g num_elem_in_group] [-p elem_num] [-d elem_num] \
#	list_prompt list_yon \
#	elem_prompt elem_yon \
#	list yes_list no_list [shell_parms]
#
# -b: add brackets around list items
# -h: push and pop voladm_help menus
# -f file: append the question output to 'file' and display 'file'
# -M list_catalog_id: specify list catalog id
# -m elem_catalog_id: specify element catalog id
# -g num_elem_in_group: specify the number of elements in a group (default: 1)
# -p elem_num: specify the group element to prompt (default: 1)
# -d elem_num: specify the group element to display.
# list_prompt: specify list prompt string
# list_yon: specify default list answer
# elem_prompt: specify element prompt string
# list_yon: specify default element answer
# list: list of elements to ask questions about (input)
# yes_list: elements with yes answers (output)
# no_list: elements with no answers (output)
#
# If -d 'elem_num' is not used or if "all" is specified, then all elements
# will displayed.
#
voladm_yorn_batch()
{
	_vadm_use_bracket=no
	_vadm_help=no
	_vadm_file=
	_vadm_list_cat=
	_vadm_elem_cat=
	_vadm_num_elem_in_group=1
	_vadm_prompt_num=1
	_vadm_dflag=no
	_vadm_disp_num=1

	OPTIND=1 # sh does not re-initialize OPTIND
	while getopts "bhf:M:m:g:p:d:" c
	do
		case $c in
		b) _vadm_use_bracket=yes;;
		h) _vadm_help=yes;;
		f) _vadm_file="$OPTARG";;
		M) _vadm_list_cat="$OPTARG";;
		m) _vadm_elem_cat="$OPTARG";;
		g) _vadm_num_elem_in_group="$OPTARG";;
		p) _vadm_prompt_num="$OPTARG";;
		d) _vadm_dflag=yes; _vadm_disp_num="$OPTARG";;
		\?) ewritemsg -s ERROR -M vxvmshm:1460 "voladm_yorn_batch: bad argument list"
		    return ;;
		esac
	done
	shift `expr $OPTIND - 1`

	if [ $# -lt 7 ]
	then
		ewritemsg -s ERROR -M vxvmshm:1460 "voladm_yorn_batch: bad argument list"
		return
	fi

	_vadm_list_msg="$1"
	_vadm_list_yon="$2"
	_vadm_elem_msg="$3"
	_vadm_elem_yon="$4"
	_vadm_select_list="$5"
	_vadm_yes_list="$6"
	_vadm_no_list="$7"
	shift 7

	if [ $_vadm_num_elem_in_group -eq 0 -o $_vadm_prompt_num -eq 0 ]
	then
		ewritemsg -s ERROR -M vxvmshm:1459 "voladm_yorn_batch: -g or -p arguments can not be 0"
		return
	fi

	if [ $_vadm_dflag = "yes" -a $_vadm_disp_num != "all" -a \
	     $_vadm_disp_num = "0" ]
	then
		ewritemsg -s ERROR -M vxvmshm:1458 "voladm_yorn_batch: -d argument can not be zero."
		return
	fi

	eval $_vadm_yes_list=
	eval $_vadm_no_list=

	# If a catalog entry was specified for the list message, get it.
	if [ -n "$_vadm_list_cat" ]
	then
		_vadm_list_msg="`\
			egettxt \"$_vadm_list_msg\" \"$_vadm_list_cat\" \"$@\"`"
	fi

	# If a catalog entry was specified for the element message, get it.
	if [ -n "$_vadm_elem_cat" ]
	then
		_vadm_elem_msg="`\
			egettxt \"$_vadm_elem_msg\" \"$_vadm_elem_cat\" \"$@\"`"
	fi

	_vadm_vld_darg=
	if [ $_vadm_dflag = "yes" ]
	then
		_vadm_vld_darg="-d $_vadm_disp_num"
	fi

	_vadm_vld_barg=
	if [ $_vadm_use_bracket = "yes" ]
	then
		_vadm_vld_barg="-b"
	fi

	if [ -z "$_vadm_file" ]
	then
		voladm_list_display \
		-g $_vadm_num_elem_in_group $_vadm_vld_darg \
		$_vadm_vld_barg `eval "$ECHO \\$$_vadm_select_list"`
	else
		voladm_list_display -a -f $_vadm_file $_vadm_vld_barg \
		-g $_vadm_num_elem_in_group $_vadm_vld_darg \
		`eval "$ECHO \\$$_vadm_select_list"`
	fi

	# If there is only one element on the list, call voladm_batch_single.
	set -- `eval "$ECHO \\$$_vadm_select_list"`
	if [ $# -eq $_vadm_num_elem_in_group ]
	then
		voladm_batch_single
		return
	fi

	[ "$_vadm_help" = "yes" ] && voladm_help_push yorn_batch_list.help

	while true
	do
		read_input "$_vadm_list_msg" "Y,N,S(elect)" $_vadm_list_yon
		case $input in
		Y|Yes)
			eval $_vadm_yes_list=\"\$$_vadm_select_list\"
			break;;
		N|No)
			eval $_vadm_no_list=\"\$$_vadm_select_list\";
			break;;
		S|Select|s|select)
			voladm_batch_select
			break;;
		r|redisplay)
			voladm_list_display \
			-g $_vadm_num_elem_in_group $_vadm_vld_darg \
			$_vadm_vld_barg `eval "$ECHO \\$$_vadm_select_list"`
			continue;;
		*)
			ewritemsg -s NOTICE -M vxvmshm:787 \
		   	"Please respond with \\\"Y\\\", \\\"N\\\", or \\\"S\\\""
			continue;;
		esac
	done

	[ "$_vadm_help" = "yes" ] && voladm_help_pop
}

# function called by voladm_yorn_batch
voladm_batch_single()
{
	[ "$_vadm_help" = "yes" ] && voladm_help_push yorn_batch_single.help
	while true
	do
		read_input "$_vadm_elem_msg" "y,n" $_vadm_elem_yon
		case $input in
		y|yes)
			eval $_vadm_yes_list=\"\$$_vadm_select_list\"
			break;;
		n|no)
			eval $_vadm_no_list=\"\$$_vadm_select_list\";
			break;;
		r|redisplay)
			voladm_list_display \
			-g $_vadm_num_elem_in_group $_vadm_vld_darg \
			$_vadm_vld_barg `eval "$ECHO \\$$_vadm_select_list"`
			continue;;
		*)
			ewritemsg -s NOTICE -M vxvmshm:788 \
			"Please respond with \\\"y\\\" or \\\"n\\\""
			continue;;
		esac
	done
	[ "$_vadm_help" = "yes" ] && voladm_help_pop
}

# function called by voladm_yorn_batch
voladm_batch_select()
{
	[ "$_vadm_help" = "yes" ] && voladm_help_push yorn_batch_elem.help
	set -- `eval $ECHO "\\$$_vadm_select_list"`
	while [ $# -gt 0 ]
	do
		if [ $_vadm_use_bracket = "yes" ]
		then
			ewritemsg `eval $ECHO "[\\${$_vadm_prompt_num}]"`
		else
			ewritemsg `eval $ECHO "\\${$_vadm_prompt_num}"`
		fi
		while true
		do
			read_input "$_vadm_elem_msg" "y,n,Y,N" $_vadm_elem_yon
			case $input in
			y|yes)
				i=$_vadm_num_elem_in_group
				while [ $i -gt 0 ]
				do
					eval \
					$_vadm_yes_list=\"\$$_vadm_yes_list $1\"
					dec i
					shift 1
				done
				break;;
			n|no)
				i=$_vadm_num_elem_in_group
				while [ $i -gt 0 ]
				do
					eval \
					$_vadm_no_list=\"\$$_vadm_no_list $1\"
					dec i
					shift 1
				done
				break;;
			Y|Yes)
				eval $_vadm_yes_list=\"\$$_vadm_yes_list $*\"
				break 2;;
			N|No)
				eval $_vadm_no_list=\"\$$_vadm_no_list $*\"
				break 2;;
			r|redisplay)
				if [ $_vadm_use_bracket = "yes" ]
				then
					ewritemsg \
					`eval $ECHO "[\\${$_vadm_prompt_num}]\"`
				else
					ewritemsg \
					`eval $ECHO "\\${$_vadm_prompt_num}\"`
				fi
				continue;;
			*)
				ewritemsg -s NOTICE -M vxvmshm:789 \
"Please respond with \\\"y\\\", \\\"n\\\", \\\"Y\\\", or \\\"N\\\""
				continue;;
			esac
		done
	done
	[ "$_vadm_help" = "yes" ] && voladm_help_pop
}

#
# usage: voladm_continue [-n] [prompt_string]
# -n: precede prompt with a newline
#
voladm_continue()
{
	if [ $# -ge 1 -a "$1" = "-n" ]
	then
		$ECHO "\n${2:-Hit RETURN to continue.}\c"
	else
		$ECHO "${1:-Hit RETURN to continue.}\c"
	fi
	read _continue <&4
	case $_continue in
	q|quit)	quit 101;;
	x)	quit 100;;
	*)	return 0;;
	esac
}

# Like voladm_continue, but no quit option.
voladm_do_continue()
{
	$ECHO "${1:-Hit RETURN to continue.}\c"
	read _continue <&4
	return 0
}

# KJP: deal with exclusion handling...

#
# check_exclude_files
#
# Check the disks.exclude and cntrls.exclude files for consistency.
# This function should be called before either of these files is
# used.
#
# If 1 is returned, the program should tell the user to fix the
# problems and then it should exit. If 0 is returned, the files
# were found to be non-existent or correct so processing can continue.
#
check_exclude_files()
{
	bad_disk_list=
	bad_cntrl_list=

	if [ -f $DISK_EXCLUDE_FILE -a -x $VOL_SCRIPTS_DIR/vxbaddxcld ]
	then
		bad_disk_list=`$VOL_SCRIPTS_DIR/vxbaddxcld $DISK_EXCLUDE_FILE`
	elif [ -f $DISK_EXCLUDE_FILE ]
	then
		for disk in `cat $DISK_EXCLUDE_FILE`
		do
			if not dogi_name_is_device $disk
			then
				append_list bad_disk_list $disk
			fi
		done
	fi

	if [ -f $CNTRL_EXCLUDE_FILE -a -x $VOL_SCRIPTS_DIR/vxbadcxcld ]
	then
		bad_cntrl_list=`$VOL_SCRIPTS_DIR/vxbadcxcld $CNTRL_EXCLUDE_FILE`
	elif [ -f $CNTRL_EXCLUDE_FILE ]
	then
		for cntrl in `cat $CNTRL_EXCLUDE_FILE`
		do
			if not dogi_name_is_cntrlr $cntrl
			then
				append_list bad_cntrl_list $cntrl
			fi
		done
	fi

	if [ -z "$bad_disk_list" -a -z "$bad_cntrl_list" ]
	then
		return 0	# No errors found.
	else 
		if [ -n "$bad_cntrl_list" ]
		then
			if [ `list_count $bad_cntrl_list` -eq 1 ]
			then
				export CNTRL_EXCLUDE_FILE bad_cntrl_list; \
				ewritemsg -N -s NOTICE -M vxvmshm:343 \
"The following controller address in the controller exclude file
  ${CNTRL_EXCLUDE_FILE} is invalid: $bad_cntrl_list"
			else
				export CNTRL_EXCLUDE_FILE; \
				ewritemsg -f $_cef_tmpfile -M vxvmshm:344 \
"The following controller addresses in the controller exclude file
  ${CNTRL_EXCLUDE_FILE} are invalid:"
				voladm_list_display -acn \
				-f $_cef_tmpfile $bad_cntrl_list
			fi
		fi

		if [ -n "$bad_disk_list" ]
		then
			if [ `list_count $bad_disk_list` -eq 1 ]
			then
				export CNTRL_EXCLUDE_FILE bad_disk_list; \
				ewritemsg -N -s NOTICE -M vxvmshm:351 \
"The following disk address in the disk exclude file
  ${DISK_EXCLUDE_FILE} is invalid: $bad_disk_list"
			else
				export CNTRL_EXCLUDE_FILE; \
				ewritemsg -f $_cef_tmpfile -M vxvmshm:352 \
"The following disk addresses in the disk exclude file
  ${DISK_EXCLUDE_FILE} are invalid:"
				voladm_list_display -acn \
				-f $_cef_tmpfile $bad_disk_list
			fi
		fi
		return 1	# Errors found.
	fi
}

#
# Read a single disk address from the standard input and set its
# address in 'device'. Honor the disks.exclude and cntrls.exclude
# files.
# 
# usage: voladm_disk_device \
#	 [-M catalog_id]
#	 prompt-string [default-result [select-list]]
#
# -M catalog_id: specify catalog id
#
voladm_disk_device()
{
	if [ "$1" = "-M" ]
	then
		_vdskd_msg="`egettxt \"$3\" \"$2\"`"
		shift 2 # Leave message as $1.
	else
		_vdskd_msg="$1"
	fi

	_vdskd_def=
	_vdskd_sel=

	if [ $# -ge 2 ]
	then
		_vdskd_def="$2"
		if [ $# -ge 3 ]
		then
			_vdskd_sel="$3"
		fi
	fi

	while true
	do
		#
		# If the exclude files exist they should have already been
		# verified with the check_exclude_files() function.
		#
		# Create egrep patterns from the files that ensure that only
		# the disk or controller specified is excluded.
		#
		# Build the exclude egrep patterns inside the input while
		# loop in case the files are edited after an exclusion is
		# reported.
		# 
		cntrl_egrep=
		disk_egrep=
		enclr_egrep=

		if [ -f $CNTRL_EXCLUDE_FILE -a -x $VOL_SCRIPTS_DIR/vxcxcld ]
		then
			cntrl_egrep=`$VOL_SCRIPTS_DIR/vxcxcld $CNTRL_EXCLUDE_FILE`
		elif [ -f $CNTRL_EXCLUDE_FILE ]
		then
			for cntrl in `cat $CNTRL_EXCLUDE_FILE`
			do
				add_cntrlr_expression $cntrl cntrl_egrep
			done
		fi

		if [ -f $DISK_EXCLUDE_FILE ]
		then
			disk_egrep=`cat $DISK_EXCLUDE_FILE`
		fi

		if [ -f $ENCLR_EXCLUDE_FILE ]
		then
			for enclr in `cat $ENCLR_EXCLUDE_FILE`
			do
				add_cntrlr_expression $enclr cntrl_egrep
			done
		fi	
		#read_input "$_vdskd_msg" "${_vdskd_sel:-<address>,list}" \
		read_input "$_vdskd_msg" "${_vdskd_sel:-<disk>,list}" \
			$_vdskd_def
		
		device=$input

		#
		# If the default value that was specified to read_input
		# was returned, then return now after having set 'device'.
		#
		if [ -n "$_vdskd_def" -a "$device" = "$_vdskd_def" ]
		then
			return 0
		fi

		if [ "X$device" = "X" ] ; then 
			ewritemsg -s ERROR -M vxvmshm:206 \
			"Input not recognized, enter ?? for help"
			continue
		fi

		if vxcheckda $device > /dev/null 2> /dev/null
		then
			if dogi_name_is_slice $device
			then
				daname=$device
			else
				dogi_name_daname $device daname
			fi
			
			if [ -n "$cntrl_egrep" ]
			then
				get_controller_of_disk $dm_ctlr_name | egrep \
					-q -e "$cntrl_egrep"
				if [ $? -eq 0 ]
				then
					export CNTRL_EXCLUDE_FILE; \
					ewritemsg -N -s INFO -M vxvmshm:429 \
"This disk that you specified has been excluded by the
  $CNTRL_EXCLUDE_FILE file: $device"
					device=
					daname=
					continue
				fi
			fi

			if [ -n "$disk_egrep" ]
			then
				# The string "${VOL_FULL_SLICE}\$" must be
				# appended to each disk device string in
				# the egrep pattern list to ensure that
				# only the specified disk(s) is filtered out.
				# Moreover, the logical-or character must be
				# introduced for this scheme to work.
				disk_egrep=`echo $disk_egrep | \
					sed "s/ /${VOL_FULL_SLICE}\$\|/g" | \
					sed "s/$/${VOL_FULL_SLICE}\$/"`

				$ECHO $daname | egrep -q -e "$disk_egrep"
				if [ $? -eq 0 ]
				then
					export DISK_EXCLUDE_FILE; \
					ewritemsg -N -s INFO -M vxvmshm:430 \
"This disk that you specified has been excluded by the
  $DISK_EXCLUDE_FILE file: $device"
					device=
					daname=
					continue
				fi
			fi

			return 0

		else
			case $device in
			list|l)	($ECHO ""; vxdevlist ) | voladm_display
				continue;;
			*) ewritemsg -s ERROR -M vxvmshm:206 \
			   "Input not recognized, enter ?? for help";;
	
			esac
		fi
	done
}

#
# Read or take as an argument a disk address pattern list and
# return a list of matching disks in 'device_list'. Honor the
# disks.exclude and cntrls.exclude files.
#
# The following disk address specifications are acceptable:
#
# all:          all disks
#
# usage: voladm_disk_device_list \
#	 [-M catalog_id] [-i input-string] \
#	 prompt-string [default-result [select-list]]
#
# -M catalog_id: specify catalog id
# -i input-string: don't call read_input, use input-string as input.
#
#  DEVFS support?
#
voladm_disk_device_list()
{
	_vdskd_iflag=no
	_vdskd_input=
	_vdskd_cat=

	OPTIND=1 # sh does not re-initialize OPTIND
	while getopts "i:M:" c
	do
		case $c in
		M) _vdskd_cat=$OPTARG;;
		i) _vdskd_iflag=yes; _vdskd_input="$OPTARG";;
		\?) ewritemsg -s ERROR -M vxvmshm:1802 "voladm_disk_device_list: bad argument list"
		    return 1;;
		esac
	done
	shift `expr $OPTIND - 1`

	if [ -n "$_vdskd_cat" ]
	then
		_vdskd_msg="`egettxt \"$1\" \"$_vdskd_cat\"`"
	else
		_vdskd_msg="$1"
	fi

	_vdskd_def=
	_vdskd_sel=

	if [ $# -ge 2 ]
	then
		_vdskd_def="$2"
		if [ $# -ge 3 ]
		then
			_vdskd_sel="$3"
		fi
	fi

	while true
	do
		#
		# If the exclude files exist they should have already been
		# verified with the check_exclude_files() function.
		#
		# Create egrep patterns from the files that ensure that only
		# the disk or controller specified is excluded.
		#
		# Build the exclude egrep patterns inside the input while
		# loop in case the files are edited after an exclusion is
		# reported.
		# 
		cntrl_egrep=
		disk_egrep=
		enclr_egrep=
		if [ -f $CNTRL_EXCLUDE_FILE -a -x $VOL_SCRIPTS_DIR/vxcxcld ]
		then
			cntrl_egrep=`$VOL_SCRIPTS_DIR/vxcxcld $CNTRL_EXCLUDE_FILE`
		elif [ -f $CNTRL_EXCLUDE_FILE ]
		then
			for cntrl in `cat $CNTRL_EXCLUDE_FILE`
			do
				add_cntrlr_expression $cntrl cntrl_egrep
			done
		fi

		if [ -f $DISK_EXCLUDE_FILE ]
		then
			disk_egrep=`cat $DISK_EXCLUDE_FILE`
		fi

		if [ -f $ENCLR_EXCLUDE_FILE ]
		then
			for enclr in `cat $ENCLR_EXCLUDE_FILE`
			do
				add_enclr_expression $enclr enclr_egrep
			done
		fi

		if [ "$_vdskd_iflag" = "yes" ]
		then
			if [ -n "$_vdskd_input" ]
			then
				input="$_vdskd_input"
				_vdskd_input=
			else
				# Return after one iteration.
				return 0
			fi
		else
			read_input "$_vdskd_msg" \
			"${_vdskd_sel:-<pattern-list>,list}" $_vdskd_def

			#
			# If the default value that was specified to
			# read_input was returned, then set device_list
			# and return.
			#
			if [ -n "$_vdskd_def" -a "$input" = "$_vdskd_def" ]
			then
				device_list="$_vdskd_def"
				return 0
			fi

			if [ -z "$input" ]
			then
				ewritemsg -s ERROR -M vxvmshm:206 \
				"Input not recognized, enter ?? for help"
				continue
			fi
		fi

		list=
		first_pass=yes

		cntrl_exclude_list=
		disk_exclude_list=
		enclr_exclude_list=

		# Process each disk address or pattern entered.
		set -- $input
		while [ $# -gt 0 ]
		do
			spec=$1
			pattern=

			if [ "$spec" = "all" ]
			then
				if [ "$first_pass" = "no" -o $# -gt 1 ]
				then
					ewritemsg -s ERROR -M vxvmshm:63 \
"'all' should appear alone, enter ?? for help"
					continue 2
				fi
			fi

			if [ "$spec" = "list"  -o "$spec" = "l" ]
			then
				if [ "$first_pass" = "no" -o $# -gt 1 ]
				then
					ewritemsg -s INFO -M vxvmshm:64 \
"'list' should appear alone, enter ?? for help"
				else
					($ECHO ""; vxdevlist) | voladm_display
				fi
				continue 2;
			fi
			
			expand_device_wildcard list \
			 "$disk_egrep" "$cntrl_egrep" "$enclr_egrep" \
			 disk_exclude_list cntrl_exclude_list \
			 enclr_exclude_list $spec

			if [ $? -ne 0 ]
			then
				if [ "$_vdskd_iflag" = "yes" ]
				then
					ewritemsg -s ERROR -M vxvmshm:766 \
"Input not recognized: \"$spec\""
					return 1
				fi
				continue 2
			fi
			
			first_pass=no
			shift
		done

		#
		# Shouldn't reach here unless ready to return. Sort the
		# list and remove any duplicates, then return.
		#
# KJP: exclusion lists need to be dealt with.
		device_list=`for d in $list; do $ECHO $d; done | sort | uniq`

		if [ -n "$cntrl_exclude_list" -o -n "$disk_exclude_list" \
			-o -n "$enclr_exclude_list" ]
		then
			cntrl_exclude_list=`
			for c in $cntrl_exclude_list; do $ECHO $c; done |
			sed "s%/dev/rdsk/\(.*\)$VOL_FULL_SLICE$%\1%"   |
			sed "s%/dev/vx/rdmp/\(.*\)$VOL_FULL_SLICE$%\1%"   |
			sed "s%/dev/ap/rdsk/\(.*\)$VOL_FULL_SLICE$%\1%"` 

			disk_exclude_list=`
			for d in $disk_exclude_list; do $ECHO $d; done |
			sed "s%/dev/rdsk/\(.*\)$VOL_FULL_SLICE$%\1%"  |
			sed "s%/dev/vx/rdmp/\(.*\)$VOL_FULL_SLICE$%\1%"  |
			sed "s%/dev/ap/rdsk/\(.*\)$VOL_FULL_SLICE$%\1%"`

			enclr_exclude_list=`
			for d in $enclr_exclude_list; do echo $d; done |
			sed "s%/dev/vx/rdmp/\(.*\)$VOL_FULL_SLICE$%\1%"  |
			sed "s%/dev/ap/rdmp/\(.*\)$VOL_FULL_SLICE$%\1%"`
			disp_file=$_vdskd_rawmatch

			if [ -n "$cntrl_exclude_list" ]
			then
				if [ `list_count $cntrl_exclude_list` -eq 1 ]
				then
					export \
					cntrl_exclude_list CNTRL_EXCLUDE_FILE; \
					ewritemsg -N -s INFO -M vxvmshm:428 \
"This disk that you specified has been excluded by the
  $CNTRL_EXCLUDE_FILE file: $cntrl_exclude_list"
					voladm_continue -n
				else
					export CNTRL_EXCLUDE_FILE; \
					ewritemsg -f $disp_file -M vxvmshm:415 \
"These disks that you specified have been excluded by the
  $CNTRL_EXCLUDE_FILE file:"
				voladm_list_display -anc \
				-f $disp_file $cntrl_exclude_list
				fi
			fi

			if [ -n "$disk_exclude_list" ]
			then
				if [ `list_count $disk_exclude_list` -eq 1 ]
				then
					export \
					disk_exclude_list DISK_EXCLUDE_FILE; \
					ewritemsg -N -s INFO -M vxvmshm:431 \
"This disk that you specified has been excluded by the
  $DISK_EXCLUDE_FILE file: $disk_exclude_list"
					voladm_continue -n
				else
					export DISK_EXCLUDE_FILE; \
					ewritemsg -f $disp_file -M vxvmshm:416 \
"These disks that you specified have been excluded by the
  $DISK_EXCLUDE_FILE file:"
					voladm_list_display -anc \
					-f $disp_file $disk_exclude_list
				fi
			fi
			
			if [ -n "$enclr_exclude_list" ]
			then
				if [ `list_count $enclr_exclude_list` -eq 1 ]
				then
					ewritemsg -N -s WARNING -M vxvmshm:2313 \
"These disks that you specified have been excluded by the
$ENCLR_EXCLUDE_FILE file: $enclr_exclude_list"
					voladm_continue -n
				else
					export ENCLR_EXCLUDE_FILE; \
					ewritemsg -f $disp_file -M vxvmshm:2312 \
"These disks that you specified have been excluded by the
$ENCLR_EXCLUDE_FILE file: "
					voladm_list_display -anc \
					-f $disp_file $enclr_exclude_list
				fi
			fi


			if [ -z "$device_list" ]
			then
				continue
			fi
		fi
		return 0
	done
}

# voladm_disk_group purpose default
#
# Sets the following "global" variables:
#	dgname - the name of the DG (or possibly "none" if 3rd param specified)
#	dg_is_cds - indicates if the DG is a CDS DG or not ('y' or 'n')
#			if the DG exists (otherwise it is set to 'n')
#
voladm_disk_group()
{
	dg_is_cds=n

	_new_group_okay=
	if [ X$1 = X-n ]
	then
		_new_group_okay=yes
		shift
	fi
	_opt="<group>,list"
	[ $# -eq 2 ] && _opt="<group>,none,list"
	_default=`vxdg defaultdg 2>/dev/null`
	if [ "$_default" = "nodg" -a "$_param_num" = "3" ]
	then
		_default="none"
	fi

	_prompt=$1
	while true
	do
		_riput=`egettxt "Which disk group" vxvmshm:509`
		if [ "$_default" = "nodg" ]; then
			read_input "$_riput" "$_opt"
		else
			read_input "$_riput" "$_opt" "$_default"
		fi
		dgname=$input
		dgname_input=$input
		case $dgname in
		$RESERVED_DG_DEFAULT)
			dgname=`vxdg defaultdg 2>/dev/null`
			;;
		$RESERVED_DG_BOOT)
			dgname=`vxdg bootdg 2>/dev/null`
			;;
		esac

		case $dgname in
		"")	ewritemsg -s ERROR -M vxvmshm:70 "A disk group name is required."
			continue;;
		list|l)	($ECHO ""; vxdg list ) | voladm_display
			continue;;
		*" "* | *"	"*)
			export dgname; ewritemsg -s ERROR -M vxvmshm:156 "Disk group name $dgname is not valid."
			continue;;
		$RESERVED_DG_NONE)
			if [ "$dgname_input" = "bootdg" -o $dgname_input = "defaultdg" ]
			then
				ewritemsg -s ERROR -M vxvmshm:3462 "$dgname_input is a reserved disk group name. Currently, it is not set."
			else 
				ewritemsg -s ERROR -M vxvmshm:1483 "$RESERVED_DG_NONE is not a valid disk group name."
			fi
			continue;;
		none)	[ $# -eq 2 ] && return 0;;
		esac
		_voldg_output=`vxdg -q list | grep "^\$dgname[ 	]"`
		if [ `list_count $_voldg_output` -ne 3 ]
		then
			if [ -n "$_new_group_okay" ]
			then
				voladm_yorn -M vxvmshm:119 "Create a new group named $dgname?" y
				if [ $? -eq 0 ]
				then
					return 10
				fi
			else
				export dgname; ewritemsg -s ERROR -n vxadm_lib -M vxvmshm:399 "There is no active disk group named $dgname."

			fi
			continue
		fi
		list_item 2 $_voldg_output | grep enabled >/dev/null
		if [ $? -ne 0 ]
		then
			export dgname; ewritemsg -s NOTICE -M vxvmshm:154 \
		 "Disk group $dgname is not currently usable due to errors."
			continue
		fi
		list_item 2 $_voldg_output | grep cds >/dev/null
		[ $? -eq 0 ] && dg_is_cds=y

		return 0
	done
}

# setup system wide default disk group
voladm_set_defaultdg()
{
	_opt="<group>,list"

	while true
	do
		_riput=`egettxt "Which disk group" vxvmshm:509`
		read_input "$_riput" "$_opt"
		dgname=$input

		case $dgname in
		"")	ewritemsg -s ERROR -M vxvmshm:70 "A disk group name is required."
			continue;;
		list|l)	(echo ""; vxdg list ) | voladm_display
			continue;;
		*" "* | *"	"*)
			export dgname; ewritemsg -s ERROR -M vxvmshm:156 "Disk group name $dgname is not valid."
			continue;;
		$RESERVED_DG_BOOT)
			dgname=`vxdg bootdg 2>/dev/null`
			;;
		q)	return 0;;
		esac

		vxdctl defaultdg $dgname 2>/dev/null
		if [ $? -ne 0 ]
		then 
			export dgname; ewritemsg -s ERROR -M vxvmshm:1940 "Failed to set up disk group named $dgname as the system wide default disk group."
			continue
		fi
		return 0
	done
}

voladm_new_disk_dmname()
{
	_default_dmname=`vxnewdmname "$1"`

	if [ "$2" ] ; then
		disk="$2"; export disk 
		_riput=`egettxt "Enter disk name for $disk" vxvmshm:184`
	else
		_riput=`egettxt "Enter disk name" vxvmshm:183`
	fi

	# gather all known names currently in use in the disk group
	while true ; do
		read_input "$_riput" "<name>" $_default_dmname
		if [ "X$input" != "X$_default_dmname" ] &&
		   not vxnewdmname -c "$input" "$1"
		then
			export input;  ewritemsg -s ERROR -M vxvmshm:546 \
		      "dmname $input already in use. Please enter a new one."
			_riput=`egettxt "Enter disk name" vxvmshm:183`
			continue
		else
                        input_len=`strlen $input`
                        if [ `expr $input_len \>= 31` = 1 ] ; then
                                ewritemsg -s ERROR -M vxvmshm:334 \
                "The disk name is too long. Please re-enter."
				_riput=`egettxt "Enter disk name" vxvmshm:183`
                                continue
                        fi
		fi
		break
	done

	dmname=$input
	return 0
}

#
# usage: voladm_new_disk_privlen [-e] [<disk>]
#	where -e indicates that the private region will be used as part of
#		 disk encapsulation (w/o -e disk initialization is assumed).
#
voladm_new_disk_privlen()
{
	if [ $# -ge 1 -a "$1" = "-e" ]
	then
		shift
		_kind="vxencap"
	else
		_kind="vxdisk"
	fi
	voladm_get_disk_layout_defaults $_kind "privlen" _default_privlen

	if [ $? -ne 0 -o -z "$_default_privlen" ]
	then
		_default_privlen=65536		# PRIVLEN_CONFIG
	fi
        _max_privlen=524288
        _min_privlen=512

	if [ $# -eq 1 ]
	then
		disk=$1
		export disk
		_riput=`egettxt "Enter private region length for $disk" vxvmshm:1344`
	else
		_riput=`egettxt "Enter desired private region length" vxvmshm:1513`
	fi
        while true ; do
                read_input "$_riput" "<privlen>" $_default_privlen
		input_len=`strlen $input`
		match_len=`expr $input : '[1-9][0-9]*'`
		if [ $input_len -ne $match_len ] ||
                	[ $input -lt $_min_privlen ] ||
                        [ $input -gt $_max_privlen ]
                then
                        export input;  ewritemsg -s ERROR -M vxvmshm:4500 \
                        "Private region length $input out of range. Please enter a length between 512 and 524288."
                        continue
                fi
                break
        done

        dmprivlen=$input;export dmprivlen

        return 0
}

#
# usage: voladm_get_disk_dmname \
#	 [-r] [-g disk-group] [-h on|off] \
#	 prompt default
# -r: restrict search to removed disks
# -n: restrict search to disks with enough free space
# -h on|off: restrict to disks with the hot-spare flag on or off
# -g disk-group: restrict search to disks in disk-group
# -a on|off: restrict to disks with the allocator-reserved flag on or off
#
voladm_get_disk_dmname()
{
	_dm_want_removed=
	_dm_want_spec_dg=
	_dm_want_hotspare=
	_dm_want_nohotuse=
	_dm_want_newdm=
	_dm_want_allocres=

	OPTIND=1 # sh does not re-initialize OPTIND
	while getopts "rg:h:n:e:a:" c
	do
		case $c in
		r) _dm_want_removed=yes;;
		g) _dm_want_spec_dg="$OPTARG";;
		h) _dm_want_hotspare="$OPTARG";; 
		e) _dm_want_nohotuse="$OPTARG";; 
		a) _dm_want_allocres="$OPTARG";; 
		n) _dm_want_newdm=yes
		   orig_len=$OPTARG;;
		\?) ewritemsg -s ERROR -M vxvmshm:1454 "voladm_get_disk_dmname: bad argument list"
		    return 1;;
		esac
	done
	shift `expr $OPTIND - 1`

	if [ -n "$_dm_want_allocres" -a \
	    "$_dm_want_allocres" != "on" -a "$_dm_want_allocres" != "off" ]
	then
		ewritemsg -s ERROR -M vxvmshm:2197 "voladm_get_disk_dmname: -a takes only 'on' or 'off'"
		return 1
	fi
	if [ -n "$_dm_want_hotspare" -a \
	    "$_dm_want_hotspare" != "on" -a "$_dm_want_hotspare" != "off" ]
	then
		ewritemsg -s ERROR -M vxvmshm:1453 "voladm_get_disk_dmname: -h takes only 'on' or 'off'"
		return
	fi

	_dm_prompt=$1
	[ X"$_dm_prompt" != "X" ] || \
		_dm_prompt=`egettxt "Enter disk name" vxvmshm:183`
	_default=$2

	while true
	do
		read_input "$_dm_prompt" "<disk>,list" $_default
		dmname=$input
		case $input in
		"")	ewritemsg -s NOTICE -M vxvmshm:252 \
	 "Please enter a disk name, or type \\\"list\\\" to list all disks."
			continue;;
		$_default)
			return 0;;
		list|l)	if [ "$_dm_want_removed" ]
			then
				($ECHO ""
				 if [ -z "$_dm_want_spec_dg" ]
				 then
				 	vxprint -Atd
				 else
				 	vxprint -td -g "$_dm_want_spec_dg"
				 fi |
					awk '$1!="dm" || $3=="-" {print $0;}'
				 $ECHO "") | voladm_display
			elif [ "$_dm_want_allocres" ]
			then
				_alloc_res=/tmp/alloc_res
				($ECHO ""
				 if [ -z "$_dm_want_spec_dg" ]
				 then
				 	LANG= vxdisk list
				 else
				 	LANG= vxdisk -g "$_dm_want_spec_dg" list
				 fi | awk '{
					found="off"
					for (i=6; i<=NF; i++)
						if ($i == "allocrsvd") {
							found="on"
						} 
					if ( $1 == "DEVICE" || (found == "'$_dm_want_allocres'" && \
						$3 != "-"))
						print $0
					}'
				) > $_alloc_res 
				lines=`cat $_alloc_res|wc -l`
				if [ $lines -eq 2 ] 
				then
					($ECHO "\nNo disk found with flag allocator_reserved=$_dm_want_allocres.") | voladm_display
				else
					($ECHO ""; cat $_alloc_res) | voladm_display
				fi
				rm -f $_alloc_res
			elif [ "$_dm_want_newdm" ]
			then
				dmlist=`vxprint -qd -g "$_dm_want_spec_dg" -F %name`
				for i in $dmlist
				do
					voladm_freespace_len $i $orig_len
					if [ $? -eq 0 ]
					then
						(echo ""
						vxdisk -qg $_dm_want_spec_dg list |\
						grep $i) | voladm_display
					fi
				done
			elif [ "$_dm_want_hotspare" ]
			then
				($ECHO ""
				 if [ -z "$_dm_want_spec_dg" ]
				 then
				 	LANG= vxdisk list
				 else
				 	LANG= vxdisk -g "$_dm_want_spec_dg" list
				 fi | awk '{
					found="off"
					foundnhu="off"
					for (i=6; i<=NF; i++)
						if ($i == "spare") {
							found="on"
						} else if ($i == "nohotuse") {
							foundnhu="on"
						}
					if ( $1 == "DEVICE" || (found == "'$_dm_want_hotspare'" && \
											foundnhu == "off" && $3 != "-" ))
						print $0
					}'
				) | voladm_display
			elif [ "$_dm_want_nohotuse" ]
			then
				(echo ""
				 if [ -z "$_dm_want_spec_dg" ]
				 then
				 	LANG= vxdisk list
				 else
				 	LANG= vxdisk -g "$_dm_want_spec_dg" list
				 fi | awk '{
					found="off"
					foundspare="off"
					for (i=6; i<=NF; i++)
						if ($i == "nohotuse") {
							found="on"
						} else if ($i == "spare") {
							foundspare="on"
						}
					if ( $1 == "DEVICE" || (found == "'$_dm_want_nohotuse'" && \
											foundspare == "off" && $3 != "-" ))
						print $0
					}'
				) | voladm_display
			else
				($ECHO ""; vxprint -Atd ) | voladm_display
			fi
			continue
			;;
		esac
		# XXX - The disks below should be filtered if _dm_want_removed
		# or _dm_want_hotspare have been specified.
		shift $# 
		if [ -z "$_dm_want_spec_dg" ]
		then
			set -- `vxprint -AQF "%dgname" "\$dmname" 2> /dev/null`
		else
			set -- `vxprint -g "$_dm_want_spec_dg" \
					-F "%dgname" "\$dmname" 2> /dev/null`
		fi
		if [ $# -eq 0 ] && [ -z "$_dm_want_spec_dg" ]
		then
			export dmname; ewritemsg -N -s ERROR -M vxvmshm:400 \
"There is no disk named $dmname in any disk group configuration.
  To get a list of disks enter \\\"list\\\"."
			continue
		elif [ $# -eq 0 ]
		then
			export dmname _dm_want_spec_dg; \
			ewritemsg -N -s ERROR -M vxvmshm:401 \
"There is no disk named $dmname in disk group $_dm_want_spec_dg.
  To get a list of disks enter \\\"list\\\"."
			continue
		fi
		break
	done
	if [ $# -eq 1 ]
	then
		dgname=$1
		return 0
	fi
	export dmname; ewritemsg -M vxvmshm:397 \
"There is a disk named $dmname in each of the following disk groups:
  	$*" "$@"
	while true
	do
		_riput=`egettxt "Enter the disk group to use" vxvmshm:187`
		read_input "$_riput" "group" $1
		for dgname in $*
		do
			if [ "X$dgname" = "X$input" ]
			then
				return 0
			fi
		done
		ewritemsg -M vxvmshm:248 \
			"Please choose a disk group from the list:\n\n\t$*" "$@"
	done
}

#
# usage: voladm_get_origin_dmname\
#	[-r] [-g disk-group] [-h on|off] \
#	prompt default
# -r: restrict search to removed disks
# -u: restrict search to disks which failed and the subdisks on it were
#     hot-relocated
# -h on|off: restrict to disks with the hot-spare flag on or off
# -g disk-group: restrict search to disks in disk-group
#
voladm_get_origin_dmname()
{
	_dm_prompt="Enter the original disk name"
	_default=$2

	while true
	do
		read_input "$_dm_prompt" "<disk>,list" $_default
		dmname=$input
		case $input in
		"")
			ewritemsg -M vxvmshm:939 \
"Please enter a disk name, or type \\\"list\\\" to list all hot-relocated disks."
			continue;;
		$_default)
			return 0;;
		list|l)
			(echo ""
			 vxprint -QAs -e 'sd_orig_dmname!=""' \
				  -F %orig_dmname | sort | uniq) | voladm_display
			continue
			;;
		esac

		shift $#

		vxprint -AQs -e'sd_orig_dmname == "'$dmname'"' \
			-F %dgname | sort | uniq > $tmpfile1
		set -- `cat $tmpfile1 2> /dev/null`

		if [ $# -eq 0 ]
		then
			export dmname; ewritemsg -N -s ERROR -M vxvmshm:935 \
"No subdisks appear to be hot-relocated from $dmname.
  To get a list of hot-relocated disks enter \\\"list\\\"."
			continue

		fi
		break
	done

	if [ $# -eq 1 ]
	then
		dgname=`cat $tmpfile1`
		return 0
	fi
	export dmname; ewritemsg -N -s NOTICE -M vxvmshm:2308 \
"There are subdisks which were relocated from $dmname in the following disk groups:
	$*" "$@"
	while true
	do
		_riput=`egettxt "Enter the disk group to use" vxvmshm:187`
		read_input "$_riput" "group" $1
		for dgname in $*
		do
			if [ "X$dgname" = "X$input" ]
			then
				return 0
			fi
		done
		ewritemsg -M vxvmshm:248 \
			"Please choose a disk group from the list:\n\n\t$*" "$@"
	done
}

# get free space length on a specified disk
voladm_freespace_len()
{
	total=0
	diskname=$1
	total=`vxdg -qa -g $dgname free $diskname | \
		awk '{sum += $5}; END {printf("%ld\n", sum)}'`

	if [ $total -ge $2 ]
	then
		return 0
	else
		return 1
	fi
}

# re-online all online disks
voladm_reonline_all()
{
	vxdisk -a online > /dev/null 2>&1
}

# get the system hostid from the volboot file in system_hostid
voladm_system_hostid()
{
	system_hostid=
	system_hostid=`LANG= vxdctl list | grep '^hostid:' |
			{ read key hostid; $ECHO "$hostid"; }`
	if [ -z "$system_hostid" ]
	then
		ewritemsg -s ERROR -M vxvmshm:442 \
	 "Unexpected internal error -- Cannot get the system's hostid."
		return 1
	fi
	return 0
}

strlen()
{
	$ECHO "$1" | awk '{ printf("%d",length($0)) }'
}

#
# table look-up increment for small positive integers
#
inc()
{
	if [ $# -ne 1 ]
	then
		$ECHO "usage: inc var-name"
		return
	fi

	_i_var=$1
	eval "_i_val=\$$_i_var"

	case $_i_val in
	0) _i_new_val=1;;
	1) _i_new_val=2;;
	2) _i_new_val=3;;
	3) _i_new_val=4;;
	4) _i_new_val=5;;
	5) _i_new_val=6;;
	6) _i_new_val=7;;
	7) _i_new_val=8;;
	8) _i_new_val=9;;
	9) _i_new_val=10;;
	10) _i_new_val=11;;
	11) _i_new_val=12;;
	12) _i_new_val=13;;
	13) _i_new_val=14;;
	14) _i_new_val=15;;
	15) _i_new_val=16;;
	16) _i_new_val=17;;
	17) _i_new_val=18;;
	18) _i_new_val=19;;
	19) _i_new_val=20;;
	20) _i_new_val=21;;
	21) _i_new_val=22;;
	22) _i_new_val=23;;
	23) _i_new_val=24;;
	24) _i_new_val=25;;
	25) _i_new_val=26;;
	26) _i_new_val=27;;
	27) _i_new_val=28;;
	28) _i_new_val=29;;
	29) _i_new_val=30;;
	30) _i_new_val=31;;
	31) _i_new_val=32;;
	32) _i_new_val=33;;
	33) _i_new_val=34;;
	34) _i_new_val=35;;
	35) _i_new_val=36;;
	36) _i_new_val=37;;
	37) _i_new_val=38;;
	38) _i_new_val=39;;
	39) _i_new_val=40;;
	40) _i_new_val=41;;
	41) _i_new_val=42;;
	42) _i_new_val=43;;
	43) _i_new_val=44;;
	44) _i_new_val=45;;
	45) _i_new_val=46;;
	46) _i_new_val=47;;
	47) _i_new_val=48;;
	48) _i_new_val=49;;
	49) _i_new_val=50;;
	50) _i_new_val=51;;
	51) _i_new_val=52;;
	52) _i_new_val=53;;
	53) _i_new_val=54;;
	54) _i_new_val=55;;
	55) _i_new_val=56;;
	56) _i_new_val=57;;
	57) _i_new_val=58;;
	58) _i_new_val=59;;
	59) _i_new_val=60;;
	60) _i_new_val=61;;
	61) _i_new_val=62;;
	62) _i_new_val=63;;
	63) _i_new_val=64;;
	64) _i_new_val=65;;
	65) _i_new_val=66;;
	66) _i_new_val=67;;
	67) _i_new_val=68;;
	68) _i_new_val=69;;
	69) _i_new_val=70;;
	70) _i_new_val=71;;
	71) _i_new_val=72;;
	72) _i_new_val=73;;
	73) _i_new_val=74;;
	74) _i_new_val=75;;
	75) _i_new_val=76;;
	76) _i_new_val=77;;
	77) _i_new_val=78;;
	78) _i_new_val=79;;
	79) _i_new_val=80;;
	80) _i_new_val=81;;
	81) _i_new_val=82;;
	82) _i_new_val=83;;
	83) _i_new_val=84;;
	84) _i_new_val=85;;
	85) _i_new_val=86;;
	86) _i_new_val=87;;
	87) _i_new_val=88;;
	88) _i_new_val=89;;
	89) _i_new_val=90;;
	90) _i_new_val=91;;
	91) _i_new_val=92;;
	92) _i_new_val=93;;
	93) _i_new_val=94;;
	94) _i_new_val=95;;
	95) _i_new_val=96;;
	96) _i_new_val=97;;
	97) _i_new_val=98;;
	98) _i_new_val=99;;
	99) _i_new_val=100;;
	*) _i_new_val=`expr $_i_val + 1`
	esac

	eval $_i_var=$_i_new_val
}

#
# table look-up decrement for small positive integers
#
dec()
{
	if [ $# -ne 1 ]
	then
		$ECHO "usage: dev var-name"
		return
	fi

	_d_var=$1
	eval "_d_val=\$$_d_var"

	case $_d_val in
	0) _d_new_val=-1;;
	1) _d_new_val=0;;
	2) _d_new_val=1;;
	3) _d_new_val=2;;
	4) _d_new_val=3;;
	5) _d_new_val=4;;
	6) _d_new_val=5;;
	7) _d_new_val=6;;
	8) _d_new_val=7;;
	9) _d_new_val=8;;
	10) _d_new_val=9;;
	11) _d_new_val=10;;
	12) _d_new_val=11;;
	13) _d_new_val=12;;
	14) _d_new_val=13;;
	15) _d_new_val=14;;
	16) _d_new_val=15;;
	17) _d_new_val=16;;
	18) _d_new_val=17;;
	19) _d_new_val=18;;
	20) _d_new_val=19;;
	21) _d_new_val=20;;
	22) _d_new_val=21;;
	23) _d_new_val=22;;
	24) _d_new_val=23;;
	25) _d_new_val=24;;
	26) _d_new_val=25;;
	27) _d_new_val=26;;
	28) _d_new_val=27;;
	29) _d_new_val=28;;
	30) _d_new_val=29;;
	31) _d_new_val=30;;
	32) _d_new_val=31;;
	33) _d_new_val=32;;
	34) _d_new_val=33;;
	35) _d_new_val=34;;
	36) _d_new_val=35;;
	37) _d_new_val=36;;
	38) _d_new_val=37;;
	39) _d_new_val=38;;
	40) _d_new_val=39;;
	41) _d_new_val=40;;
	42) _d_new_val=41;;
	43) _d_new_val=42;;
	44) _d_new_val=43;;
	45) _d_new_val=44;;
	46) _d_new_val=45;;
	47) _d_new_val=46;;
	48) _d_new_val=47;;
	49) _d_new_val=48;;
	50) _d_new_val=49;;
	51) _d_new_val=50;;
	52) _d_new_val=51;;
	53) _d_new_val=52;;
	54) _d_new_val=53;;
	55) _d_new_val=54;;
	56) _d_new_val=55;;
	57) _d_new_val=56;;
	58) _d_new_val=57;;
	59) _d_new_val=58;;
	60) _d_new_val=59;;
	61) _d_new_val=60;;
	62) _d_new_val=61;;
	63) _d_new_val=62;;
	64) _d_new_val=63;;
	65) _d_new_val=64;;
	66) _d_new_val=65;;
	67) _d_new_val=66;;
	68) _d_new_val=67;;
	69) _d_new_val=68;;
	70) _d_new_val=69;;
	71) _d_new_val=70;;
	72) _d_new_val=71;;
	73) _d_new_val=72;;
	74) _d_new_val=73;;
	75) _d_new_val=74;;
	76) _d_new_val=75;;
	77) _d_new_val=76;;
	78) _d_new_val=77;;
	79) _d_new_val=78;;
	80) _d_new_val=79;;
	81) _d_new_val=80;;
	82) _d_new_val=81;;
	83) _d_new_val=82;;
	84) _d_new_val=83;;
	85) _d_new_val=84;;
	86) _d_new_val=85;;
	87) _d_new_val=86;;
	88) _d_new_val=87;;
	89) _d_new_val=88;;
	90) _d_new_val=89;;
	91) _d_new_val=90;;
	92) _d_new_val=91;;
	93) _d_new_val=92;;
	94) _d_new_val=93;;
	95) _d_new_val=94;;
	96) _d_new_val=95;;
	97) _d_new_val=96;;
	98) _d_new_val=97;;
	99) _d_new_val=98;;
	*) _d_new_val=`expr $_d_val - 1`
	esac

	eval $_d_var=$_d_new_val
}

# contents of default vxvm.exclude and vxdmp.exclude files
default_exclude_file()
{
        echo "exclude_all 0"
        echo "paths"
        echo "#"
        echo "controllers"
        echo "#"
        echo "product"
        echo "#"
        echo "pathgroups"
        echo "#"
}


# create the default exclude files if they are not present
create_exclude_files()
{
        if [ ! -f $VOL_CONFIG_DIR/vxvm.exclude ]
        then
                default_exclude_file > $VOL_CONFIG_DIR/vxvm.exclude
        fi

        if [ ! -f $VOL_CONFIG_DIR/vxdmp.exclude ]
        then
                default_exclude_file > $VOL_CONFIG_DIR/vxdmp.exclude
        fi
}


# prints the input on a single line
print_line()
{
        count=0

        echo
        printf "\t"
        set -- $@
        while [ $# -gt 0 ]
        do
                printf "%-15s" `basename $1`
                shift 1
                count=`expr $count + 1`
                [ $count -eq 4 ] && { count=0; echo; printf "\t"; }
        done
        echo;echo
}

# returns a list of device names given a list of slice names. uses the function
# dogi_slice_to_device
slice_to_device()
{
        set -- $@

        while [ $# -gt 0 ]
        do
                dogi_slice_to_device $1 devicename
                echo $devicename
                shift 1
        done
}

# checks if any path under the given controller is the last enabled path to a
# disk containing a volume
check_ctlr_for_volumes()
{
        ctlr=$1
        pathsonctlr=`vxdmpadm getsubpaths ctlr=$ctlr | sed -n '3,$p' | \
                                                        awk '{ print $1; }'`
        volume_list=
        lastpath=0
        for path in $pathsonctlr
        do
                dmpnode=`vxdmpadm getdmpnode nodename=$path | sed -n '3,$p' | \
                                                        awk '{ print $1; }'`
                npaths_enabled=`vxdisk list $dmpnode | grep "state=enabled" | \
                                                        wc -l`
                path_state=`vxdisk list $dmpnode | grep "^$path" | \
                                                        awk '{ print $2; }'`
                if [ "$npaths_enabled" -eq 1 ] && \
                                        [ "$path_state" = "state=enabled" ]
                then
                        dmname=`vxdisk list | awk '$1 == "'$dmpnode'" && $3 != "-" { print $3; }'`
                        dgname=`vxdisk list | awk '$1 == "'$dmpnode'" && $4 != "-" { print $4; }'`
                        [ -z "$dmname" ] && continue
                        /etc/vx/bin/vxckdiskrm -g $dgname $dmname \
                                                        > $tmpfile1 2>/dev/null
                        if [ -s $tmpfile1 ]
                        then
                                lastpath=1
                                vol=`cat $tmpfile1 | awk '{print $1}'`
                                volume_list="$volume_list $vol"
                        fi
                fi
        done

        if [ $# -gt 1 ]
        then
                eval "${2}=\"$volume_list\""
        fi

        rm -f $tmpfile1

        [ "$lastpath" -eq 0 ]
}

check_ctlr_for_enclosures()
{
        ctlr=$1
        enclr_list=
        enclrs_on_ctlr=`vxdmpadm listctlr ctlr=$ctlr | sed -n '3,$p' | \
                                                awk '{ print $4; }'`
        lastctlr=0
        for enclr in $enclrs_on_ctlr
        do
                nctlrs_enabled=`vxdmpadm listctlr enclosure=$enclr | \
                                        sed -n '3,$p' | grep "ENABLED" | wc -l`

                ctlr_state=`vxdmpadm listctlr ctlr=$ctlr enclosure=$enclr | \
                                        sed -n '3,$p' | awk '{ print $3; }'`
                if [ "$nctlrs_enabled" -eq 1 ] && \
                                        [ "$ctlr_state" = "ENABLED" ]
                then
                        lastctlr=1
                        enclr_list="$enclr_list $enclr"
                fi
        done

        if [ $# -gt 1 ]
        then
                eval "${2}=\"$enclr_list\""
        fi

        [ "$lastctlr" -eq 0 ]
}

# checks if the given path is the last enabled path to a disk containing
# volume(s)
check_path_for_volumes()
{
        path=$1
        lastpath=0
        dmpnode=`vxdmpadm getdmpnode nodename=$path | sed -n '3,$p' | \
                                                        awk '{ print $1; }'`
        npaths_enabled=`vxdisk list $dmpnode | grep "state=enabled" | wc -l`
        path_state=`vxdisk list $dmpnode | grep "^$path" | awk '{ print $2; }'`

        if [ "$npaths_enabled" -eq 1 ] && [ "$path_state" = "state=enabled" ]
        then
                dmname=`vxdisk list | awk '$1 == "'$dmpnode'" && $3 != "-" { print $3; }'`
                dgname=`vxdisk list | awk '$1 == "'$dmpnode'" && $4 != "-" { print $4; }'`

                [ -z "$dmname" ] && return -1
                /etc/vx/bin/vxckdiskrm -g $dgname $dmname >$tmpfile1 2>/dev/null
                if [ -s $tmpfile1 ]
                then
                        lastpath=1
                        if [ $# -gt 1 ]
                        then
                                eval "${2}=`cat $tmpfile1 | awk '{print $1}'`"
                        fi
                fi
        fi

        rm -f $tmpfile1

        [ "$lastpath" -eq 0 ]
}

# this is the message given after any include/exclude operation that needs
# a reboot to take effect
reboot_message()
{
        ewritemsg -s NOTICE -M vxvmshm:1241 \
"This operation will take effect only after a reboot."
        voladm_continue -n
}

# this function lists all the paths seen by DMP. It uses the vxdmpadm
# command to gather the list of all paths.
listpaths()
{
        allpaths=
        for enclr in `vxdmpadm listenclosure all | sed -n '3,$p' |\
                        grep -v "DISCONNECTED" | awk '{print $1;}'`
        do
                for dmpnode in `vxdmpadm getdmpnode enclosure=$enclr |\
                        grep -v "NONAME" | sed -n '3,$p' | awk '{ print $1; }'`
                do
                        paths=`vxdmpadm getsubpaths dmpnodename=$dmpnode |\
                        grep -v "NONAME" | sed -n '3,$p' | awk '{ print $1; }'`
                        allpaths="$allpaths $paths"
                done
        done
        echo $allpaths
}



# this function lists all the paths seen by DMP. It uses the vxdmpadm
# command to gather the list of all paths. the output of this function
# will be in the form <(s|h)dX  dmpnodename#>
list_dmpnode_paths()
{
        allpaths=
      if [ "$1" = "exclude_pp" ]
      then
              enclr_list=`vxdmpadm listenclosure all | sed -n '3,$p' | grep -v "PP_" |\
                              grep -v "DISCONNECTED" | awk '{print $1;}'`
      else
              enclr_list=`vxdmpadm listenclosure all | sed -n '3,$p' |\
                        grep -v "DISCONNECTED" | awk '{print $1;}'`
      fi
      for enclr in $enclr_list
        do
                for dmpnode in `vxdmpadm getdmpnode enclosure=$enclr |\
                                grep -v "NONAME" | sed -n '3,$p' | \
                                awk '{ print $1; }'`
                do
                        dmpnodedev=`slice_to_device $dmpnode`
                        for path in \
                                `vxdmpadm getsubpaths dmpnodename=$dmpnode | \
                                grep -v "NONAME" | sed -n '3,$p' | awk '{print $1}'`
                        do
                                path=`slice_to_device $path`
                                allpaths="$allpaths $path $dmpnodedev"
                        done
                done
        done

        set -- $allpaths
        printf "%-10s %s\n" "PATH" "(DMPNODE)"
        printf "%-10s %s\n" "----" "---------"

        while [ $# -gt 1 ]
        do
                printf "%-10s %s\n" $1 $2
                shift 2
        done
}


# this function checks if the given VID:PID combination matches the VID:PID
# of the rootdisk. Returns true if it does, false otherwise.
check_rootdisk_vidpid()
{
        set_rootdisk
        dogi_whole_slice $rootdisk rootdev
	input_len=`strlen $rootdev`
	match_len=`expr $rootdev : 's.*'`
	if [ $input_len -eq $match_len ]
	then 
        	rootvid=`/etc/vx/diag.d/vxdmpinq $DISK_RAWPATH/$rootdev |\
                        grep "Vendor" | awk '{ print $4 }'`
        	rootpid=`/etc/vx/diag.d/vxdmpinq $DISK_RAWPATH/$rootdev |\
                        grep "Product" | awk '{ print $4 }'`
	        root_vid_pid1="$rootvid:$rootpid"
        	root_vid_pid2="*:$rootpid"
        	root_vid_pid3="$rootvid:*"

        	[ "$1" = "$root_vid_pid1" ] || [ "$1" = "$root_vid_pid2" ] ||\
                	[ "$1" = "$root_vid_pid3" ]
	else
		return 1
	fi
}

check_path_for_disk()
{
	path=$1
	last_path=0
	diskname=`vxdmpadm getdmpnode nodename=$path | sed -n '3,$p' | \
					awk '{ print $1; }'`
	npaths_enabled=`vxdmpadm getdmpnode nodename=$path | sed -n '3,$p' | \
					awk '{ print $5; }'`
	path_state=`vxdmpadm getsubpaths dmpnodename=$diskname | sed -n '3,$p' | \
					grep "^$path" | awk '{ print $2; }'`

	if [ "$npaths_enabled" -eq 1 ] && [ "$path_state" = "ENABLED" ]
	then
		last_path=1
	fi

	if [ $# -gt 1 ]
	then
		eval "${2}=\"$diskname\""
	fi

	[ "$last_path" -eq 0 ]
}
#
# voladm_alignval -- round up a unaligned value to an alignment boundary
#
# Usage: voladm_align <up|down> <value> <alignment>
#
# The result is sent to stdout.
#
voladm_alignval()
{
	_al_dir=$1
        _al_val=$2
        _al_align=$3

        _al_balance=`expr $_al_val % $_al_align`
        if [ $_al_balance -ne 0 ]
        then
                _al_val=`expr $_al_val - $_al_balance`

		if [ "$_al_dir" = "up" ]
		then
                	_al_val=`expr $_al_val + $_al_align`
        	fi
	fi
        echo $_al_val
}

cmd_supported()
{
	cmd=$1
}

. ${VOLADM_LIB:-/usr/lib/vxvm/voladm.d/lib}/vxadm_syslib.sh
