#!/bin/sh -
# $Id: vxadm_syslib.sh,v 1.63.10.21.32.1 2009/08/17 02:39:02 qwang Exp $
# #ident "$Source: /project/unixvm-cvs/src/linux/cmd/vxvm/voladm/vxadm_syslib.sh,v $"

# $Copyright: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $


#
# vxadm_syslib.sh -- library of routines that need to be ported
#		     for each platform
#

#
# Determine if the platform is running with SELinux enabled (i.e. enforcing
# or permissive)
# Returns 1 if not enabled, 0 if enabled
#
is_selinux_enabled()
{
	retval=0
	[ -x /usr/sbin/selinuxenabled ] && selinuxenabled || retval=1
	return $retval
}

#
# For the 'add' operation check for any inuse devices. The user will
# decide whether these inuse partitions should be encapsulated.
#

bail()
{
	quit 10
}

xcmd()
{
	if [ ! -z "$VOL_VERBOSE" ]
	then
		echo "!$VOL_XCMD $@" >&2
	fi
	$VOL_XCMD "$@"
	[ $? = 0 ] || bail
}

inuse_check()
{
	mount_outfile=$tmpfile2

	/bin/mount > $mount_outfile

	inuse_list=
	not_inuse_list=

	shift $#	# Reset argument count to zero.
	set -- $dev_list
	while [ $# -gt 0 ]
	do
		device=$1
		daname=$2

		voladm_disk_in_use $device
		if [ $? -eq 0 ]
		then
			append_list inuse_list $device $daname
		else
			append_list not_inuse_list $device $daname
		fi
		shift $dev_num
	done

	if [ -n "$inuse_list" ]
	then
		if [ `list_count $inuse_list` -eq $dev_num ]
		then
			ewritemsg -f $disp_file -M vxvmshm:242 \
"One or more partitions of the following disk device are in use as
  a mounted file system. The disk cannot be initialized, but you can
  encapsulate the existing disk partitions as volumes.
  Output format: [Device_Name]"
		else
			ewritemsg -f $disp_file -M vxvmshm:243 \
"One or more partitions of the following disk devices are in use as
  a mounted file system. The disks cannot be initialized, but you can
  encapsulate the existing disk partitions as volumes.
  Output format: [Device_Name]"
		fi

		voladm_yorn_batch -h \
		-g $dev_num -p $dev_index -d $dev_index \
		-f $disp_file -M vxvmshm:1004 -m vxvmshm:1005 \
		"Encapsulate these devices?" Y \
		"Encapsulate this device?" y \
		inuse_list encap_list ignore_list

		if [ -n "$encap_list" ]
		then
			append_list ckinit_encap $encap_list
		fi
	fi

	# If any not-inuse disks were found, then set dev_list to
	# the not-inuse list and continue. Otherwise if no not-inuse
	# disks were found all other disks have been set-up for
	# encapsulation or have been ignored so return.
	if [ -n "$not_inuse_list" ]
	then
		dev_list="$not_inuse_list"
	fi
}
#
# get_mountdevice dir basevarname 
#
# Locate dir in the fstab file and set the following variables as
# appropriate:
#
#	basevarname_bdev	- the block device for the file system
#	basevarname_rdev	- the raw device for the file system
#	basevarname_disk	- the disk device name, if the
#				  root file system is on a regular disk
#	basevarname_vol		- the volume containing the file system,
#	basevarname_dg		- the disk group containing the volume
#
#
get_mountdevice()
{
	# On Linux read the /etc/fstab to get mount information.
	__bdev=
	__cdev=
	__disk=
	__vol=
	__dg=
	exec 9<&0 < $SYSROOT/etc/fstab
	while read _special _mountp _fstype _mntopts a b
	do
		case ${_special} in
		'#'* | '')	#  Ignore comments, empty lines
				continue ;;
		'-')		#  Ignore no-action lines
				continue ;;
		esac
		if [ "${_mountp}" = "$1" ]
		then
			__bdev=$_special
			case $__bdev in
		'/dev/disk/by-id/'*)
			__bdev=`readlink --canonicalize $_special`
			;;
		'/dev/disk/by-uuid/'*)
			__bdev=`readlink --canonicalize $_special`
			;;
		'/dev/disk/by-part/'*)
			__bdev=`readlink --canonicalize $_special`
			;;
		'LABEL='*) label=`echo $__bdev | awk '{n=split($1,x,"="); print x[n] ;}'`;
                           blkdev=`vxgetdev -L $label` ;
                           if [ $? -ne 0 ]; then
                                label="$label" ugettxt -s ERROR "Can't find corr device for label \${label}" vxvmshm:3103;
                                quit 0 ;
                           fi
                           __bdev=$blkdev ;
                                ;;
		'UUID='*) uuid=`echo $__bdev | awk '{n=split($1,x,"="); print x[n] ;}'`;
                           blkdev=`vxgetdev -U $uuid` ;
                           if [ $? -ne 0 ]; then
                                label="$label" ugettxt -s ERROR "Can't find corr device for label \${label}" vxvmshm:3103;
                                quit 0 ;
                           fi
                           __bdev=$blkdev ;
                                ;;
		'/dev/VolGroup'*)
			# For LVM only look at the first disk in the volume
			# group, for the moment.
			__vg=`echo $__bdev | awk -F[/-] '{print $3}'`
			__pv=`pvdisplay -C -o pv_name,vg_name --noheadings \
				2>/dev/null | awk "\\\$2~/$__vg/{print \\\$1}" |
				head -1`
			__bdev=`voladm_basename $__pv`
				;;

			esac
	
			__cdev=$__bdev
			__vol=`expr "\${__bdev}" : '/dev/vx/dsk/\(.*\)' \
								2>/dev/null`
			case $__vol in
			*"/"*"/"*) __vol=;;
			*"/"*)	__dg="`expr "\$__vol" : '\(.*\)/.*'`"
				__vol="`expr "\$__vol" : '.*/\(.*\)'`";;
			*)      __vol=;;
			esac
			if [ "X$__vol" = "X" ]; then
				dogi_slice_to_device ${__bdev} __disk
				__disk=`voladm_basename $__disk`
			fi
			break
		fi
	done
	exec <&9 9<&-
	# rootdisk will  be empty if the root is defined as a volume in
	# the /etc/vfstab file.
	eval "${2}_bdev=$__bdev ${2}_cdev=$__cdev ${2}_disk=$__disk"
	eval "${2}_vol=$__vol ${2}_dg=$__dg"
}

#
# get_dmpname pathname basename
#
#	basename_dmp - dmpnodename for the path $pathname
#
# get_dmpname - returns the dmpnodename of the path
#
get_dmpname()
{
	__dmp=
	__full=

	# DMP:
	# Check if rootdisk is part of a multipath
	# DMP device, then return name of the actual
	# DMP device rather than the subpath

	if [ -b /dev/vx/dmpconfig ]
	then
		dogi_whole_slice $1 __full
		dmpdisk=`/usr/sbin/vxdmpadm getdmpnode nodename=$__full 2> /dev/null | sed -n '3,$p' | awk '{print $1}'`
		dogi_slice_to_device $dmpdisk __dmp
		if [ -z "$__dmp" ]
		then
			__dmp=$1
		fi

	else
		__dmp=$1
	fi
	eval "${2}_dmp=$__dmp"
}
	
#
# set_bootdisk - set parameters for the root file system
#
set_rootdisk()
{
	mpdev=
	rootslice=
	get_mountdevice / root
	root_is_vol=`echo $root_bdev | grep -c rootvol`
	if [ $root_is_vol -ne 0 ]; then

              #  Note: the root disk may not be unique when the 'rootvol'    
              #  is mirrored, so care should be taken when calling this
              #  routine.  The 'find_bootloader()' is ok because  
              #  the MBR is written to mirrors.  This is also not an issue 
              #  for 'vxvm-reconfig' since mult plexes are not allowed 
              #  on the 'rootvol' for the "unroot" operation.

		boot_dg=`vxdg bootdg`
		rootplex=`vxprint -g $boot_dg -p \
			-e 'pl_kstate==ENABLED && assoc.v_use_type="root"' \
			-F "%pl_name" | tail -n 1 2>/dev/null`
		rootsd=`vxprint -g $boot_dg -F "%aslist" $rootplex 2>/dev/null` 
		rootdisk=`vxprint -g $boot_dg $rootsd -F "%da_name" 2>/dev/null`
		return
	fi

	# for backward compatibility, set rootdisk

	# iff DMP:
	# Check if rootdisk is part of a multipath
	# DMP device, then return name of the actual
	# DMP device rather than the subpath

	if [ -b /dev/vx/dmpconfig ]
	then
		dogi_whole_slice $root_disk rootslice
		rootdmpdisk=`/usr/sbin/vxdmpadm getdmpnode nodename=$rootslice 2> /dev/null | sed -n '3,$p' | awk '{print $1}'`
		dogi_slice_to_device $rootdmpdisk mpdev
		rootdisk=$mpdev
		if [ -z "$rootdisk" ]
		then
			rootdisk=$root_disk
		fi

	else
		rootdisk=$root_disk
	fi
}

#
# Set OS specific variables
#
set_OS_variables()
{
	VOL_FULL_SLICE=
	VOL_GHOST_LEN=1
	VOL_PRIV_OFFSET=1
	PRIVLEN_NOCONFIG=160
	PRIVLEN_CONFIG=65536		# 32 MB
	PRIVLEN_MAX=524288
	EDVTOC=vxedpart
	PRTVTOC=vxprtpart
	VOLD=vxconfigd
	CDS_ALIGN=16			# 8 KB
	CDS_PRIV_OFFSET=256		# 128 KB
	CDS_PRIV_SLICE=7

	PRIVLEN_PREV=2048		# 1MB		
	PRIVLEN_OLD=1024		# 512K	

	DISK_RAWPATH="/dev"
	DISK_BLKPATH="/dev"
	VOL_DISK_RAWPATH="/dev/vx/rdmp"
	VOL_DISK_BLKPATH="/dev/vx/dmp"

# ROOTABILITY
	VOL_ROOTABLE_FILESYSTEMS="/ /usr /var"
	VOL_KEEPER_FILESYSTEMS=""

	VOL_VFSTAB_PATH="/etc/fstab"

	export VOL_FULL_SLICE VOL_PRIV_SLICE VOL_PUB_SLICE VOL_PRIV_OFFSET 
	export EDVTOC PRTVTOC VOLD
	export VOL_GHOST_LEN
	export VOL_DISK_RAWPATH VOL_DISK_BLKPATH
	export VOL_AP_DISK_RAWPATH VOL_AP_DISK_BLKPATH
	export VOL_ROOTABLE_FILESYSTEMS
	export VOL_VFSTAB_PATH
}

voladm_mounted_filesystems()
{
	fslist=`/bin/mount | sed 's/ .*//'`
	if [ $# -gt 0 ]
	then
		eval "${1}=\"$fslist\""
	else
		echo $fslist
	fi
}

voladm_basename()
{
        cnt=`echo $1 | grep -c "/dev/vx/rdmp/"`
        if [ $cnt -ne 0 ]; then
                d=`expr "$1" : '/dev/vx/rdmp/\(.*\)\$'`;
        else
                cnt=`echo $1 | grep -c "/dev/vx/dmp/"`
                if [ $cnt -ne 0 ]; then
                        d=`expr "$1" : '/dev/vx/dmp/\(.*\)\$'`;
                else
                        d=`expr "$1" : '/dev/\(.*\)\$'`;
                fi
        fi

	if [ "$d" != "" ]
	then
		echo $d
	else
		echo $1
	fi
}
 
#
# voladm_mount_table - output (or copy to a file) the
# table of "filesystems to be mounted". On some unix systems,
# root (/) is not part of the fstab, so it may need to
# be represented explicitly.
 
voladm_mount_table()
{
	if [ $# -gt 0 ]
	then
		xcmd cp $SYSROOT/$VOL_VFSTAB_PATH ${1}
		if is_selinux_enabled
		then
			xcmd chcon --reference=$SYSROOT/$VOL_VFSTAB_PATH ${1}
		fi
	else
		cat $SYSROOT/$VOL_VFSTAB_PATH
	fi
}

voladm_disk_in_use()
{
	disk=`voladm_basename $1`

	allsubpaths=
	if [ -f /usr/sbin/vxdmpadm ]
	then
		allsubpaths=`/usr/sbin/vxdmpadm getsubpaths dmpnodename=$disk \
				2> /dev/null | sed -n '3,$p' | \
				grep -v "=" | awk '{print $1}'`
	fi

	# check if mounted as disk name
	_diskdev=
	dogi_slice_to_device "$disk" _diskdev
	if [ "X$_diskdev" != "X" ] ; then
		mount | egrep "/dev(/|/vx/dmp/)$_diskdev[0-9]*\b" \
					> /dev/null 2> /dev/null
		if [ $? -eq 0 ] ; then
			return 0
		fi
	fi

	if [ "X${allsubpaths}" = X ] ; then
		return 1
	fi

	# check if mounted as dmp name
	for subpath in $allsubpaths
	do
		_diskdev=
		dogi_slice_to_device "$subpath" _diskdev
                if [ "X$_diskdev" = "X" ] ; then
                        continue
                fi

                mount | egrep "/dev/$_diskdev[0-9]*\b" \
                                                > /dev/null 2> /dev/null
                if [ $? -eq 0 ] ; then
                        return 0
                fi
        done
        return 1
}

pipe_escape()
{
	echo $1 | sed -e 's/|/\\\|/g'
}

is_dmp_enabled()
{
        is_enabled=0
        is_enabled=`lsmod | grep -c vxdmp 2>/dev/null`
        if [ $is_enabled -ne 0 ]; then
                if [ -b /dev/vx/dmpconfig ]; then
                        return 0;
                fi
        fi

        return 1;
}

# add_enclr_expression -- add the given enclosure name to the
# expression to egrep out disk access names. The second argument should
# be the variable name of the expression to be used.
#
add_enclr_expression()
{
	enclr=$1
	
	eval "expression=\${${2}}"

	if [ -z "$expression" ]
	then
		expression="${enclr}_"
	else
		expression="${expression}|${enclr}_"
	fi
	
	expression=`pipe_escape $expression`
	eval "${2}=$expression"
}

create_enclr_expression()
{
	enclr=$1
	eval "expression=\${${2}}"
	if [ -z "$expression" ]
	then
		expression="${enclr}"
	else
		expression="${expression}|${enclr}"
	fi
	eval "${2}=$expression"
}
#
# controllers_on_system -- list all controllers on the system
#
controllers_on_system()
{
# DEVFS 
	cntr_list=
	bad_cntr=
	for cntr_num in `$DIAGDIR/vxautoconfig | awk '{print $3}' | cut -d= -f2`
	do
		cntr=c$cntr_num
		echo $bad_cntr | egrep "$cntr:" 1>/dev/null 2>/dev/null
		if [ $? -eq 0 ]
		then
			continue
		fi

		echo $cntr_list | egrep "$cntr:" 1>/dev/null 2>/dev/null
		if [ $? -ne 0 ]
		then
			if [ -f $VOL_CONFIG_DIR/vxvm.exclude ]
			then
				vxdmpadm checkexclude vxvm ctlr=$cntr:$cntr 2>/dev/null
				ret=$?
				if [ $ret -eq 0 -o $ret -eq 3 ]
				then
					cntr="$cntr" egettxt "<excluding $cntr>" vxvmshm:67 >&2
					bad_cntr="$bad_cntr$cntr:"
					continue
				fi
			fi

			if [ -f $CNTRL_EXCLUDE_FILE ]
			then
				if fgrep -x "$cntr" $VOL_CONFIG_DIR/cntrls.exclude > /dev/null 2>&1
				then
					cntr="$cntr" egettxt "<excluding $cntr>" vxvmshm:67 >&2
					bad_cntr="$bad_cntr$cntr:"
					continue
				fi
			fi

			cntr_list="$cntr_list$cntr:"
		fi
	done
	echo $cntr_list | sed 's/:$//g' | sed 's/:/\
/g'

}

#
# disks_on_controller -- given a controller number, list all disks
# that we can find on that controller to stdout.
#
disks_on_controller()
{
# DEVFS:  plus maybe a better permanent sol'n?
	ctlr=$1
	ctlr_num=`expr $ctlr : 'c\([0-9][0-9]*\)'`
	${DIAGDIR}/vxautoconfig | egrep "ctlr=$ctlr_num$" | awk '{print $1}'

}

#
# disks_on_enclosure -- given an enclosure name, list all disks
# that we can find on that enclosure to stdout.
#
disks_on_enclosure()
{
	enclr=$1
	vxdmpadm getdmpnode enclosure=$enclr | sed -n '3,$p' | awk '{ print $1; }'
}

#
# expand_device_wildcard -- expand a disk specification into all
# matching devices. This is used in voladm_disk_device_list. 
#
expand_device_wildcard()
{
# DEVFS 
	list_var=$1
	eval "disklist=\${${list_var}}"
	excl_disks=$2
	excl_cntrlr=$3
	excl_enclr=$4
	disk_excluded_var=$5
	eval "disk_excl=\${$disk_excluded_var}"
	cntrlr_excluded_var=$6
	eval "cntrlr_excl=\${$cntrlr_excluded_var}"
	enclr_excluded_var=$7
	shift 7

	# The string "${VOL_FULL_SLICE}\$" must be appended to each disk
	# device string in the egrep pattern list to ensure that only the
	# specified disk(s) is filtered out. Moreover, the logical-or
	# character must be introduced for this scheme to work.
	if [ -n "$excl_disks" ]
	then

	   excl_disks=`echo $excl_disks | sed "s/ /${VOL_FULL_SLICE}\$\|/g" | \
				sed "s/$/${VOL_FULL_SLICE}\$/"`
	fi

	while [ $# -gt 0 ]
	do
		spec=$1
		pattern=

		# Verify that it's valid
		slen=`strlen $spec`
		ldir=
		case "$spec" in
			*_)
			   mlen=`expr $spec : '.*_'`
			   [ $mlen -eq $slen ] && pattern="$spec*"
			   ldir=$VOL_DISK_RAWPATH
			   ;;
			*_*)
			   mlen=`expr $spec : '.*_[0-9a-fA-F][0-9a-fA-F]*'`
			   [ $mlen -eq $slen ] && pattern="$spec"
			   ldir=$VOL_DISK_RAWPATH
			   ;;
			allenclr)
			   pattern="*_[0-9]*"
			   ldir=$VOL_DISK_RAWPATH
			   ;;
			*) pattern=$spec
			   ldir=$VOL_DISK_RAWPATH
			   ;;
		esac

		if [ -z "$pattern" ]
		then
			ewritemsg -s ERROR -M vxvmshm:766 \
"Input not recognized: \"$spec\""
			return 1
		fi
		push_list disklist \
		 `ls -1 ${ldir}/${pattern}${VOL_FULL_SLICE} 2>/dev/null | \
			tee $_vdskd_rawmatch |
			if [ -n "$excl_cntrlr" ]
			then
				egrep -v -e "$excl_cntrlr"
			else
				cat
			fi |
			tee $_vdskd_cntrlmatch |
			if [ -n "$excl_disks" ]
			then
				egrep -v -e "$excl_disks"
			else
				cat
			fi |
			tee $_vdskd_diskmatch |
			if [ -n "$excl_enclr" ]
			then
				egrep -v -e "$excl_enclr"
			else
				cat
			fi |
			tee $_vdskd_enclrmatch |
			sed "s%${ldir}/\(.*\)${VOL_FULL_SLICE}$%\1%"`

		# Add excluded controllers and disks to the lists
		append_list $cntrlr_excluded_var \
		 `comm -3 $_vdskd_rawmatch $_vdskd_cntrlmatch`
		append_list $disk_excluded_var \
		 `comm -3 $_vdskd_cntrlmatch $_vdskd_diskmatch`
		append_list $enclr_excluded_var \
		  `comm -3 $_vdskd_diskmatch $_vdskd_enclrmatch`

		shift
	done

	eval "$list_var=\"$disklist\""
	return 0
}

#
# These are the dogi_* routines used by the rest of the administrative
# scripts. These should mask out any system-specific DA-naming or
# disk-related dependencies.
#

#
# dogi_is_mylex -- returns true if give device name is /dev/rd/cXdX{pX} or
#  /dev/dac960/cXdX{pX} format
#
dogi_is_mylex()
{
	ismylex=`expr $1 : '.*\(rd\/\).*'`
	ismylex2=`expr $1 : '.*\(dac960\/\).*'`
	iscompaq2=`expr $1 : '.*\(cciss\/\).*'`
	iscompaq=`expr $1 : '.*\(ida\/\).*'`
	if [ "$ismylex" != "" -o "$ismylex2" != "" -o "$iscompaq" != "" -o \
		"$iscompaq2" != "" ]
	then
		echo 1
	else
		echo 0
	fi
}

#
# dogi_name_is_device -- returns true if the (single) argument is
# in the form of a device name.
#
dogi_name_is_device()
{
	dogi_slice_num $1 slc
	[ "$slc" = "" ]
}

#
# dogi_name_is_slice -- returns true if the (single) argument is in
# the form of a slice name.
#
dogi_name_is_slice()
{
	name=`basename $1`
	case "$name" in
		*_*)
		   name_len=`strlen $name`;
		   match_len1=`expr $name : \
		   '.*_[0-9a-fA-F][0-9a-fA-F]*'`
		   match_len2=`expr $name : \
		   '.*_[0-9a-fA-F][0-9a-fA-F]*s[0-9][0-9]*'`
		   [ "$name_len" -eq "$match_len1" ] || \
		   [ "$name_len" -eq "$match_len2" ]
		   ;;
		s*)
		   dogi_slice_num $1 slc
		   [ "$slc" != "" ]
		   ;;
		h*)
		   dogi_slice_num $1 slc
		   [ "$slc" != "" ]
		   ;;
		c*)
		   dogi_slice_num $1 slc
		   [ "$slc" != "" ]
		   ;;
		*)
		   dogi_slice_num $1 slc
		   [ "$slc" != "" ]
		   ;;
	esac
}
#
# dogi_slice_num -- echo the slice part of the complete slice name
#
#	If optional second argument is provided, will set a variable by
#	that name to the slice portion of the first argument
#
#	For standard naming, the slice num is any number on the end. 
#	Special cases for mylex done here, and any use of DEVFS will need also
#
dogi_slice_num()
{
	if [ `dogi_is_mylex $1` -eq 0 ]
	then
		name=`basename $1`
		case "$name" in
			*_*)
			    s=`expr $name : '.*_[0-9a-fA-F][0-9a-fA-F]*\(s[0-9][0-9]*\)$'`
			    ;;
			s*)
			   s=`expr $1 : '.*[^0-9]\([0-9]*\)'` ;;
			h*)
			   s=`expr $1 : '.*[^0-9]\([0-9]*\)'` ;;
			*)
			   vxdisk list $name 1>/dev/null 2>&1
		   	   if [ $? -ne 0 ]
		   	   then
				vxdmpadm getsubpaths dmpnodename="$name" \
							> /dev/null 2>&1
				if [ $? -ne 0 ]
				then
					s=
				else
					s=`expr $1 : '.*[^0-9]\([0-9]*])'`
				fi
			   fi
			   ;;
		esac
	else
		s=`expr $1 : '.*\(p[0-9]*\)'`
	fi
	if [ $# -gt 1 ]
	then
		eval "${2}=$s"
	else
		echo $s
	fi
}
#
# dogi_is_whole_slice -- return true if the slice names the special "whole"
# slice for the disk
#
dogi_is_whole_slice()
{
	dogi_slice_num $1 slice_name
	[ "$slice_name" = "${VOL_FULL_SLICE}" ]
}

#
# dogi_device_slice -- create a slice name out of the given device
# name and slice
dogi_device_slice()
{
	if [ `dogi_is_mylex $1` -eq 0 ]
	then
		name=`basename $1`
		case "$name" in
			*_*)
			   eval "${3}=${1}s${2}" ;;
			s*)
			   eval "${3}=${1}${2}" ;;
			h*)
			   eval "${3}=${1}${2}" ;;
			*)
			   eval "${3}=${1}${2}" ;;
		esac
	else
		eval "${3}=${1}p${2}"
	fi
}
#
# dogi_slice_to_device -- given name, lop off the "slice" part, if there is one
#
#
dogi_slice_to_device()
{
	name=`voladm_basename $1`
	if not dogi_name_is_slice $name
	then
		d=`voladm_basename $1`		# watch overloading!
	else
		if [ `dogi_is_mylex $1` -eq 0 ]
		then
			name=`basename $1`
			case "$name" in
				*_*)
				   d=`expr $1 : '\(.*_[0-9a-fA-F][0-9a-fA-F]*\)'` ;;
				s*)
				   d=`expr $1 : '\(.*[^0-9]\)[0-9]*'` ;;
				h*)
				   d=`expr $1 : '\(.*[^0-9]\)[0-9]*'` ;;
				*)
				   vxdisk list $name 1>/dev/null 2>&1
				   if [ $? -eq 0 ]
				   then
					d=$name
				   else
					daname=`vxdmpadm getsubpaths \
						dmpnodename="$name" | \
						grep $name | \
						awk '{print $1}' 2>/dev/null`
					if [ ! -z "$daname" ]
					then
					     d=`expr $1 : '\(.*[^0-9]\)[0-9]*'`
					else
					     d=""
					fi
				   fi
				   ;;
			esac
		else
			d=`expr $1 : '\(.*\)p[0-9]*'`
		fi
	fi
	if [ $# -gt 1 ]
	then
		eval "${2}=$d"
	else
		echo $d
	fi
}

#
# dogi_name_to_device -- return the device name, given a name
#
dogi_name_to_device()
{
	ntd_name=`voladm_basename $1`
	if dogi_name_is_device $ntd_name
	then
		d=$ntd_name
	else
		dogi_slice_to_device $ntd_name d
	fi

	if [ $# -gt 1 ]
	then
		eval "${2}=$d"
	else
		echo $d
	fi
}

#
# dogi_slice_from_path -- given a disk pathname, strip off everything
# but the slice name.
#
dogi_path_to_slice()
{
	if [ `dogi_is_mylex $1` -eq 0 ]
	then
		name=`basename $1`
		case "$name" in
			*_*)
			   d=`expr "$1" : '.*/\(s.*\)\$'`;;
			s*)
			   d=`expr "$1" : '.*/\(.*\)\$'`;;
			h*)
			   d=`expr "$1" : '.*/\(.*\)\$'`;;
			*)
			   d=`expr "$1" : '.*/\(.*\)\$'`;;
		esac
	else
		d=`expr "$1" : '.*/.*\(p.*\)\$'`;
	fi

	if [ $# -gt 1 ]
	then
		eval "${2}=$d"
	else
		echo $d
	fi
}
#
# dogi_whole_slice -- given a device, return the slice name of the
# "whole slice"
#
dogi_whole_slice()
{
	ws_name=`voladm_basename $1`
	if not dogi_name_is_device $ws_name
	then
		return 1
	fi

	p="${ws_name}${VOL_FULL_SLICE}"
	if [ $# -gt 1 ]
	then
		eval "${2}=${p}"
	else
		echo $p
	fi
}

#
# dogi_device_rawpath -- return the path to the character node for the device
#

dogi_device_rawpath()
{
	if not dogi_name_is_device $1
	then
		return 1
	fi
	
	dogi_slice_rawpath $1 pth

	if [ $# -gt 1 ]
	then
		eval "${2}=${pth}";
	else
		echo "$pth";
	fi
}

dogi_slice_rawpath()
{
	name=`voladm_basename $1`

	if is_dmp_enabled
        then
                p="${VOL_DISK_RAWPATH}/${name}";
        else
                p="${DISK_RAWPATH}/${name}";
        fi


	if [ $# -gt 1 ]
	then
		eval "${2}=${p}";
	else
		echo "$p";
	fi
}

#
# dogi_device_blkpath -- return the path to the block node for the device
#
dogi_device_blkpath()
{
	if not dogi_name_is_device $1
	then
		return 1
	fi
	
	dogi_slice_blkpath $1 pth
	if [ $# -gt 1 ]
	then
		eval "${2}=${pth}"
	else
		echo "$pth"
	fi
}

dogi_slice_blkpath()
{

	name=`voladm_basename $1`

	if is_dmp_enabled
	then
		p="${VOL_DISK_BLKPATH}/${name}";
	else
		p="${DISK_BLKPATH}/${name}";
	fi

	if [ $# -gt 1 ]
	then
		eval "${2}=${p}";
	else
		echo "$p";
	fi
}

#
# dogi_device_blkpath -- return the path to the block node for the device
#
dogi_device_ospath()
{
	if not dogi_name_is_device $1
	then
		return 1
	fi
	
	dogi_slice_ospath $1 pth
	if [ $# -gt 1 ]
	then
		eval "${2}=${pth}"
	else
		echo "$pth"
	fi
}

dogi_slice_ospath()
{
	name=`voladm_basename $1`

	p="${DISK_BLKPATH}/${name}";

	if [ $# -gt 1 ]
	then
		eval "${2}=${p}";
	else
		echo "$p";
	fi
}

#
# dogi_name_daname -- return an appropriate DA record name for the
# given device name
#
dogi_name_daname()
{
	nd_name=`voladm_basename $1`

	if not dogi_name_is_device $nd_name
	then
		return 1
	fi

	dogi_whole_slice "$nd_name" daname || return 1

	if [ $# -gt 1 ]
	then
		eval "${2}=${daname}"
	else
		echo "$daname"
	fi
}

#
# dogi_root_device -- return true if the named device is the boot disk
#
dogi_root_device()
{
	dname=$1
	if not dogi_name_is_device $dname
	then
		return 1
	fi
	
	set_rootdisk
	[ "$dname" = "$rootdisk" ]
}

#
# dogi_name_is_cntrlr -- returns true if the argument is in the form
# of a controller name.
#
dogi_name_is_cntrlr()
{
# DEVFS 
	return 1
}

#
# dogi_darecs_on_device -- list any DA records that are defined on
# the named device. The form of the list is "<da> <tm> <type>". The
# list is put into the variable named by the second argument. If any
# are found, return true.
#
dogi_darecs_on_device()
{
	name=`voladm_basename $1`

        # three cases:
        #       A) enclosure based name
        #       B) i) mylex device
        #          ii) normal device(sdx,hdx)

        >| $tmpfile4
        ENCLOSURE_DEVICE_SEPERATOR="_"

	if [ `echo $name | grep $ENCLOSURE_DEVICE_SEPERATOR` ]; then
		is_simple_disk=`vxdisk list | grep simple | grep -c "^${name}s[0-9][0-9]*" `
		if [ $is_simple_disk -ne 0 ]; then
			LANG= vxdisk list | grep "^${name}s[0-9][0-9]*" > $tmpfile4
		fi
	else
		if [ `dogi_is_mylex $name` -eq 0 ]; then
			is_simple_disk=`vxdisk list | grep -c "^${name}[0-9][0-9]*" `
			if [ $is_simple_disk -ne 0 ]; then
				LANG= vxdisk list | grep "^${name}[0-9][0-9]*" > $tmpfile4
			fi
		else
			is_simple_disk=`vxdisk list | grep -c "^${name}p[0-9][0-9]*" `
			if [ $is_simple_disk -ne 0 ]; then
				LANG= vxdisk list | grep "^${name}p[0-9][0-9]*" > $tmpfile4
			fi
		fi
	fi

	if [ $is_simple_disk -eq 0 ]; then
		LANG= vxdisk list | awk -v aname=$name \
		'$1 == aname { print $0}' >$tmpfile4
	fi

	xlist=`cat $tmpfile4`
	rm $tmpfile4

	if [ $# -gt 1 ]
	then
		eval "${2}=\"${xlist}\""
	else
		if [ "$xlist" != "" ]
		then
			echo "$xlist"
		fi
	fi
	[ `list_count $xlist` -ne 0 ]
}

#
# dogi_valid_vtoc -- return false if the disk does not have a valid,
# usuable partition table that contains at least one partition
#
dogi_valid_vtoc()
{
	vv_name=`voladm_basename $1`
	if dogi_name_is_slice $vv_name
	then
		dogi_slice_to_device $vv_name vv_name
	elif not dogi_name_is_device $vv_name
	then
		return 1
	fi

	dogi_device_rawpath $vv_name diskpath
	#
	# Try to read partition table - if error then return immediately
	#
	_name_line=`$PRTVTOC $diskpath 2>&1 | egrep "^Name:"`
	[ $? -ne 0 ] && return 1

	_name_line=`expr "$_name_line" : "Name:.*Type \([0-9]\) .*"`

	if [ "1" = "$_name_line" ]
	then
		#
		# For SUN labels, don't consider the standard back-up slice
		#
		$PRTVTOC $diskpath 2>&1 |  egrep -v "^num 3.*tag 0x5" | \
			egrep "^num" > /dev/null
		[ $? -eq 0 ] && return 0
	else
		$PRTVTOC $diskpath 2>&1 | egrep "^num" > /dev/null
		[ $? -eq 0 ] && return 0
	fi
	return 1
}

#
# dogi_is_boot_device -- return true if the named device or disk media 
#		      record is the boot disk
#		      disk names are in the proper form
#
dogi_is_boot_device()
{
      dname=`voladm_basename $1`
      dgname=$2
      get_mountdevice / root

      #
      # If the aegument is a real disk device then check 
      # to see if it matches the root_disk var set by the
      # get_mountdevice call
      #
      if dogi_name_is_device $dname
      then
	      [ "$dname" = "$rootdisk" ] && return 1
	      return 0
      fi

      #
      # The argument must be a dm name
      #
      dm=$dname

      dmlist=`vxprint -g $dgname -F %dm -e as.as=\"$root_vol\" | sort -u`

      for bootdm in $dmlist
      do
	      [ "$bootdm" = "$dm" ] && return 1
      done

      return 0
}
#
# voladm_get_disk_layout_defaults -- get entries from a given defaults file
#
# SYNTAX: voladm_get_disk_layout_defaults which-file which-attr ret-val
#		where
#			which-file = vxdisk | vxencap
#			which-attr = "" | format | privlen
#				"" => all
#
# RETURNS: 0 on success
#	   1 on failure
#
#	NOTE: success might not return any value since the existence of
#		the file and any particular attribute is optional; that is,
#		upon success ret-val may be ""
#
voladm_get_disk_layout_defaults()
{
        if [ $# -le 2 ]
        then
                echo "voladm_get_disk_layout_defaults: missing arguments"
                return 1
        fi
        if [ "$1" = "vxdisk" ]
	then
                _file=$VOL_VXDISK_DFLT_FILE

        elif [ "$1" = "vxencap" ]
	then
                _file=$VOL_VXENCAP_DFLT_FILE
        else
                eval "${3}="

                echo "voladm_get_disk_layout_defaults: bad first argument: $1"
                return 1
        fi
        #
        # Get default format and/or privlen from the default file if it exists.
        # If there is more than one entry in the file, return the last one.
        #
        if [ ! -f $_file ]
        then
                eval "${3}="
		return 0
        fi
	#
	# Parse the file.
	#
	_privlen=
	_format=
	_valid_entries=0
	_def=`egrep '^privlen=[0-9][0-9]*$' $_file | tail -1`
	[ -n "$_def" ] && {
		eval _$_def

		if [ -z "$_privlen" -o $_privlen -lt 1 -o $_privlen -gt 524288 ]
		then
			export _file _privlen; ewritemsg -s ERROR -M vxvmshm:2572 \
	"Private region length out of range in $_file: privlen=$_privlen."
			return 1
		fi
		_valid_entries=`expr $_valid_entries + 1`
	}
	_def=`egrep '^format=[^ 	#][^ 	#]*$' $_file | tail -1`
	[ -n "$_def" ] && {
		eval _$_def

		if [ "$_format" = "cdsdisk" ] ||
		   [ "$_format" = "sliced" ] ||
		   [ "$_format" = "simple" -a "$1" != "vxencap" ]
		then
			_valid_entries=`expr $_valid_entries + 1`
		else
			export _file _format; ewritemsg -s ERROR -M vxvmshm:2598 \
				"Syntax error in $_file: format=$_format."
			return 1
		fi
	}
	_entries=`grep -v "^#" $_file | wc -l | awk '{print $1}'`

	if [ "$_valid_entries" != "$_entries" ]
	then
		export _file; ewritemsg -s ERROR -M vxvmshm:2027 \
			"Syntax error or duplicate entries in $_file."
		return 1
	fi
	#	
	# Set the requested info.
	#
	if [ -z "$2" ]
	then
        	[ -n "$_privlen" -a -n "$_format" ] && _privlen="$_privlen "

        	eval "${3}=${_privlen}${_format}"

 	elif [ "$2" = "privlen" ]
	then
        	eval "${3}=${_privlen}"

	elif [ -z "$2" -o "$2" = "format" ]
	then
        	eval "${3}=${_format}"
	else
                eval "${3}="

                echo "voladm_get_disk_layout_defaults: bad second argument: $2"
                return 1
	fi
        return 0
}
#
# voladm_set_disk_layout_default -- set an entry in a given defaults file
#
# SYNTAX: voladm_set_disk_layout_defaults which-file which-attr val
#		where
#			which-file = vxdisk | vxencap
#
# RETURNS: None
#
voladm_set_disk_layout_default()
{
        if [ $# -le 2 ]
        then
                echo "voladm_set_disk_layout_default: missing argument"
                return 1
        fi
        if [ "$1" = "vxdisk" ]
	then
                _file=$VOL_VXDISK_DFLT_FILE

        elif [ "$1" = "vxencap" ]
	then
                _file=$VOL_VXENCAP_DFLT_FILE
        else
                echo "voladm_set_disk_layout_default: bad argument"
                return 1
        fi
	_dir=`dirname $_file`
	[ ! -d $_dir ] && mkdir -p $_dir

        _attr=$2
        _val=$3
        #
        # remove any existing lines with the specified attribute and then
        # write a line of the form "attribute=value" to the file.
        #
        _tmp_file=${_file}.$$

        [ -f $_file ] && grep -v "$_attr" $_file > $_tmp_file 2>/dev/null

        [ -n "$_val" ] && echo "${_attr}=${_val}" >> $_tmp_file

        mv $_tmp_file $_file
}
#
# voladm_query_disk_format -- query the user for a disk format
#
# SYNTAX: voladm_query_disk_format [-e] [-p] ret-format [y]
#	where
#		-e - indicates the private region will be used as part of disk
#			encapsulation (w/o -e disk initialization is assumed).
#		-p - indicates that the DG for the disk is precds
#		y - indicates to also query the user about using
#			the default format
# RETURNS: 0 on success
#	   otherwise failure (see also voladm_yorn, read_input)
#
voladm_query_disk_format()
{
 	_kind="vxdisk"
 	_precds="no"
 	while [ $# -gt 1 ]
 	do
 		case $1 in
 		-e)	_kind="vxencap"	;;
 		-p)	_precds="yes"	;;
 		*)	break		;;
 		esac
 		shift
 	done
 	if [ "$_kind" = "vxencap" ]
	then
 		if [ "$_precds" = "yes" ]
 		then
         		eval "${1}=sliced"
         		return 0
 		else
 			_input_list="cdsdisk,sliced"
 		fi
	else
 		if [ "$_precds" = "yes" ]
 		then
 			_input_list="sliced,simple"
 		else
 			_input_list="cdsdisk,sliced,simple"
 		fi
	fi
	voladm_get_disk_layout_defaults $_kind "format" _default_format
	if [ $? -ne 0 -o -z "$_default_format" ]
	then
		_default_format="cdsdisk"
	fi
 	if [ "$_precds" = "yes" -a "$_default_format" = "cdsdisk" ] ||
 	   [ "$_kind" = "vxencap" -a "$_default_format" = "simple" ]
 	then
 		_default_format="sliced"
 	fi
        if [ $# -eq 0 ]
        then
                echo "voladm_query_disk_format: missing argument"
                return 1
        fi
        voladm_help_push disk.format.help

	if [ -n "$2" -a "$2" = "y" ]
	then
		voladm_yorn -M vxvmshm:1545 \
                "Use the default disk format for the disk?" y

		if [ $? -ne 1 ]
                then
        		voladm_help_pop
        		eval "${1}="
        		return 0
		fi
	fi
        _riput=`egettxt "Enter the desired format" vxvmshm:1514`

        while true ; do

		read_input "$_riput" "$_input_list" $_default_format

		if [ "$input" = "cdsdisk" -a "$_precds" = "no" ] ||
		   [ "$input" = "simple" -a "$_kind" != "vxencap"  ] ||
                   [ "$input" = "sliced"  ]
                then
                        break
                fi
                ewritemsg -M vxvmshm:1515 \
                        "Entered format is not valid. Please re-enter."
        done
        voladm_help_pop

        eval "${1}=$input"
        return 0
}
#
# voladm_query_disk_privlen -- query the user for a (disk's) priv region length 
#
# SYNTAX: voladm_query_disk_privlen [-e] ret-privlen [y]
#	where
#		-e - indicates the private region will be used as part of disk
#			encapsulation (w/o -e disk initialization is assumed).
#		 y - indicates to also query the user about using the default
#			private region length
# RETURNS: 0 on success
#	   otherwise failure (see also voladm_yorn, voladm_new_disk_privlen)
#
voladm_query_disk_privlen()
{
	if [ $# -ge 1 -a "$1" = "-e" ]
	then
		shift
		_eopt="-e"
	else
		_eopt=
	fi
        if [ $# -eq 0 ]
        then
                echo "voladm_query_disk_privlen: missing argument"
                return 1
        fi
        voladm_help_push disk.privlen.help

	if [ -n "$2" -a "$2" = "y" ]
	then
		voladm_yorn -M vxvmshm:1548 \
                "Use the default private region length for the disk?" y

		if [ $? -ne 1 ]
                then
        		voladm_help_pop
        		eval "${1}="
        		return 0
		fi
	fi
        voladm_new_disk_privlen $_eopt
        voladm_help_pop
        eval "${1}=$dmprivlen"
        return 0
}

#
# voladm_query_disk_siteid - query the user for a site tag
#
# syntax: voladm_query_disk_siteid enclosure_name 
#       where
#		enclosure_name is name of enclosure on system
# RETURNS: 0 on success


voladm_query_disk_siteid()
{
	if [ $# -eq 0 ]
	then
		echo "voladm_query_disk_siteid: missing argument"
		return 1
	fi
	voladm_help_push disk.siteid.help
	enc_name="$1"; export enc_name
	_riput=`egettxt "Enter site tag for selected disks on enclosure $enc_name" vxvmshm:000`
	read_input "$_riput" "<name>"
	sitename=$input
	voladm_help_pop
	grep $enc_name $_disk_enc | awk '{ print $2 }' > $_disk_list_site
	for i in `cat $_disk_list_site`; do vxdisk settag $i site=$sitename;done
	return 0
}
	
#
# voladm_parse_disk_enc - parse out the enclosure of a disk
#
# syntax: voladm_parse_disk_enc disk
# RETURNS: 0 on success


voladm_parse_disk_enc()
{
	if [ $# -eq 0 ]
	then
		echo "voladm_parse_disk_enc: missing argument"
		return 1
	fi
	disk=$1

	enc=`/usr/sbin/vxdmpadm getsubpaths dmpnodename=$disk  2>/dev/null | sed -n '3,$p' | awk '{ print $6 }' |sort -u`
	echo $enc $disk >> $_disk_enc

	return 0
}

#
# voladm_parse_enc_siteid - parse out list of enclosures
# RETURNS: 0 on success

voladm_parse_enc_siteid()
{
	cat $_disk_enc | awk '{ print $1}' |sort -u | xargs echo > $_site_enc
	return 0
}

#
# voladm_set_siteid - displays message to user and then 
# calls voladm_query_disk_siteid to set the site tag
#
# syntax: voladm_query_disk_siteid disk_list
#       where
#		disk_list is the list of disks user picked
# RETURNS: 0 on success

voladm_set_siteid()
{
	if [ $# -eq 0 ]
	then
		echo "voladm_set_siteid: missing argument"
		return 1
	fi
	new_disk_list=$1
	voladm_parse_enc_siteid
	ewritemsg -M vxvmshm:000 "The following disk(s):"
	voladm_list_display -g $dev_num -d $dev_index $new_disk_list
	ewritemsg -M vxvmshm:000 "belong to enclosure(s):"
	ewritemsg -M vxvmshm:000 "`cat $_site_enc`"
	for i in `cat $_site_enc`; do voladm_query_disk_siteid $i; done
}

#
# voladm_query_disk_layout - query the user for a disk layout
#
# syntax: voladm_query_disk_layout [-e] fmt_ret plen_ret [y]
#	where
#		-e - indicates the private region will be used as part of disk
#			encapsulation (w/o -e disk initialization is assumed).
#		y - indicates to also query the user about using
#			the default disk layout
# RETURNS: 0 on success
#	   otherwise failure (see also voladm_yorn, voladm_query_disk_format
#				voladn_query_disk_privlen)
#
voladm_query_disk_layout()
{
	if [ $# -ge 1 -a "$1" = "-e" ]
	then
		shift
		_eopt="-e"
	else
		_eopt=
	fi
	if [ $# -lt 2  ]
        then
                echo "voladm_query_disk_layout: missing argument"
                return 1
        fi
	if [ -n "$3" -a "$3" = "y" ]
	then
		voladm_help_push disk.layout.help

		voladm_yorn -M vxvmshm:1546 \
	                "Use the default disk layout for the disk?" y
		yorn=$?

        	voladm_help_pop

		if [ $yorn -ne 1 ]
                then
        		eval "${1}="
        		eval "${2}="
        		return 0
		fi
	fi
	voladm_query_disk_format $_eopt _fmt $3
	voladm_query_disk_privlen $_eopt _plen $3

        eval "${1}=$_fmt"
        eval "${2}=$_plen"
        return 0
}
#
# voladm_get_disk_reinit_size - determine the public region size for a disk
#				that would exist after 'initializing' the disk
#
# Usage: voladm_get_disk_reinit_size <ret size> <da name> <setup attrs>
#
# This routine returns (in <ret size>) the size of the public region that
# would exist on the disk (indicated by <da name>) after calling vxdisksetup
# with the attributes, if any, indicated by <setup attrs>.
#
# Currently: this routine only supports the setup attrs of:
#		format, privlen and noreserve
#
# Returns -1 on error; otherise, 0
#
voladm_get_disk_reinit_size()
{
	[ $# -ge 1 ] && eval "$1=0"	# set 0 for error returns
	[ $# -lt 3 ] && return -1

	_daname=$2
	_setup_attrs=$3
	#
	# Parse <setup attrs>.
	#
	_fmt=
	_privlen=
	_nores=
	for _attr in $_setup_attrs
	do
		case $_attr in
		format=*)	_fmt=`expr "$_attr" : "format=\(.*\)"` ;;
		privlen=*)	_privlen=`expr "$_attr" : "privlen=\(.*\)"` ;;
		noreserve)	_nores=y ;;
		*)		return -1 ;;
		esac
	done
	#
	# If no format is specified in <setup attrs>, determine the default
	# value (either from the defaults file or the compiled in default).
	# Same for privlen.
	#
	if [ -z "$_fmt" ]
	then
		voladm_get_disk_layout_defaults vxdisk "format" _fmt
		[ $? -ne 0 ] && return -1
		[ -z "$_fmt" ] && _fmt=cdsdisk
	fi
	if [ -z "$_privlen" ]
	then
		voladm_get_disk_layout_defaults vxdisk "privlen" _privlen
		[ $? -ne 0 ] && return -1
		[ -z "$_privlen" ] && _privlen=65536	# PRIVLEN_CONFIG
	fi
	#
	# Determine desired disk info.
	#
	dogi_name_to_device $_daname _device
	dogi_device_rawpath $_device _path
	eval_cmd=`vxparms -s $_path 2> /dev/null`
	[ $? -ne 0 ] && return -1

	eval $eval_cmd
	#
	# NOTE: the eval above just set STAT_xxx and part_type variables
	#
	[ -z "$STAT_DISKNUMSEC" -o -z "$STAT_SECT" ] && return -1
	[ -z "$STAT_SECPERCYL" -o -z "$part_type" ] && return -1

	if [ "$VOL_SUN_PART_TYPE" = "$part_type" ]
	then
		_soldevsize=$STAT_DISKNUMSEC
	else
		#
		# For non-Solaris labelled disks, need to determine the
		# Solaris disk size (if possible).
		#
		eval_cmd=`/usr/lib/vxvm/diag.d/vxscsi -g $_path 2> /dev/null`

		if [ $? -eq 0 ]
		then
			eval_cmd=`echo $eval_cmd | sed 's/.*://'`
			eval $eval_cmd
			# NOTE: the eval above just set nhead, nsect, and ncyl
		else
			#
			# Check if the disk is a SCSI disk, if so error out.
			#
			hdparm  -i $rawpath 2>&1 | grep "SCSI disk" >/dev/null
			[ $? -eq 0 ] && return -1 
			#
			# For non-CDS disks, estimate Solaris cyls as DOS cyls.
			# For CDS disks, error out non-SCSI disks.
			#
			case $_fmt in
			cds*)	return -1 ;;	# CDS requires SCSI disk
			*)	;;
			esac
			nhead=$STAT_HEADS
			nsect=$STAT_SECT
			ncyl=$STAT_CYLS
		fi
		[ -z "$nhead" -o -z "$nsect" -o -z "$ncyl" ] && return -1
		_soldevsize=`expr $nhead \* $nsect \* \( $ncyl - 2 \)`
	fi
	#
	# Determine the public region size based on the known info.
	#
	case $_fmt in
	cds*)	_8k=16
		_privlen=`voladm_alignval up $_privlen $_8k`
		_devsize=`voladm_alignval down $_soldevsize $_8k`
		_res=256	# priv offset
		;;
	simple|sliced)
		if [ "$VOL_SUN_PART_TYPE" = "$part_type" ]
		then
			_devspc=$STAT_SECPERCYL
			_devsize=`voladm_alignval down $_soldevsize $_devspc`
			if [ -z "$_nores" ]
			then
				_res=`voladm_alignval up 256 $_devspc`
				_privlen=`expr $_privlen + 256`
			else
				_res=0
			fi
			if [ "sliced" = "$_fmt" ]
			then
				_privlen=`voladm_alignval up $_privlen $_devspc`
			fi
		else
			#
			# If doing CDS reserve, the public and private regions
			# cannot lie in the last two Solaris cylinders. Note:
			# its possible that the DOS geom is less than Solaris.
			# Also, all sizes should be track-aligned.
			#
			if [ $STAT_DISKNUMSEC > $_soldevsize -a -z "$_nores" ]
			then
				_devsize=$_soldevsize
			else 
				_devsize=$STAT_DISKNUMSEC
			fi
			_devtrk=$STAT_SECT
			_devsize=`voladm_alignval down $_devsize $_devtrk`
			#
			# The first track of the disk in not used for the
			# extended partition nor is the first track of the
			# extended partition used for the public/private
			# paritions.
			#
			_res=`expr $_devtrk \* 2`
		
			if [ -z "$_nores" ]
			then
				if [ $_res -lt 256 ]
				then
					_res=`voladm_alignval up 256 $_devtrk`
				fi
				_privlen=`expr $_privlen + 256`
			fi
			if [ "sliced" = "$_fmt" ]
			then
				_privlen=`voladm_alignval up $_privlen $_devtrk`
				#
				# For sliced disks, the private and public
				# partitions must be separated by 1 track
				#
				_privlen=`expr $_privlen + $_devtrk`
			fi
		fi
		;;
	*)	return -1 ;;
	esac
	_pubsize=`expr $_devsize - $_privlen - $_res`
	eval "$1=$_pubsize"
	return 0
}
shuffle_fstab()
{
	shuf_file=$1
	shuf_dev=$2
	
	tmpfstab="/tmp/fstab_shuf.$$"
	origfstab="$SYSROOT/$VOL_VFSTAB_PATH"

	index=1
	while [ $index -le 15 ]; do
		newslc[$index]=$index
		index=$(( $index + 1 ))
	done

	exec 9<&0 < $shuf_file
	while read oslice nslice; do
		newslc[$oslice]=$nslice
	done
	exec <&9 9<&-

	# Has to be a cleaner way but I'm in a hurry :(
	if [ `dogi_is_mylex $shuf_dev` -ne 0 ]
	then
		shuf_dev=`echo $shuf_dev | sed -e s/\\\\//\\\\\\\\\\\\//g`p
	fi

	index=15
	while [ $index -ge 1 ]; do
		if [ $index -ne ${newslc[$index]} ]; then
			sed -e s/$shuf_dev${newslc[$index]}/$shuf_dev$index/g \
					< $origfstab >| $tmpfstab
			if is_selinux_enabled; then
				chcon --reference=$origfstab $tmpfstab
			fi
			/bin/cp -f $tmpfstab $origfstab
		fi
		index=$(( $index - 1 ))
	done
	/bin/rm -f $tmpfstab >/dev/null 2>&1
}

restore_fstab()
{
	shuf_file=$1
	shuf_dev=$2
	
	tmpfstab="/tmp/fstab_shuf.$$"
	origfstab="$SYSROOT/$VOL_VFSTAB_PATH"

	index=1
	while [ $index -le 15 ]; do
		newslc[$index]=$index
		index=$(( $index + 1 ))
	done

	exec 9<&0 < $shuf_file
	while read oslice nslice; do
		newslc[$oslice]=$nslice
	done
	exec <&9 9<&-

	# Has to be a cleaner way but I'm in a hurry :(
	if [ `dogi_is_mylex $shuf_dev` -ne 0 ]
	then
		shuf_dev=`echo $shuf_dev | sed -e s/\\\\//\\\\\\\\\\\\//g`p
	fi

	index=1
	while [ $index -le 15 ]; do
		if [ $index -ne ${newslc[$index]} ]; then
		  sed -e s/$shuf_dev$index/$shuf_dev${newslc[$index]}/g \
					< $origfstab >| $tmpfstab
		  if is_selinux_enabled; then
			chcon --reference=$origfstab $tmpfstab
		  fi
	  	  /bin/cp -f $tmpfstab $origfstab
		fi
		index=$(( $index + 1 ))
	done
	/bin/rm -f $tmpfstab >/dev/null 2>&1
}

find_shuffled_slice()
{
        shuf_file=$1
        inp_slc=$2
 
        exec 8<&0 < $shuf_file
        while read oslice nslice; do
                if [ $oslice -ne $nslice -a $nslice -eq $inp_slc ]; then
                        echo $oslice
                        return;
                fi
        done
        exec <&8 8<&-
        echo $inp_slc
        return
}
#
# add_cntrlr_expression -- add the given controller name to the
# expression to egrep out controller names. The second argument should
# be the variable name of the expression to be used.
#
add_cntrlr_expression()
{
	cntrlr=$1

	eval "expression=\${${2}}"

	if [ -z "$expression" ]
	then
		expression="${cntrlr}"
	else
		expression="${expression}|${cntrlr}"
	fi

	expression=`pipe_escape $expression`

	eval "${2}=$expression"

}

#This function returns the controller name of the disk passed as an argument
get_controller_of_disk()
{
	disk=$1
	ctlr_num=`$DIAGDIR/vxautoconfig | sed 's/ /:/g' | grep ":$disk:" | cut -d: -f4 | cut -d= -f2`
	echo "c$ctlr_num"
}

# Set boot loader type   
#   - set "bootloader" variable 
set_bootloader()
{
        bootloader=
        find_bootloader
	ret=$?
	if [ $ret -eq 1 ]; then
                bootloader="lilo"
        fi
        if [ $ret -eq 2 ]; then
                bootloader="grub"
	fi
	if [ $ret -eq 3 ]; then
		bootloader="elilo"
	fi
	if [ $ret -eq 4 ]; then
		bootloader="yaboot"
	fi
}

# this routine returns
#	1 for 'lilo'
#	2 for 'grub'
#	3 for 'elilo'
#	4 for 'yaboot'
#	-1 for unknown
find_bootloader()
{
	set_rootdisk

	boottmp="$SYSROOT/tmp/bootsect.$$"
	if [ -a ${VOL_DISK_BLKPATH}/$rootdisk ]; then
		dd if=${VOL_DISK_BLKPATH}/$rootdisk of=$boottmp count=1 >/dev/null 2>&1
	else
		if [ -a "${DISK_BLKPATH}/$rootdisk" ]; then
			dd if=${DISK_BLKPATH}/$rootdisk of=$boottmp count=1 >/dev/null 2>&1
		else
			return -1
		fi
	fi

	is_lilo=0; is_grub=0;

	is_grub=`grep -c "GRUB" $boottmp 2>&1`
	if [ $is_grub -ne 0 ]; then
		rm -f $boottmp >/dev/null 2>&1
		return 2
	fi

	is_lilo=`grep -c "LILO" $boottmp 2>&1`
	if [ $is_lilo -ne 0 ]; then
		rm -f $boottmp >/dev/null 2>&1
		return 1
	fi

	rm -f $boottmp >/dev/null 2>&1

	if [ -d /boot/efi -a -f /etc/elilo.conf ] ; then
		# Probably safe to assume elilo then
		return 3
	fi

	if [ -d /usr/lib/yaboot -a -f /etc/yaboot.conf ] ; then

		# Probably safe to assume yaboot then
		return 4
	fi

	# For ppc64 we're normally running on yaboot. However, SuSE hides this
	# behind a modified lilo installation, so we return '1' if we're ppc64
	# and SuSE. Callers have to explicitly check for ppc64 processor type

	if [ `uname -p` == "ppc64" ]; then
		if [ -f /etc/SuSE-release ]; then
			return 1
		fi
	fi

	return -1
}


voladm_disk_is_mounted()
{
	thisdisk=$1
	is_mylex=`dogi_is_mylex $thisdisk`
	if [ "X$is_mylex" = "X0" ]; then
		dogi_slice_blkpath $thisdisk diskpath
		grp_cnt=`/bin/mount | grep -c "$diskpath[0-9][0-9]*" 2>/dev/null`
		if [ $grp_cnt -eq 0 ] ; then
			grp_cnt=`/bin/mount | awk -v aname=$diskpath '$1 == aname { print 1;}'`
        		if [ "X$grp_cnt" != "X" ]; then
				return 0;
			else
				return 1;
			fi
		else
			return 0;
        	fi
	else
		dogi_slice_blkpath $thisdisk diskpath
		grp_cnt=`/bin/mount | grep -c "$diskpath[p][0-9][0-9]*" 2>/dev/null`
		if [ $grp_cnt -eq 0 ] ; then
			grp_cnt=`/bin/mount | awk -v aname=$diskpath '$1 == aname { print 1;}'`
        		if [ "X$grp_cnt" != "X" ]; then
				return 0;
			else
				return 1;
			fi
		else
			return 0;
        	fi
	fi
}

# return the offset of the public region of a given disk in pub_off 
get_pub_offset() {

	[ -z "$VOL_PUB_SLICE_TAG" ] && 	VOL_PUB_SLICE_TAG=0x7e
	disk=$1
	dogi_device_rawpath $disk rawpath
	$PRTVTOC -f $tmpfile1 $rawpath
	exec 3<&0 < $tmpfile1
	pub_off=0

	while read slice start size tag
	do
		if [ "$tag" = "$VOL_PUB_SLICE_TAG" ] ; then
			pub_off=$start
		fi
	done
	exec 0<&3 3<&-
}

#
# DMP:
# dmpdisk_to_subpath: Function to check which subpath was specified
# in the /etc/fstab and put that subpath correctly for the mount
# point entries.  If this information is not available from reconfig.d
# directory then use any of the active subpaths, hopefully system should
# work after vxunroot. If there is only one subpath for the dmp root
# device then use that directly. If not DMP device, use the name
# directly.
#

dmpdisk_to_subpath()
{
	dmpdisk=$1

	vxdisk list $dmpdisk  > $tmpfile1
	grep '^numpaths' $tmpfile1 > $tmpfile2 2> /dev/null

	if [ $? -ne 0 ]
	then
		eval "${2}=$dmpdisk"
		return 0
	fi

	exec 3<&0 < $tmpfile2
	read numpaths num
	exec 0<&3 3<&-

	if [ $num = 1 ]
	then
		eval "${2}=$dmpdisk"
		return 0
	fi

	vxdisk list $dmpdisk  > $tmpfile1
	grep "^c[0-9]" $tmpfile1 > $tmpfile2 2> /dev/null
	exec 3<&0 < $tmpfile2
	while read line; do
		subpath=`echo $line | cut -f 1 -d " "`
		state=`echo $line | cut -f 2 -d " "`
		if [ X"$state" = Xstate=enabled ]
		then
			break
		fi
	done
	exec 0<&3 3<&-
	
	if [ X"$subpath" != X ]
	then
		dogi_slice_to_device $subpath subpathdev
		eval "${2}=$subpathdev"
		return 0
	fi

	grep '^#rename' $DISKDIR/$dmpdisk/newpart > $tmpfile1 2> /dev/null
	if [ $? -eq 0 ]
	then
		exec 3<&0 < $tmpfile1
		read rename subpath voldmpdev
		exec 0<&3 3<&-
		dogi_slice_to_device $subpath subpathdev
		eval "${2}=$subpathdev"
		return 0
	fi

	eval "${2}=$dmpdisk"
	return 0
}

#
# PPC Lilo configuration file parsing support routines.
#
# find_liloppc_idx_entry:
# Check to see if there's a match in the LILO_CONF_FILE for the specified
# tagged entry
#	Arguments	$1	label to match
#			$2	tag to search on	(e.g. "label", "image")
# 	Returns
#		0	if no match found
#		index of entry in LILO_CONF_FILE otherwise
find_liloppc_idx_entry()
{
	ppcMatch=$1
	ppcTag=$2
	ret=`awk -F= -vMatch=$ppcMatch -vTag=$ppcTag ' {
		if (match($1,Tag)) {
			idx++;
			if (index($2, Match) != 0)
				found=idx;
			}
		}
		BEGIN {found=0;idx=0;}
		END { printf("%d\n", found); }' < $LILO_CONF_FILE`
	echo $ret
}

# find_liloppc_entry_idx:
#	Obtain the corresponding '<arg2> =' name for the given index (arg 1)
find_liloppc_entry_idx()
{
	ppcIndex=$1
	ppcTag=$2
	ret=`awk -F= -vIndex=$ppcIndex -vTag=$ppcTag ' {
		if (match($1,Tag)) {
			idx++;
			if (idx == Index)
				print $2;
			}
		}
		BEGIN {idx=0;}' < $LILO_CONF_FILE`
	echo $ret
}

#
# change_liloppc_entry_idx
#
# Change the tagged 'entry' with index 'idx' to the new 'value'
#
# Usage: change_liloppc_entry_idx root 2 /dev/sda3
#
# Parameters:
#	$1	entry tag (e.g. 'root')
#	$2	index of entry in file
#	$3	new value for entry
#
# Assumes LILO_CONF_FILE set to scanned file
change_liloppc_entry_idx()
{
	ppcTag=$1
	ppcIdx=$2
	ppcVal=$3

	awk -F= -vIndex=$ppcIdx -vTag=$ppcTag -vVal=$ppcVal '
		{ if (match($1, Tag)) {
			idx++;
			if (idx == Index) {
				n=gsub("[ 	]","",$1);
				printf("%s=%s\n", $1, Val);
				next;
			}
			}
		}
		{ print $0; }
		BEGIN {idx = 0;}' < $LILO_CONF_FILE
}

# dmp <-> alias name conversion routines for HPUX 11.31
# dogi_get_alias   - Return alias name for daname (new)
#                    taking into account the naming scheme
#                    and / or mode set
# dogi_get_dmpname - Return dmp name for daname (legacy)
#                    taking into account the naming scheme
#                    and / or mode set
dogi_get_alias()
{
        if [ $# -gt 1 ]
        then
                eval "${2}=${1}";
        else
                echo "$1";
        fi
}

dogi_get_dmpname()
{
        device=`voladm_basename $1`
        device=`/etc/vx/diag.d/vxdmpalias -d $device 2> /dev/null | \
        grep "Dmpname" | awk '{print $2}'`

        if [ $# -gt 1 ]
        then
                if [ -z "$device" ]
                then
                        eval "${2}=${1}";
                else
                        eval "${2}=${device}";
                fi
        else
                if [ -z "$device" ]
                then
                        echo "$1";
                else
                        echo "$device";
                fi
        fi
}

