#!/bin/sh
# $Id: skel.vras.sh,v 2.18.2.3 2009/05/07 18:51:33 vraichur Exp $
#ident "$Source: /project/vras-cvs/src/linux/cmd/vras/init.d/skel.vras.sh,v $ $Revision: 2.18.2.3 $"
#
# $Copyright: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#

### BEGIN INIT INFO
# Provides: vras-vradmind
# Should-Start:
# Required-Start: vxvm-boot
# Required-Stop:
# Default-Start:  2 3 4 5
# Default-Stop:
# Description:
### END INIT INFO

# Volume Replicator Administrative Services (VRAS) Daemon Script 
# start up / shut down vradmind daemon
# Starts vradmind daemon only if VVR license is installed

if [ ! -d /usr/sbin ]
then					# /usr not mounted
	echo "/usr not mounted"
	quit
fi

. ${VOLADM_LIB:-/usr/lib/vxvm/voladm.d/lib}/vxadm_lib.sh

progname=$0
daemon="vradmind"

#
# All environment variables used by vradmind are defined in this file
#
ENVFILE=/etc/vx/vras/vras_env

findproc() {		# return pid of the named process(es)
	pid=`/bin/ps -e -o pid,comm |
		/bin/grep " $1" | /bin/grep -v grep |
		/bin/awk '{ print $1 }'`
}

check_vvr_license() { 		# return 0 if has vvr license 
							# and return 1 otherwise
	/sbin/vxlictest -n "VERITAS Volume Manager" -f "VVR" >/dev/null 2>&1
	if [ "$?" -eq 0 ]; then
		return 0
	else
		return 1
	fi
}

case "$1" in
'start')		
	# check for vvr license
	/sbin/vxlictest -n "VERITAS Volume Manager" -f "VVR" >/dev/null 2>&1
	if [ $? != 0 ]; then
		#initlog -n "VRAS daemon" -s "N/A"
		export dname=VRAS; ugettxt -n "VVR" "$dname daemon              N/A" vras.mesg:2414
			quit 1
	fi

	# make sure file install-db does not exist
	if [ -f /etc/vx/reconfig.d/state.d/install-db ]; then
		export daemon; ugettxt -n "VVR" -s "INFO" \
			"VM installation not complete; $daemon not started." vras.mesg:2410
		quit 1
	fi

	# check if vradmind is already running
	start_vradmind=1
	findproc vradmind;
	if [ "X$pid" != "X" ]; then
		start_vradmind=0
	fi

	# check if vvr_stats is already running
	start_stats=1
	findproc vvr_stats
	if [ "X$pid" != "X" ]; then
		start_stats=0
	fi

	if [ "$start_vradmind" -eq 1 ]; then 
		# source the environment variables defined in ENVFILE
		if [ -f ${ENVFILE} ]; then
			. ${ENVFILE}
		fi

		# start vradmind
		/usr/sbin/vradmind 
		#initlog -n "VRAS daemon" -s "Started"
		export dname=VRAS; ugettxt -n "VVR" "Starting $dname daemon: [  OK  ]" vras.mesg:2415
	fi

	if [ "$start_stats" -eq 1 ]; then 
		#start vvr_stats
		/etc/vx/vras/templates/vvr_stats 
	fi
	;;
'stop')
	findproc vradmind;
	if [ "X$pid" != "X" ]; then
		/bin/kill -15 $pid
	fi
	findproc vvr_stats;
	if [ "X$pid" != "X" ]; then
		/bin/kill -15 $pid
	fi
	;;
'restart')
	$0 stop
	$0 start
	;;
*)
	export progname; ugettxt -n "VVR" -s "INFO" \
			"Usage: $progname { start | stop | restart }" vras.mesg:0
	quit 1
	;;
esac

quit 0
