#!/bin/sh
#
# $Id: setup.sh,v 1.32.6.2 2008/07/07 14:17:21 prallabh Exp $
# $Copyright: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
# 

######################################################################
# Default input variables:
#
# Use gridtrace="yes" to turn on provider stdout debugging in registry.
#
: ${gridtrace:="no"}
#
######################################################################

OBC="$PKG_INSTALL_ROOT/opt/VRTSobc"
PAL_VERSION=pal33
OBC_ETC="$PKG_INSTALL_ROOT/opt/VRTSobc/$PAL_VERSION/etc"
GRID_INSTALLPATH="$PKG_INSTALL_ROOT/opt/VRTSccg"
GRID_ETC="$GRID_INSTALLPATH/etc"
GRID_LIB="$GRID_INSTALLPATH/lib"
GRID_CONFIG="$GRID_INSTALLPATH/config"

# commands
VXOBJECTTYPE="$OBC/$PAL_VERSION/bin/vxobjecttype"
VXSECAZ="$OBC/$PAL_VERSION/bin/vxsecaz"
VXINTERFACE="$OBC/$PAL_VERSION/bin/vxinterface"
REGOPS="$PKG_INSTALL_ROOT/opt/VRTSobc/$PAL_VERSION/bin/vxregctl"
REGFILE="$PKG_INSTALL_ROOT/etc/vx/isis/Registry"
VXPAL="$OBC/$PAL_VERSION/bin/vxpal" 

#exit error codes
SYSTEM_SECCONFIG_ERROR=2
GRIDAGENT_SECCONFIG_ERROR=3
GRIDAGENT_SECCONFIG_ALREADY_EXISTS=4
GRID_AGENT_START_ERROR=5

BASEDIR=$OBC/$PAL_VERSION
export BASEDIR

mkdir -p /var$GRID_ETC
touch /var$GRID_ETC/nodedomain.cf

#--------------------
# unbackup cf files
#--------------------
if [ -f $ETCDIR/nodedomain.cf.bkp ]; then
  mv $ETCDIR/nodedomain.cf.bkp $ETCDIR/nodedomain.cf
fi
if [ -f $ETCDIR/centraldomain.cf.bkp ]; then
  mv $ETCDIR/centraldomain.cf.bkp $ETCDIR/centraldomain.cf
fi


#--------------------
# set up xml files  	 
# -------------------- 	 
# WPAR compliance check #
PLAT=`uname -s`
if [ $PLAT != "AIX" ]
then
cp $OBC/$PAL_VERSION/etc/objecttypes.dtd $GRID_ETC 	 
$VXOBJECTTYPE -i $OBC_ETC/objecttypes.xml -o $GRID_ETC/gridtypes.xml -m $GRID_CONFIG/gridtypes.xml 	 

cp $OBC/$PAL_VERSION/etc/interfaceform.dtd $GRID_ETC 	 
$VXINTERFACE -i $OBC_ETC/interfaceform.xml -o $GRID_ETC/gridintf.xml -m $GRID_CONFIG/gridintf.xml 	 

cp $OBC/$PAL_VERSION/etc/securityform.dtd $GRID_ETC 	 
$VXSECAZ -i $OBC_ETC/securityform.xml -o $GRID_ETC/gridsec.xml -m $GRID_CONFIG/gridsec.xml 	 
fi # WPAR compliance check #  	 
#--------------------
# 
#--------------------

queryreg()
{
  key=$1
  name=$2
  raw=`$REGOPS $REGFILE queryvalue $key $name 2>/dev/null`
  result=`echo $raw |sed -e 's/^.*: //g'`
  echo $result
}

#--------------------
# set up registry
#--------------------
setupregistry()
{
  gridagent=$1
  PORT=$2
  shortname=$3

  DOI=`date +%D`

  echo "        setting up registry for $gridagent"

  AGENTS=Software/VERITAS/VRTSobc/pal33/Agents
  $REGOPS $REGFILE createkey $AGENTS/$gridagent/ 2>/dev/null
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/ InstallDir REG_SZ $GRID_INSTALLPATH 2>/dev/null
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/ Port REG_INT $PORT 2>/dev/null
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/ Company  REG_SZ "Symantec Corporation"  2>/dev/null
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/ Copyright  REG_SZ "2008"  2>/dev/null
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/ Classification  REG_SZ "Gridnode"  2>/dev/null
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/ Description  REG_SZ "Veritas Grid Node"  2>/dev/null
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/ UmiID  REG_SZ ""  2>/dev/null
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/ PSTAMP  REG_SZ ""  2>/dev/null
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/ InstallDate  REG_SZ $DOI  2>/dev/null

  $REGOPS $REGFILE createkey $AGENTS/$gridagent/ClientExtensions/ 2>/dev/null
  $REGOPS $REGFILE createkey $AGENTS/$gridagent/MessageCatalogs/ 2>/dev/null
  $REGOPS $REGFILE createkey $AGENTS/$gridagent/Groups/ 2>/dev/null
  $REGOPS $REGFILE createkey $AGENTS/$gridagent/ProviderGroup/ 2>/dev/null
  $REGOPS $REGFILE createkey $AGENTS/$gridagent/Providers/ 2>/dev/null
  $REGOPS $REGFILE createkey $AGENTS/$gridagent/Providers/$gridagent/Registration/ 2>/dev/null
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/Providers/$gridagent/Registration/ Type REG_SZ Agent_Provider 2>/dev/null
  $REGOPS $REGFILE createkey $AGENTS/$gridagent/Providers/LeafRegistration/ 2>/dev/null
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/Providers/LeafRegistration/ IMAGEFILE REG_SZ /opt/VRTSobc/pal33/lib/libleafregn.$SHLIB_EXT 2>/dev/null
  $REGOPS $REGFILE createkey $AGENTS/$gridagent/Registration/ 2>/dev/null
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/Registration/ Type REG_SZ Agent 2>/dev/null
  $REGOPS $REGFILE createkey $AGENTS/$gridagent/SecuredManagers/ 2>/dev/null
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/SecuredManagers/ ManagerList REG_SZ $GRID_ETC/"$shortname"domain.cf 2>/dev/null
  ###$REGOPS $REGFILE createkey $AGENTS/$gridagent/Tracing/ 2>/dev/null
  ###$REGOPS $REGFILE createkey $AGENTS/$gridagent/Tracing/vxisis/ 2>/dev/null

  ### provider entries
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/Providers/$gridagent Imagefile REG_SZ $GRID_LIB/libgridprov.$SHLIB_EXT
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/Providers/$gridagent debug REG_SZ $gridtrace
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent/Providers/$gridagent persistdir REG_SZ /var/opt/VRTSccg/$shortname

  ### xml files
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent ObjectTypeFILE REG_SZ $GRID_ETC/gridtypes.xml
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent SecurityFormFile REG_SZ $GRID_ETC/gridsec.xml
  $REGOPS $REGFILE setvalue $AGENTS/$gridagent InterfaceFormFile REG_SZ $GRID_ETC/gridintf.xml


  $REGOPS $REGFILE createkey $AGENTS/$gridagent/Network 2>/dev/null
  $REGOPS $REGFILE createkey $AGENTS/$gridagent/Network/Params 2>/dev/null
  $REGOPS $REGFILE createkey $AGENTS/$gridagent/Network/Params/Security 2>/dev/null
}

#--------------------
# set up security
#--------------------

setupsecurity()
{
  gridagent=$1

  defmode=`queryreg Software/VERITAS/VRTSobc/$PAL_VERSION/Agents/__defaults Mode`
  if [ "$defmode" != "2" ]
  then
    return 0 # running on host controller mode
  fi

  echo "        setting up security for $gridagent"

  SECKEY=Software/VERITAS/VRTSobc/$PAL_VERSION/Agents/__defaults/Network/Params/Security

  sec=`queryreg $SECKEY VXPALSECURITY`
  hostname=`queryreg $SECKEY LocalSystem`
  dc=`queryreg $SECKEY DomainController`
  ab=`queryreg $SECKEY AuthenticationBroker`

  if [ "$hostname" = "" -o \
       "$dc" = "" -o\
       "$ab" = "" ]
  then
       echo "Cannot configure $gridagent security.  Use cfgsecurity.sh by hand."
       return $SYSTEM_SECCONFIG_ERROR
  fi

  #----

  if [ "$sec" = "y" -a \
       "$AGENTPASSWORD" = "" -a \
       "$SECURITYADMINPASSWORD" = "" ]
  then
    # These are collected in request script (on solaris only)
    # On linux and when this script is run by hand, we need to ask.
    # Given fixes in security code, we should not need to ask for
    # passwords at all.
    echo ""
    echo "Enter Security administrator password: \c"
    read SECURITYADMINPASSWORD
    echo ""
    echo "Enter password for the grid agents: \c"
    read AGENTPASSWORD

    # defaults
    if [ "X$SECURITYADMINPASSWORD" = "X" ]
    then
        SECURITYADMINPASSWORD=Vxadmin
        echo "Default password being used, please change it later.";
    fi
    if [ "X$AGENTPASSWORD" = "X" ]
    then
        AGENTPASSWORD=veritas
    fi
  fi

  #----

  if [ "X$PKG_INSTALL_ROOT" = "X" -a \
       "$UPDATE" != "yes" -a \
       "$sec" = "y" -a \
       "X$AGENTPASSWORD" != "X" -a \
       "X$SECURITYADMINPASSWORD" != "X" ]
  then
        cmd="$OBC/$PAL_VERSION/install/cfgsecurity.sh -a $gridagent -h $hostname -d $dc -p $AGENTPASSWORD -b $ab -s $SECURITYADMINPASSWORD" 
        echo "Running: $cmd"
        $cmd
        if [ $? -ne 0 ]
        then
           echo "Security configuration for $gridagent agent failed"
           return $GRIDAGENT_SECCONFIG_ERROR
        fi
  else
        echo
        echo "Assuming security is already configured.  If not, run this:"
        echo "  $OBC/$PAL_VERSION/install/cfgsecurity.sh -a $gridagent -h $hostname -d $dc -p $AGENTPASSWORD -b $ab -s $SECURITYADMINPASSWORD"
        echo
        return $GRIDAGENT_SECCONFIG_ALREADY_EXISTS
  fi

  return 0
}

install_rcfiles()
{
  gridagent=$1

  echo "        setting up startup files for $gridagent"

  case $osname in
    "SunOS") rcdir=/etc;;
    "HP-UX") rcdir=/sbin;;
    "AIX")   rcdir=/etc/rc.d;;
    "Linux") rcdir=/etc;;
    *) rcdir=/etc;;
  esac

  script_name=$PKG_INSTALL_ROOT/$rcdir/init.d/vxpal.$gridagent
  Sscript_name=$PKG_INSTALL_ROOT/$rcdir/rc2.d/S750vxpal.$gridagent
  Kscript_name=$PKG_INSTALL_ROOT/$rcdir/rc2.d/K750vxpal.$gridagent
  Kscript_name_rc1=$PKG_INSTALL_ROOT/$rcdir/rc1.d/K750vxpal.$gridagent
  Kscript_name_rc0=$PKG_INSTALL_ROOT/$rcdir/rc0.d/K750vxpal.$gridagent
  Kscript_name_rcS=$PKG_INSTALL_ROOT/$rcdir/rcS.d/K750vxpal.$gridagent

  cp $GRID_CONFIG/start$gridagent.sh $script_name

  chown root:sys $script_name
  chmod 0744 $script_name
  ln -s $script_name $Sscript_name >/dev/null 2>&1
  ln -s $script_name $Kscript_name >/dev/null 2>&1
  if [ $osname != "HP-UX" ]
  then
     ln -s $script_name $Kscript_name_rc1 >/dev/null 2>&1
     ln -s $script_name $Kscript_name_rc0 >/dev/null 2>&1
     ln -s $script_name $Kscript_name_rcS >/dev/null 2>&1
  fi

  if [ $osname = "AIX" ]
  then
    lsitab vxpal.$gridagent >/dev/null 2>&1
    result=$?
    if [ "X$result" != "X0" ]
    then
      mkitab vxpal.$gridagent:2:once:$script_name
    fi
  elif [ $osname = "Linux" ]
  then
      /sbin/chkconfig --add vxpal.$gridagent
      if [ $? -eq 1 ]
      then
        echo "Unable to update runlevel information for vxpal.$gridagent"
      fi
  fi
}

startvxpal()
{
  gridagent=$1
  if [ "X$PKG_INSTALL_ROOT" = "X" ]
  then
    echo "Starting $gridagent"
    $VXPAL -a $gridagent
    if [ $?  != 0 ]
    then
        return $GRID_AGENT_START_ERROR
    fi
  fi
}


#--------------------
# main
#--------------------
usage()
{
  echo "usage: $0 --gridnode [--start]"
  echo "usage: $0 --gridcentral [--start]"
  exit 1
}

installgridcentral=NO
installgridnode=NO
startagent=NO
cpi=NO

while [ $# -ge 1 ]
do
  case $1 in
    "--cpi") cpi=YES;; 
    "--gridnode") installgridnode=YES;;
    "--gridcentral") installgridcentral=YES;;
    "--start") startagent=YES;;
    *) usage ;;
  esac
  shift
done

#--------------------
# get runtime variables
#--------------------
osname=`uname`
if [ $osname = "HP-UX" ]
then
  SHLIB_EXT=sl
else
  SHLIB_EXT=so
fi

if [ $installgridnode = NO -a $installgridcentral = NO ]
then
  usage
else
    ####
    #### gridnode configuration main
    ####
    if [ $installgridnode = YES ]
    then
      setupregistry gridnode 1556 node
      if [ $cpi = NO ]
      then
          setupsecurity gridnode
          rc=$?
          if [ $rc != 0 ]
          then
              exit $rc
          fi
      fi
      install_rcfiles gridnode
      if [ $startagent = YES ]
      then
        startvxpal gridnode
      fi
    fi

    ####
    #### gridcentral configuration main
    ####
    if [ $installgridcentral = YES ]
    then
      setupregistry gridcentral 1556 central
      if [ $cpi = NO ]
      then
          setupsecurity gridcentral
          rc=$?
          if [ $rc != 0 ]
          then
              exit $rc
          fi
      fi
      install_rcfiles gridcentral
      if [ $startagent = YES ]
      then
        startvxpal gridcentral
      fi
    fi
fi

exit 0

