#!/bin/sh

# $Id: unsetup.sh,v 1.33 2008/01/16 11:44:19 prallabh Exp $
# $Copyright: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $

JUMPSTART_PATH="/"
if [ "X$PKG_INSTALL_ROOT" != "X" ]
then
    JUMPSTART_PATH="$PKG_INSTALL_ROOT"
fi

GRID_ETC=$PKG_INSTALL_ROOT/opt/VRTSccg/etc
VXPALCTRL=$PKG_INSTALL_ROOT/opt/VRTSobc/pal33/bin/vxpalctrl
VEACONFIG=$PKG_INSTALL_ROOT/opt/VRTSobc/pal33/bin/veaconfig
REGOPS="$PKG_INSTALL_ROOT/opt/VRTSobc/pal33/bin/vxregctl"
REGFILE="$PKG_INSTALL_ROOT/etc/vx/isis/Registry"
AGENTS=Software/VERITAS/VRTSobc/pal33/Agents
RM=/bin/rm
RMDIR=/bin/rmdir
OSNAME=`uname`

####

queryreg()
{
  key=$1
  name=$2
  raw=`$REGOPS $REGFILE queryvalue $key $name 2>/dev/null`
  result=`echo $raw |sed -e 's/^.*: //g'`
  echo $result
}

leavedomDRD()
{
    gridagent=$1
    msg NOTE "The vxsvc will be informed that the $gridagent is no longer available when the inactive system image is booted.."
    initfile=/sbin/init.d/xgridrcdrdLEAVE
    rcfile=/sbin/rc2.d/S999xgridrcdrdLEAVE
    rm -rf $rcfile $initfile

    print "#!/sbin/sh" > $initfile
    print "$VEACONFIG -c leavedc -a $gridagent -u vea_agent@vea_domain.vx --no-prompt"  >> initfile
    print "/usr/bin/rm -rf $initfile $rcfile " >> $initfile
    print "    exit \$? "  >> $initfile

    chmod 555 $initfile
    chown bin:bin $initfile
    ln -s $initfile $rcfile

}

leavedom()
{
  gridagent=$1
  gridmode=`queryreg $AGENTS/$gridagent/Providers/$gridagent Mode`
  if [ "$gridmode" = 1 ]
  then
      $VEACONFIG -b $JUMPSTART_PATH -c leavedc -a $gridagent --no-prompt 2>/dev/null
  else
      if [ "$gridmode" = 2 ]
      then
         $VEACONFIG -b $JUMPSTART_PATH -c leavedc -a $gridagent -u vea_agent@vea_domain.vx --no-prompt
      else
         gridmode=`queryreg $AGENTS/__defaults Mode`
         if [ "$gridmode" = 1 ]
         then
            $VEACONFIG -b $JUMPSTART_PATH -c leavedc -a $gridagent --no-prompt 2>/dev/null
         else
            $VEACONFIG -b $JUMPSTART_PATH -c leavedc -a $gridagent -u vea_agent@vea_domain.vx --no-prompt
         fi
      fi
  fi
}


stopagent()
{
  gridagent=$1
  $VXPALCTRL -a $gridagent -c stop
}

cleanreg()
{
  gridagent=$1
  $REGOPS $REGFILE deletekeyrecursive $AGENTS/$gridagent 2>/dev/null
}

cleandirs()
{
  gridagent=$1

  griddir=`queryreg $AGENTS/$gridagent/Providers/$gridagent persistdir`
  if [ "X$griddir" = X ]
  then
    return
  fi
  $RM -rf $griddir/requests
  $RM -rf $griddir/results
  $RM -rf $griddir/priv
  $RM -rf $griddir/files
  $RM -rf $griddir/tmp
  $RMDIR $griddir
}

clean_rcfiles()
{
  gridagent=$1

  osname=`uname`

  case $osname in
    "SunOS") rcdir=/etc;;
    "HP-UX") rcdir=/sbin;;
    "AIX")   rcdir=/etc/rc.d;;
    "Linux") rcdir=/etc;;
    *) rcdir=/etc;;
  esac

  if [ $osname = 'Linux' ] 
  then
     /sbin/chkconfig --del vxpal.$gridagent > /dev/null
  fi

  $RM -f $rcdir/init.d/vxpal.$gridagent
  $RM -f $rcdir/rc2.d/S750vxpal.$gridagent
  $RM -f $rcdir/rc2.d/K750vxpal.$gridagent
  $RM -f $rcdir/rc1.d/K750vxpal.$gridagent
  $RM -f $rcdir/rc0.d/K750vxpal.$gridagent
  $RM -f $rcdir/rcS.d/K750vxpal.$gridagent

  if [ $osname = 'AIX' ]
  then
    rmitab vxpal.$gridagent
  fi
}

clean_xmlfiles()
{
  nodeisinstalled=`queryreg $AGENTS/gridnode/Providers/gridnode Imagefile`
  centralisinstalled=`queryreg $AGENTS/gridcentral/Providers/gridcentral Imagefile`

  # only clean the shared xml files if both gridcentral and gridnode are uninstalled
  if [ "$nodeisinstalled" = ""  -a "$centralisinstalled" = "" ]
  then
    $RM -f $GRID_ETC/securityform.dtd
    $RM -f $GRID_ETC/interfaceform.dtd
    $RM -f $GRID_ETC/objecttypes.dtd
    $RM -f $GRID_ETC/gridtypes.xml
    $RM -f $GRID_ETC/gridintf.xml
    $RM -f $GRID_ETC/gridsec.xml
  fi
}

clean_cffiles()
{
  cfname=$1

  $RM -f $GRID_ETC/"$cfname"domain.cf
  $RM -f $GRID_ETC/"$cfname"domain.cf.Tmp
}

#--------
# main
#--------
usage()
{
  echo "usage: $0 --gridnode"
  echo "usage: $0 --gridcentral"
  exit 1
}

uninstallgridnode=NO
uninstallgridcentral=NO

while [ $# -ge 1 ]
do
  case $1 in
    "--cpi") ;; # ignored
    "--gridnode") uninstallgridnode=YES;;
    "--gridcentral") uninstallgridcentral=YES;;
    *) usage ;;
  esac
  shift
done

if [ $uninstallgridnode = NO -a $uninstallgridcentral = NO ]
then
  usage
else
    ####
    #### gridnode remove configuration main
    ####
    if [ $uninstallgridnode = YES ]
    then
        if [ $OSNAME = 'HP-UX' ]
        then
	    if [[ $SW_SESSION_IS_DRD -ne 1 ]]
	    then
                stopagent gridnode
                leavedom gridnode
            else
                leavedomDRD  gridnode
            fi
        else
           leavedom gridnode
	fi
        cleandirs gridnode
        cleanreg gridnode
        clean_rcfiles gridnode
        clean_xmlfiles 
        clean_cffiles node
    fi

    ####
    #### gridcentral remove configuration main
    ####
    if [ $uninstallgridcentral = YES ]
    then
        if [ $OSNAME = 'HP-UX' ]
        then
	    if [[ $SW_SESSION_IS_DRD -ne 1 ]]
	    then
                stopagent gridcentral
                leavedom gridcentral
            else
                leavedomDRD  gridcentral      
            fi
        else
            leavedom gridcentral
	fi
        cleandirs gridcentral
        cleanreg gridcentral
        clean_rcfiles gridcentral
        clean_xmlfiles 
        clean_cffiles central
    fi
fi
exit 0

