/* $Id: fcl.h,v 4.19 2006/01/20 22:07:09 bhim Exp $  */
/* #ident "@(#)vxfs:$RCSfile: fcl.h,v $	$Revision: 4.19 $" */

/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#ifndef _FCLHDR_H
#define _FCLHDR_H

/*
 *  fcl.h -- Exposed File Change Log (FCL) structures, types & defines
 */

/*
 * The FCL full filename is "/<mntpt>/lost+found/changelog".  The
 * directory name and filename are hard-coded here.
 */

#define VX_FCL_NAME	"changelog"
#define VX_FCL_DIR	"lost+found"

/*
 * The FCL magic number is used to determine that the superblock
 * at the start of the FCL file is an FCL file.
 */

#define VX_FCL_MAGIC	0xa506fcf5

/*
 * The FCL version number.  This allows the layout of the FCL, including
 * the FCL superblock, FCL record, etc. to change and gives applications
 * a method of determining these changes.
 */

#define VX_FCL_VERSION3		3

#define VX_FCL_VERSION		VX_FCL_VERSION3

/*
 * fcl_chgtype_t -- file change types
 */

typedef enum {
	VX_FCL_NO_CHANGE,		/* no change */
	VX_FCL_INFO_CHG,		/* file info change */
	VX_FCL_CREATE,			/* file create */
	VX_FCL_LINK,			/* file link added */
	VX_FCL_UNLINK,			/* file unlink / file deleted */
	VX_FCL_RENAME,			/* file rename */
	VX_FCL_UNDELETE,		/* file undelete */
	VX_FCL_DATA_EXTNDWRITE,		/* file data extending write */
	VX_FCL_DATA_OVERWRITE,		/* file data overwrite */
	VX_FCL_DATA_TRUNCATE,		/* file data truncate */
	VX_FCL_EATTR_CHG,		/* file extended attribute change */
	VX_FCL_HOLE_PUNCHED,		/* file hole punched */
	VX_FCL_HEADER,			/* block header record */
	VX_FCL_SYMLINK,			/* symbolic link created */
	VX_FCL_INOEX_CHG,		/* inode extent attributes changed */
	VX_FCL_INORES_CHG,		/* inode reservation changed */
	VX_FCL_IMODE_CHG,		/* inode mode changed. */
	VX_FCL_IOWN_CHG,		/* inode owner changed. */
	VX_FCL_IGRP_CHG,		/* inode grp changed. */
	VX_FCL_ITIMES_CHG,		/* inode times changed. */
	VX_FCL_MTIME_CHG,		/* inode mtime changed. */
	VX_FCL_V3_MAX,			/* maximum FCL V3 type + 1 */
	VX_FCL_ACCESSINFO,		/* User, pid, Node information. */
	VX_FCL_FILEOPEN,		/* File Open record. */
	VX_FCL_FILESTATS,		/* File READ/WRITE stats. */
	VX_FCL_EVNTMSK_CHG,		/* Fcl Event mask change. */
	VX_FCL_FILEINFO_CHG,		/* Change in file size/blocks */
	VX_FCL_RENAME_NEW,		/* New file name in a rename */
	VX_FCL_V4_MAX			/* maximum FCL type + 1 */
} fcl_chgtype_t;

#define VX_FCL_MAX			VX_FCL_V4_MAX

/*
 * fcl_state_t -- FCL state can be either activated or deactivated
 */

typedef enum { VX_FCLS_OFF, VX_FCLS_ON } fcl_state_t;

/*
 * fcl_timeval32_t -- the FCL 32-bit timeval structure.  We need an 8-byte
 * timeval structure that is consistent across all platforms and between
 * 32-bit and 64-bit applications.  tv_sec and tv_nsec are the number of
 * seconds since the Epoch (00:00:00 UTC, January 1, 1970).  Note that
 * they are both unsigned 32-bit integers.
 */

typedef struct fcl_timeval32 {
	uint32_t	tv_sec;
	uint32_t	tv_nsec;
} fcl_timeval32_t;

/*
 * fcl_ent -- the FCL entry structure.  A file change record details
 * the type of change, the inode number and inode generation count for
 * the file, and the time the change occurred.  When the record type
 * is a rename or unlink the inode number for the directory containing
 * the file being changed is stored in fe_dino and the length of the
 * trailing file name is recorded in fe_nmlen.
 *
 * All fcl_ent entries start on a sizeof (struct fcl_ent) boundary.
 */

struct fcl_ent {
	uint64_t	fe_ino;		/* 0x00 inode number */
	uint64_t	fe_dino;	/* 0x08 inode number */
	fcl_timeval32_t	fe_time;	/* 0x10 timestamp */
	uint32_t	fe_gen;		/* 0x18 inode gen count */
	uint16_t	fe_type;	/* 0x1c change type */
	uint16_t	fe_nmlen;	/* 0x1e filename length */
					/* 0x20 is length */
};

/*
 * fcl_sb -- the FCL on-disk superblock.  The superblock will always
 * be written to the first block of the FCL file, at offset 0.  The
 * first block of the FCL file contains only the FCL superblock.
 * FCL records initially begin from the offset of the second block.
 *
 * fc_magic	The magic number which identifies an FCL file.
 *
 * fc_version	The FCL version number.
 *
 * fc_state	The current state of the FCL, which is either
 *		activated (VX_FCLS_ON) or deactivated (VX_FCLS_OFF).
 *
 * fc_sync	The sync count of the FCL.  This value is incremented
 *		by the VX_FCL_SYNC ioctl.
 *
 * fc_atime	The activation time of the FCL.  This is used by
 *		applications to determine whether this instance of
 *		the FCL is the same as the last time the FCL was
 *		read by the application.
 *
 * fc_foff	The first valid offset into the FCL file containing
 *		an FCL record.  This offset is derived by multiplying
 *		the block number in the file by the file system block
 *		size.  fc_foff is initially set to the offset of the
 *		second block of the FCL file, but as holes are punched
 *		in the beginning of the FCL file this offset will be
 *		updated to point to the offset of the first block after
 *		the hole.  The first block of the FCL file is always
 *		reserved for the FCL superblock (fcl_sb).
 *
 * fc_loff	The last valid offset into the FCL file.
 * fc_oldfile	The filename relative to root where the previous FCL
 *		file is stored. Currently not in use. This field is
 *		reserved for future use. When FCL file size reaches MAX
 *		file size supported, contents of the previous FCL are
 *		copied to fc_oldfile and the current file starts logging
 *		afresh. In this case the activation timestamp need not
 *		be reset.

 * fc_eventmask List of events being recorded in FCL. It is a 128 bit
 *	        entity but currently only 64 of these are going to be used.
 *
 * fc_emtime	Time at which the list of events getting recorded was last
 *		changed. If this time is > activation time, applications might
 *		want to scan FCL for these records?
 */

struct fcl_sb {
	uint32_t	fc_magic;	/* 0x00 FCL magic number */
	uint32_t	fc_version;	/* 0x04 FCL version number */
	fcl_state_t	fc_state;	/* 0x08 FCL state */
	uint32_t	fc_sync;	/* 0x0c FCL sync count */
	fcl_timeval32_t	fc_atime;	/* 0x10 FCL activation time */
	uint64_t	fc_foff;	/* 0x18 first valid offset in FCL file*/
	uint64_t	fc_loff;	/* 0x20 last valid offset in FCL file */
	char		fc_oldfile[64]; /* 0x60 previous fcl file.
						Current not used. */
	uint64_t	fc_eventmask[2]; /* 0x68 Event Mask. */
	fcl_timeval32_t	fc_emtime;	/* 0x78 FCL event mask change time. */
					/* 0x80 is length */
};

/*
 * Definitions for masks which can be used to determine if a particular event
 * is being tracked. These masks can also be used to construct the eventmask
 * parameter to the FCL API function- vxfs_fcl_read. This event-mask specifies
 * the subset of FCL records the user wishes to read.
 */
#define	VX_FCL_CREATE_MASK		(1 << VX_FCL_CREATE)
#define	VX_FCL_LINK_MASK		(1 << VX_FCL_LINK)
#define	VX_FCL_UNLINK_MASK		(1 << VX_FCL_UNLINK)
#define	VX_FCL_RENAME_MASK		(1 << VX_FCL_RENAME)
#define	VX_FCL_UNDELETE_MASK		(1 << VX_FCL_UNDELETE)
#define	VX_FCL_DATA_EXTNDWRITE_MASK	(1 << VX_FCL_DATA_EXTNDWRITE)
#define	VX_FCL_DATA_OVERWRITE_MASK	(1 << VX_FCL_DATA_OVERWRITE)
#define	VX_FCL_DATA_TRUNCATE_MASK	(1 << VX_FCL_DATA_TRUNCATE)
#define	VX_FCL_EATTR_CHG_MASK		(1 << VX_FCL_EATTR_CHG)
#define	VX_FCL_HOLE_PUNCHED_MASK	(1 << VX_FCL_HOLE_PUNCHED)
#define	VX_FCL_SYMLINK_MASK		(1 << VX_FCL_SYMLINK)
#define	VX_FCL_INOEX_CHG_MASK		(1 << VX_FCL_INOEX_CHG)
#define	VX_FCL_INORES_CHG_MASK		(1 << VX_FCL_INORES_CHG)
#define	VX_FCL_IMODE_CHG_MASK		(1 << VX_FCL_IMODE_CHG)
#define	VX_FCL_IOWN_CHG_MASK		(1 << VX_FCL_IOWN_CHG)
#define	VX_FCL_IGRP_CHG_MASK		(1 << VX_FCL_IGRP_CHG)
#define	VX_FCL_ITIMES_CHG_MASK		(1 << VX_FCL_ITIMES_CHG)
#define	VX_FCL_MTIME_CHG_MASK		(1 << VX_FCL_MTIME_CHG)
#define	VX_FCL_ACCESSINFO_MASK		(1 << VX_FCL_ACCESSINFO)
#define	VX_FCL_FILEOPEN_MASK		(1 << VX_FCL_FILEOPEN)
#define	VX_FCL_FILESTATS_MASK		(1 << VX_FCL_FILESTATS)
#define	VX_FCL_EVNTMSK_CHG_MASK		(1 << VX_FCL_EVNTMSK_CHG)
#define	VX_FCL_FILEINFO_CHG_MASK	(1 << VX_FCL_FILEINFO_CHG)
#define	VX_FCL_RENAME_NEW_MASK		(1 << VX_FCL_RENAME_NEW)

#define FCL_ALL_V4_EVENTS  (((uint64_t)1 << VX_FCL_V4_MAX) - 1)

#endif /* _FCLHDR_H */
