/* $Id: vxfsutil.h,v 4.51.14.5.2.1 2008/07/12 15:29:43 markhe Exp $  */
/* #ident "@(#)vxfs:$RCSfile: vxfsutil.h,v $	$Revision: 4.51.14.5.2.1 $" */

/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#ifndef	_VXFSUTIL_H
#define	_VXFSUTIL_H

#ifdef __cplusplus
extern "C" {
#endif

#include <time.h>

#ifndef STATIC
#define STATIC	static
#endif

#define	NAMELEN			256

#define MNT_LOCAL		0x01
#define MNT_PRIMARY		0x02
#define MNT_SECONDARY		0x04

/*
 * Fields within vx_inode which can be externally visible
 */

struct vx_ioffsets {
	unsigned int	ioff_dev;	/* offset of i_dev within vx_inode */
	unsigned int	ioff_dev_sz;	/* size of i_dev field in bytes */
	unsigned int	ioff_number;	/* offset of i_number within vx_inode */
	unsigned int	ioff_number_sz;	/* size of i_number field in bytes */
	unsigned int	ioff_nlink;	/* offset of i_nlink within vx_inode */
	unsigned int	ioff_nlink_sz;	/* size of i_nlink field in bytes */
	unsigned int	ioff_size;	/* offset of i_size within vx_inode */
	unsigned int	ioff_size_sz;	/* size of i_size field in bytes */
};

struct vxfsu_bc_counters {
	uint64_t	vxi_bawrite;		/*vx_bawrite called */
	uint64_t	vxi_bcache_curkbyte;	/* vxfs bcache cur memory */
	uint64_t	vxi_bcache_maxkbyte;	/* vxfs bcache max memory */
	uint64_t	vxi_bcache_recycleage;	/* avg. age of recycled buf */
	uint64_t	vxi_bc_chunksteal;	/* number of chunks reused */
	uint64_t	vxi_bc_hits;		/* vxfs buffer cache hits */
	uint64_t	vxi_bc_lookups;		/* vxfs buffer cache lookup*/
	uint64_t	vxi_bc_reuse;		/* count of buffer reuse */
	uint64_t	vxi_bc_subflush;	/* internal bcache flushing */
	uint64_t	vxi_bc_waits;		/* wait for buffer lock */
	uint64_t	vxi_bdwrite;		/* vx_bdwrite called */
	uint64_t	vxi_bdwrite_tflush;	/* tranflush on
							vx_bdwrite call */
	uint64_t	vxi_bread;		/* vx_breada called */
	uint64_t	vxi_brelse;		/* vx_brelse called */
	uint64_t	vxi_brelse_tflush;	/* tranflush on vx_brelse call*/
	uint64_t	vxi_bwrite;		/* vx_bwrite called */
};

struct vxfsu_dnlc_counters {
	uint64_t	vxi_dnlc_hit;		/* +ve hit in the dnlc */
	uint64_t	vxi_dnlc_enter;		/* +ve enter in the dnlc */
	uint64_t	vxi_dnlc_miss;		/* miss in the dnlc */
	uint64_t	vxi_dnlc_neg_hit;	/* -ve hit in the dnlc */
	uint64_t	vxi_dnlc_neg_enter;	/* -ve enter in the dnlc */
	uint64_t	vxi_dnlc_size;		/* number of entries in dnlc */
};

typedef struct vx_cfs_mountinfo {
	unsigned int	cmi_type;	/* LOCAL | PRIMARY | SECONDARY */
	unsigned int	cmi_pri_node;	/* nodeid of primary */
	char	cmi_nodename[NAMELEN];
} vx_cfs_mountinfo_t;

#define FSAP_NAMESZ	64
#define FSAP_MAXDEVS	256
#define FSAP_MAXSUBP	64
#define FSDEV_NAMESZ	32
#define FSVOL_NAMESZ	FSDEV_NAMESZ

struct fsap_info {
	char		ap_name[FSAP_NAMESZ];
	uint32_t	ap_flags;
	uint32_t	ap_order;
	uint32_t	ap_ndevs;
	char		ap_devs[FSAP_MAXDEVS][FSDEV_NAMESZ];
};

/*
 * New variable length fsap_info for layout version 7 and higher.
 */

struct fsap_info2 {
	char		ap_name[FSAP_NAMESZ];
	uint32_t	ap_flags;
	uint32_t	ap_order;
	uint64_t	ap_chunk;
	uint32_t	ap_ncomp;
	uint32_t	ap_pad;
	union {
		char	apc_vols[1][FSVOL_NAMESZ];
		char	apc_subp[1][FSAP_NAMESZ];
	} ap_comp;
};

#define	ap_vols	ap_comp.apc_vols
#define	ap_subp	ap_comp.apc_subp

#define VX_ROUNDUP(x, bnd) \
		(((x) + ((bnd) - 1)) & ~((bnd) - 1))
#define VX_QUADALIGN(x)			VX_ROUNDUP((x), 8)
#define STRUCTOFF(field, structnm)	((size_t)(&(((structnm *)0L)->field)))
#define VXFS_AP_INFOSIZE(ncomp, flags)					       \
	(STRUCTOFF(ap_comp, struct fsap_info2) +			       \
	 (ncomp) * (((flags) & FSAP_SUBPOLICIES) ? FSAP_NAMESZ : FSVOL_NAMESZ))
#define VXFS_AP_NEXTINFO(info2p)					       \
	((struct fsap_info2 *)((char *)(info2p) + VX_QUADALIGN(		       \
	 VXFS_AP_INFOSIZE((info2p)->ap_ncomp, (info2p)->ap_flags))))

/*
 * Policy flags.
 */

#define FSAP_ORDER_ASGIVEN	0x0000	/* in the order given */
#define FSAP_ORDER_LEASTFULL	0x0001	/* use least full (by blocks) first */
#define FSAP_ORDER_ROUNDROBIN	0x0002	/* round-robin (roughly) */
#define FSAP_ORDER_BALANCE	0x0003	/* load-balance across volumes */

/*
 * Policy define operation flags (vxfs_ap_define argument).
 */

#define FSAP_CREATE		0x0001	/* deprecated, do not use */

/*
 * Policy definition flag (ap_flags).
 */

#define FSAP_ANYUSER		0x0001	/* any user can assign policy */
#define FSAP_SUBPOLICIES	0x0002	/* ap_comp has subpolicies    */

/*
 * Policy enforce flags.
 */
#define	FSAP_ENF_STRICT		0x0001	/* strict enforcement */

/*
 * Per file policy assignment flag (vxfs_ap_assign_file argument).
 */

#define FSAP_INHERIT		0x0001	/* policy is inherited */

struct fsdev_info {
	int		dev_id;
	uint64_t	dev_size;
	uint64_t	dev_free;
	uint64_t	dev_avail;
	char		dev_name[FSDEV_NAMESZ];
};

/*
 * flags defined for vxfs_nattr_check
 */

#define NATTR_ENABLED	1
#define NATTR_EXISTS	2

/*
 * File I/O statistics defines.
 */

#define	VX_FIOSTATS_ON		1
#define	VX_FIOSTATS_OFF		2
#define	VX_FIOSTATS_RESET	3

struct vx_fileiostats{
	uint64_t	nbytesread;
	uint64_t	nbyteswrite;
	uint32_t	nreads;
	uint32_t	nwrites;
	uint32_t	readtime;
	uint32_t	writetime;
	struct {
	    uint32_t	tv_sec;
	    uint32_t	tv_usec;
	} lastreset;
};

struct vx_rangeiostats{
	uint64_t	nbytesread;
	uint64_t	nbyteswrite;
	uint32_t	nreads;
	uint32_t	nwrites;
};

/*
 * File name pattern based policies
 */

#define FSAP_PATTERN_SZ		256
#define FSAP_PATTERN_MAX	256
#define FSAP_PATTERN_ID_NONE	-1

/*
 * pe_flags - unused for now.
 */
struct fsap_pattern_entry {
	uint32_t	pe_flags;
	uid_t		pe_uid;
	gid_t		pe_gid;
	char		pe_filename[FSAP_PATTERN_SZ];
	char		pe_data_policy[FSAP_NAMESZ];
	char		pe_meta_policy[FSAP_NAMESZ];
};

/*
 * pt_flags
 */

#define FSAP_PATTERN_INHERIT	0x01

struct fsap_pattern_table {
	uint32_t			pt_flags;
	uint32_t			pt_num;
	struct fsap_pattern_entry	pt_data[1];
};

/*
 * FCL API defines
 */

#define FCL_MISSEDRECORDS	0xf500
#define FCL_ENOSPC		0xf501

#define FCL_SEEK_COOKIE		0
#define FCL_SEEK_END		1
#define FCL_SEEK_START		2

struct fcl_iostats {
	uint64_t	nbytesread;
	uint64_t	nbyteswrite;
	uint32_t	nreads;
	uint32_t	nwrites;
	uint32_t	readtime;
	uint32_t	writetime;
	struct {
	    uint32_t	tv_sec;
	    uint32_t	tv_nsec;
	} lastreset;
	uint32_t	nodeid;
	uint32_t	reset;
};

struct fcl_acsinfo {
	uint32_t	fa_ruid;
	uint32_t	fa_rgid;
	uint32_t	fa_euid;
	uint32_t	fa_egid;
	uint32_t	fa_pid;
	uint32_t	fa_nodeid;
};

struct fcl_nminfo {
	uint64_t	fn_pinonum;	/* Parent Inode Number. */
	uint32_t	fn_pinogen;	/* Parent Inode Gen cnt. */
	char		*fn_filename;
};

struct fcl_evmaskinfo {
	uint64_t	oldmask;	/* Old event mask. */
	uint64_t	newmask;	/* New event mask. */
};

typedef struct fcl_time {
	uint32_t	tv_sec;
	uint32_t	tv_nsec;
} fcl_time_t;

/*
 * Structures like accessinfo, iostat etc. as well as variable length info like
 * filename, command name are represented as pointers in the FCL record
 * returned. The actual structure or the string is stored immediately after the
 * fcl_record structure and the pointers are made to point to the data. The
 * strings are null terminated and reclen is updated appropriately to
 * accomodate the whole data. Validity bits for the optional fields which are
 * populated are set.
 */
struct fcl_record {
	uint32_t	fr_reclen;	/* Record length */
	uint16_t	fr_op;		/* Operation type. */
	uint16_t	fr_unused1;	/* unused field */
	uint32_t	fr_acsinfovalid : 1;	/* fr_acsinfo field valid */
	uint32_t	fr_newnmvalid : 1; /* fr_newfilename field is valid */
	uint32_t	fr_pinogenvalid : 1; /* fr_fr_pinogen field is valid */
	uint32_t	fr_unused2 : 29;	/* Future use */
	uint64_t	fr_inonum;	/* Inode Number. */
	uint32_t	fr_inogen;	/* Inode Generation Count. */
	fcl_time_t	fr_time;	/* Time. */
	union fcl_vardata {
		char			*fv_cmdname;
		struct fcl_nminfo	fv_nm;
		struct fcl_iostats	*fv_stats;
		struct fcl_evmaskinfo	fv_evmask;
	} fr_var;
	uint64_t		fr_tdino;		/* Target dir ino */
	char			*fr_newfilename;	/* For rename */
	struct fcl_acsinfo	*fr_acsinfo;		/* Access Info */
};

#define fr_cmdname	fr_var.fv_cmdname
#define fr_stats	fr_var.fv_stats
#define fr_oldmask	fr_var.fv_evmask.oldmask
#define fr_newmask	fr_var.fv_evmask.newmask
#define fr_pinonum	fr_var.fv_nm.fn_pinonum
#define fr_pinogen	fr_var.fv_nm.fn_pinogen
#define fr_filename	fr_var.fv_nm.fn_filename

struct fcl_info {
	uint32_t	fi_state;
	uint32_t	fi_version;
};

struct fcl_cookie {
	char	fc_cookie[24];
};

struct fsvol_queryinfo {
	uint64_t	vol_flags;
	char		vol_name[FSVOL_NAMESZ];
};

#ifdef _AIX
/*
 * cmd argument to vxacl
 */

#define GETACLCNT	1
#define GETACL		2
#define SETACL		3

#define MAX_ACL_ENTRIES		(1024)	/* max entries of each type */
#define MIN_ACL_ENTRIES		(4)	/* min entries */

#define VX_USER_OBJ	(0x01)		/* owner entry */
#define VX_USER		(0x02)		/* additional users */
#define VX_GROUP_OBJ	(0x04)		/* owning group entry */
#define VX_GROUP	(0x08)		/* additional groups */
#define VX_CLASS_OBJ	(0x10)		/* mask entry */
#define VX_OTHER_OBJ	(0x20)		/* other entry */
#define VX_ACL_DEFAULT	(0x1000)	/* default flag */

#endif

typedef struct {
	int		a_type;	/* the type of ACL entry */
	uid_t		a_id;	/* uid or gid */
	unsigned short	a_perm;	/* permission */
	unsigned short	a_pad;
} vx_aclent_t;

typedef struct vx_time32 {
	uint32_t	tv_sec;
	uint32_t	tv_nsec;
} vx_time32_t;

struct vxstat {
	uint64_t	st_ino;
	uint64_t	st_nlink;
	uint64_t	st_size;
	uid_t		st_uid;
	gid_t		st_gid;
	vx_time32_t	st_atim;
	vx_time32_t	st_mtim;
	vx_time32_t	st_ctim;
	uint32_t	st_mode;
	uint32_t	st_blksize;
	uint32_t	st_blocks;
	int		st_dev;
};

/*
 * volume flags, defined exactly same as definition of Device flags
 * in vx_layout.h (struct vx_devconf, dc_flags)
 */

#define VX_VOLFLAG_DATAONLY	0x1	/* dataonly volume */
#define VX_VOLFLAG_METADATAONLY	0x2	/* metadataonly volume */
#define VX_VOLFLAG_NOALLOC	0x4	/* do not allocate from this device */

/*
 * NBU DST API defines and flags.
 */

#define VXFS_LIB_INIT		0x00000100
#define VXFS_LIB_FINI		0x00000200
#define VXFS_GETEXTFS		0x00000400
#define VXFS_GETEXTFS_RELE	0x00000800
#define VXFS_GETEXTFILE		0x00001000
#define VXFS_GETEXTFILE_RELE	0x00002000
#define VXFS_SETEXTFS		0x00004000
#define VXFS_SETEXTFILE_PRE	0x00008000
#define VXFS_SETEXTFILE_POST	0x00010000
#define VXFS_NO_DST		0x00000001

#define VXFS_PROTCONFIG_FILE	"/etc/vx/fsprotconfig"
#define VXFS_PROTCONFIG_PREFKEY	"preferred_protocol"

extern int vxfs_vol_add(int, char *, uint64_t);
extern int vxfs_vol_add2(int, char *, uint64_t, uint64_t);
extern int vxfs_vol_remove(int, char *);
extern int vxfs_vol_resize(int, char *, uint64_t);
extern int vxfs_vol_stat(int, char *, struct fsdev_info *);
extern int vxfs_vol_enumerate(int, int *, struct fsdev_info *);
extern int vxfs_vol_encapsulate(char *, char *, uint64_t);
extern int vxfs_vol_encapsulate_bias(char *, char *, uint64_t, uint64_t);
extern int vxfs_vol_deencapsulate(char *);
extern int vxfs_vol_setflags(int, uint64_t, int, char **);
extern int vxfs_vol_clearflags(int, uint64_t, int, char **);
extern int vxfs_vol_queryflags(int, int, int *, struct fsvol_queryinfo *);

extern struct fsap_info2 *vxfs_ap_alloc2(uint32_t, uint32_t);
extern void vxfs_ap_free2(struct fsap_info2 *);
extern int vxfs_ap_define(int, struct fsap_info *, uint32_t);
extern int vxfs_ap_define2(int, struct fsap_info2 *, uint32_t);
extern int vxfs_ap_remove(int, char *);
extern int vxfs_ap_query(int, struct fsap_info *);
extern int vxfs_ap_query2(int, char *, struct fsap_info2 **);
extern int vxfs_ap_query_file(int, char *, char *, uint32_t *);
extern int vxfs_ap_assign_file(int, char *, char *, uint32_t);
extern int vxfs_ap_query_ckpt(int, char *, char *, char *);
extern int vxfs_ap_assign_ckpt(int, char *, char *, char *);
extern int vxfs_ap_assign_ckpt_def(int, char *, char *);
extern int vxfs_ap_query_ckpt_def(int, char *, char *);
extern int vxfs_ap_query_fs(int, char *, char *);
extern int vxfs_ap_assign_fs(int, char *, char *);
extern int vxfs_ap_enforce_file(int, char *, char *);
extern int vxfs_ap_enforce_file2(int, char *, char *, uint32_t);
extern int vxfs_ap_enumerate(int, int *, struct fsap_info *);
extern int vxfs_ap_enumerate2(int, uint32_t *, struct fsap_info2 **);

extern int vxfs_nattr_check(char *, int);
extern int vxfs_nattr_fcheck(int, int);
extern int vxfs_nattr_open(int, char *, int, int);
extern int vxfs_nattr_unlink(int, char *);
extern int vxfs_nattr_link(int, char *, char *);
extern int vxfs_nattr_rename(int, char *, char *);
extern int vxfs_nattr_utimes(int, char *, struct timeval *);

extern int vxfs_fcl_sync(char *, uint64_t *);
extern int vxfs_fcl_open(char *, uint32_t, void **);
extern int vxfs_fcl_close(void *);
extern int vxfs_fcl_getinfo(void *, struct fcl_info *);
extern int vxfs_fcl_read(void *, char *, size_t *, uint64_t, uint32_t *);
extern int vxfs_fcl_getcookie(void *, struct fcl_cookie *);
extern int vxfs_fcl_geteventmask(void *, uint64_t *);
extern int vxfs_fcl_seek(void *, struct fcl_cookie *, int whence);
extern int vxfs_fcl_seektime(void *, fcl_time_t);
extern int vxfs_fcl_copyrec(struct fcl_record *, struct fcl_record *, size_t);

extern int vxfs_inotopath(char *, uint64_t, int, char ***, int *);
extern int vxfs_inotopath_gen(char *, uint64_t, uint32_t, int, char ***, int *);
extern int vxfs_get_mntinfo(struct vx_cfs_mountinfo *, size_t, char *);
extern int vxfs_get_ioffsets(struct vx_ioffsets *, size_t);

extern int vxfsu_get_ioffsets(struct vx_ioffsets *, size_t);
extern int vxfsu_get_bc_counters(struct vxfsu_bc_counters *, size_t);
extern int vxfsu_get_dnlc_counters(struct vxfsu_dnlc_counters *, size_t);
extern int vxfs_ap_assign_file_pat(int ,struct fsap_pattern_table *, uint32_t);
extern int vxfs_ap_query_file_pat(int ,struct fsap_pattern_table **, uint32_t);
extern int vxfs_ap_assign_fs_pat(int ,struct fsap_pattern_table *, uint32_t);
extern int vxfs_ap_query_fs_pat(int ,struct fsap_pattern_table **, uint32_t);

extern int vxfs_getcache(int, int *);
extern int vxfs_setcache(int, int);

extern int vxfs_fiostats_set(int, int, uint64_t, uint64_t, uint32_t);
extern int vxfs_fiostats_getconfig(int, int *, uint64_t *, uint64_t *,
				uint32_t *);
extern int vxfs_fiostats_dump(int, int, struct vx_fileiostats *,
				struct vx_rangeiostats *, uint32_t *);

extern int vxacl(char *, int, int, vx_aclent_t *);
extern int vxfacl(int, int, int, void *);

extern int vxfs_meta_backup(int, int, void **, uint64_t *, void **);
extern int vxfs_meta_restore(int, int, void *, uint64_t, void **);

extern int vxfs_ap_assign_ckptchain(int, char *, char *);
extern int vxfs_ap_enforce_ckpt(int, char*, char *, char *, uint64_t);
extern int vxfs_ap_enforce_ckptchain(int, char *, char *, uint64_t);
extern int vxfs_inostat(int, uint64_t, struct vxstat *);
extern int vxfs_inotofd(int, uint64_t, int, int *);

extern int vxfs_ts_reclaim(char *, uint64_t, uint64_t, int32_t,
			   uint64_t, uint64_t *, uint32_t);
#ifdef __cplusplus
}
#endif

#endif /* _VXFSUTIL_H */
