/* $Id: stubs.c,v 4.51.14.3 2008/05/25 07:59:31 akadlag Exp $  */
/* #ident "@(#)vxfs:$RCSfile: stubs.c,v $	$Revision: 4.51.14.3 $" */

/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#include <stdlib.h>
#include <dlfcn.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <vxfsosdep.h>
#include <sys/time.h>
#include <vxfsutil.h>

#ifndef VXFSU_GET_IOFFSETS
#define VXFSU_GET_IOFFSETS(ioffp) -1
#endif

#ifndef VXFSU_GET_MNTINFO
#define VXFSU_GET_MNTINFO(bufp) -1
#endif

#define FCL_API_VERSION 4

#define BODY(funcname, proto, args)					\
	int	rv;							\
	int	(*func)proto;						\
									\
	fsutil_setup((funcname), (void *)&func);			\
	if (func == NULL) {						\
		return ENOTSUP;						\
	}								\
	rv = (*func)args;						\
	return rv;

#define BODY1(funcname, proto, args)					\
	int	rv;							\
	int	(*func)proto;						\
									\
	fsutil_setup((funcname), (void *)&func);			\
	if (func == NULL) {						\
		errno=ENOTSUP;						\
		return -1;						\
	}								\
	rv = (*func)args;						\
	return rv;

STATIC void	fsutil_setup(char *, void **);
STATIC void	*fsutil_handlep = NULL;

STATIC void
fsutil_setup(
	char		*funcname,
	void		**funcp)
{
	char	*implpath;

	*funcp = NULL;
	if (fsutil_handlep == NULL) {

		implpath = getenv("LIBVXFSUTIL_DLL_PATH");
		if (implpath == NULL) {
			if (sizeof (void *) == 8) {
				implpath = LIBVXUTIL_IMPL_PATH_64;
			} else {
				implpath = LIBVXUTIL_IMPL_PATH_32;
			}
		}

		if (implpath) {
			fsutil_handlep = dlopen(implpath, RTLD_LAZY);
		}
		if (fsutil_handlep == NULL) {
			return;
		}
	}

	*funcp = dlsym(fsutil_handlep, funcname);
	return;
}

int
vxfs_get_ioffsets(
	struct vx_ioffsets	*buf,
	size_t			size)
{
	int			(*func)(struct vx_ioffsets *, size_t);
	int			rv;

	fsutil_setup("_vxfs_get_ioffsets", (void *)&func);
	if (func == NULL) {
		if (size < sizeof (*buf)) {
			return -1;
		}
		rv = VXFSU_GET_IOFFSETS(buf);
		return rv;
	}
	rv = (*func)(buf, size);
	return rv;
}

int
vxfsu_get_ioffsets(
	struct vx_ioffsets	*buf,
	size_t			size)
{
	int			(*func)(struct vx_ioffsets *, size_t);
	int			rv;

	fsutil_setup("_vxfsu_get_ioffsets", (void *)&func);
	if (func == NULL) {
		if (size < sizeof (*buf)) {
			return -1;
		}
		rv = VXFSU_GET_IOFFSETS(buf);
		return rv;
	}
	rv = (*func)(buf, size);
	return rv;
}

int
vxfsu_get_bc_counters(
	struct vxfsu_bc_counters	*buf,
	size_t				size)
{
	int			(*func)(struct vxfsu_bc_counters *, size_t);
	int			rv;

	fsutil_setup("_vxfsu_get_bc_counters", (void *)&func);
	if (func == NULL) {
		if (size < sizeof (*buf)) {
			return EINVAL;
		}
		return ENOENT;
	}
	rv = (*func)(buf, size);
	return rv;
}

int
vxfsu_get_dnlc_counters(
	struct vxfsu_dnlc_counters	*buf,
	size_t				size)
{
	int			(*func)(struct vxfsu_dnlc_counters *, size_t);
	int			rv;

	fsutil_setup("_vxfsu_get_dnlc_counters", (void *)&func);
	if (func == NULL) {
		if (size < sizeof (*buf)) {
			return EINVAL;
		}
		return ENOENT;
	}
	rv = (*func)(buf, size);
	return rv;
}

int
vxfs_fcl_sync(
	char		*fname,
	uint64_t	*offp)
{
	BODY("_vxfs_fcl_sync",
	     (char *, uint64_t *),
	     (fname, offp))
}

int
vxfs_fcl_open(
	char		*pathname,
	uint32_t	flags,
	void		**handle)
{
	BODY("_vxfs_fcl_open",
	     (char *, uint32_t, void **, uint32_t),
	     (pathname, flags, handle, FCL_API_VERSION))
}

int
vxfs_fcl_close(
		void	*handle)
{
	BODY("_vxfs_fcl_close",
	     (void *), (handle))
}

int
vxfs_fcl_getinfo(
		void		*handle,
		struct fcl_info	*info)
{
	BODY("_vxfs_fcl_getinfo",
	     (void *, struct fcl_info *),
	     (handle, info))
}

int
vxfs_fcl_getcookie(
		void			*handle,
		struct fcl_cookie	*cookie)
{
	BODY("_vxfs_fcl_getcookie",
	     (void *, struct fcl_cookie *),
	     (handle, cookie))
}

int
vxfs_fcl_geteventmask(
		void			*handle,
		uint64_t		*evmaskp)
{
	BODY("_vxfs_fcl_geteventmask",
	     (void *, uint64_t *),
	     (handle, evmaskp))
}

int
vxfs_fcl_seek(
	void			*handle,
	struct fcl_cookie	*cookie,
	int			whence)
{
	BODY("_vxfs_fcl_seek",
	     (void *, struct fcl_cookie *, int),
	     (handle, cookie, whence))
}

int
vxfs_fcl_seektime(
	void		*handle,
	fcl_time_t	timeval)
{
	BODY("_vxfs_fcl_seektime",
	     (void *, fcl_time_t),
	     (handle, timeval))
}

int
vxfs_fcl_read(
		void		*handle,
		char		*buf,
		size_t		*bufsz,
		uint64_t	eventmask,
		uint32_t	*nentries)
{
	BODY("_vxfs_fcl_read",
	     (void *, char *, size_t *, uint64_t, uint32_t *),
	     (handle, buf, bufsz, eventmask, nentries))
}

int vxfs_fcl_copyrec(
		struct fcl_record *src,
		struct fcl_record *tgt,
		size_t size)
{
	BODY("_vxfs_fcl_copyrec",
	     (struct fcl_record *, struct fcl_record *, size_t),
	     (src, tgt, size))
}

int
vxfs_vol_add(
	int		fd,
	char		*devname,
	uint64_t	size)
{
	BODY("_vxfs_vol_add",
	     (int, char *, uint64_t),
	     (fd, devname, size))
}

int
vxfs_vol_add2(
	int		fd,
	char		*devname,
	uint64_t	size,
	uint64_t	flags)
{
	BODY("_vxfs_vol_add2",
	     (int, char *, uint64_t, uint64_t),
	     (fd, devname, size, flags))
}

int
vxfs_vol_remove(
	int		fd,
	char		*devname)
{
	BODY("_vxfs_vol_remove",
	     (int, char *),
	     (fd, devname))
}

int
vxfs_vol_resize(
	int		fd,
	char		*devname,
	uint64_t	newsize)
{
	BODY("_vxfs_vol_resize",
	     (int, char *, uint64_t),
	     (fd, devname, newsize))
}

int
vxfs_vol_stat(
	int			fd,
	char			*devname,
	struct fsdev_info	*infop)
{
	BODY("_vxfs_vol_stat",
	     (int, char *, struct fsdev_info *),
	     (fd, devname, infop))
}

int
vxfs_vol_enumerate(
	int			fd,
	int			*countp,
	struct fsdev_info	*infop)
{
	BODY("_vxfs_vol_enumerate",
	     (int, int *, struct fsdev_info *),
	     (fd, countp, infop))
}

int
vxfs_vol_encapsulate(
	char			*fname,
	char			*devname,
	uint64_t		size)
{
	BODY("_vxfs_vol_encapsulate",
	     (char *, char *, uint64_t),
	     (fname, devname, size))
}

int
vxfs_vol_encapsulate_bias(
	char			*fname,
	char			*devname,
	uint64_t		size,
	uint64_t		bias)
{
	BODY("_vxfs_vol_encapsulate_bias",
	     (char *, char *, uint64_t, uint64_t),
	     (fname, devname, size, bias))
}

int
vxfs_vol_deencapsulate(
	char			*fname)
{
	BODY("_vxfs_vol_deencapsulate",
	     (char *),
	     (fname))
}

int
vxfs_vol_setflags(
	int		fd,
	uint64_t	flags,
	int		nvols,
	char		**vollist)
{
	BODY("_vxfs_vol_setflags",
	     (int, uint64_t, int, char **),
	     (fd, flags, nvols, vollist))
}

int
vxfs_vol_clearflags(
	int		fd,
	uint64_t	flags,
	int		nvols,
	char		**vollist)
{
	BODY("_vxfs_vol_clearflags",
	     (int, uint64_t, int, char **),
	     (fd, flags, nvols, vollist))
}

int
vxfs_vol_queryflags(
	int				fd,
	int				getall,
	int				*countp,
	struct fsvol_queryinfo		*qinfop)
{
	BODY("_vxfs_vol_queryflags",
	     (int, int, int *, struct fsvol_queryinfo *),
	     (fd, getall, countp, qinfop))
}

int
vxfs_ap_define(
	int			fd,
	struct fsap_info	*fsap,
	uint32_t		flags)
{
	BODY("_vxfs_ap_define",
	     (int, struct fsap_info *, uint32_t),
	     (fd, fsap, flags))
}

int
vxfs_ap_remove(
	int	fd,
	char	*name)
{
	BODY("_vxfs_ap_remove",
	     (int, char *),
	     (fd, name))
}

int
vxfs_ap_query(
	int			fd,
	struct fsap_info	*fsap)
{
	BODY("_vxfs_ap_query",
	     (int, struct fsap_info *),
	     (fd, fsap))
}

int
vxfs_ap_query_file(
	int		fd,
	char		*datapol,
	char		*indirpol,
	uint32_t	*flags)
{
	BODY("_vxfs_ap_query_file",
	     (int, char *, char *, uint32_t *),
	     (fd, datapol, indirpol, flags))
}

int
vxfs_ap_assign_file(
	int		fd,
	char		*datapol,
	char		*indirpol,
	uint32_t	flags)
{
	BODY("_vxfs_ap_assign_file",
	     (int, char *, char *, uint32_t),
	     (fd, datapol, indirpol, flags))
}

int
vxfs_ap_query_ckpt(
	int	fd,
	char	*ckptname,
	char	*datapol,
	char	*indirpol)
{
	BODY("_vxfs_ap_query_ckpt",
	     (int, char *, char *, char *),
	     (fd, ckptname, datapol, indirpol))
}

int
vxfs_ap_query_ckpt_def(
	int	fd,
	char	*datapol,
	char	*indirpol)
{
	BODY("_vxfs_ap_query_ckpt_def",
	     (int, char *, char *),
	     (fd, datapol, indirpol))
}

int
vxfs_ap_assign_ckpt(
	int	fd,
	char	*ckptname,
	char	*datapol,
	char	*indirpol)
{
	BODY("_vxfs_ap_assign_ckpt",
	     (int, char *, char *, char *),
	     (fd, ckptname, datapol, indirpol))
}

int
vxfs_ap_assign_ckpt_def(
	int	fd,
	char	*datapol,
	char	*indirpol)
{
	BODY("_vxfs_ap_assign_ckpt_def",
	     (int, char *, char *),
	     (fd, datapol, indirpol))
}

int
vxfs_ap_query_fs(
	int	fd,
	char	*datapol,
	char	*indirpol)
{
	BODY("_vxfs_ap_query_fs",
	     (int, char *, char *),
	     (fd, datapol, indirpol))
}

int
vxfs_ap_assign_fs(
	int	fd,
	char	*datapol,
	char	*indirpol)
{
	BODY("_vxfs_ap_assign_fs",
	     (int, char *, char *),
	     (fd, datapol, indirpol))
}

int
vxfs_ap_enforce_file(
	int	fd,
	char	*datapol,
	char	*indirpol)
{
	BODY("_vxfs_ap_enforce_file",
	     (int, char *, char *),
	     (fd, datapol, indirpol))
}

int
vxfs_ap_enforce_file2(
	int		fd,
	char		*datapol,
	char		*indirpol,
	uint32_t	flags)
{
	BODY("_vxfs_ap_enforce_file2",
	     (int, char *, char *, uint32_t),
	     (fd, datapol, indirpol, flags))
}

int
vxfs_ap_enumerate(
	int			fd,
	int			*countp,
	struct fsap_info	*infop)
{
	BODY("_vxfs_ap_enumerate",
	     (int, int *, struct fsap_info *),
	     (fd, countp, infop))
}

int
vxfs_get_mntinfo(
	struct vx_cfs_mountinfo	*buf,
	size_t	size,
	char	*mnt )
{
	int			(*func)(vx_cfs_mountinfo_t *, size_t, char *);
	int			rv;

	fsutil_setup("_vxfs_get_mntinfo", (void *)&func);
	if (func == NULL) {
		if (size < sizeof (*buf)) {
			return -1;
		}
		rv = VXFSU_GET_MNTINFO(buf);
		return rv;
	}
	rv = (*func)(buf, size, mnt);
	return rv;
}

int
vxfs_inotopath(
	char			*mntpoint,
	uint64_t		inode,
	int			all,
	char			***bufp,
	int			*nentries)
{
	BODY("_vxfs_inotopath",
	     (char *, uint64_t, int, char ***, int *),
	     (mntpoint, inode, all, bufp, nentries))
}

int
vxfs_inotopath_gen(
	char			*mntpoint,
	uint64_t		inode,
	uint32_t		gen,
	int			all,
	char			***bufp,
	int			*nentries)
{
	BODY("_vxfs_inotopath_gen",
	     (char *, uint64_t, uint32_t, int, char ***, int *),
	     (mntpoint, inode, gen, all, bufp, nentries))
}

int
vxfs_nattr_check(
	char		*path,
	int		flag)
{
	int	error,fd;

	if((fd = open(path, O_RDONLY)) < 0) {
		return fd;
	}
	error = vxfs_nattr_fcheck(fd, flag);
	close(fd);
	return error;
}

int
vxfs_nattr_fcheck(
	int		fd,
	int		flag)
{
	BODY1("_vxfs_nattr_check",
		(int, int), (fd, flag))
}

int
vxfs_nattr_open(
	int		fd,
	char		*name,
	int		fmode,
	int		cmode)
{
	BODY1("_vxfs_nattr_open",
		(int, char *, int, int), (fd, name, fmode, cmode))
}

int
vxfs_nattr_unlink(
	int		fd,
	char		*name)
{
	BODY1("_vxfs_nattr_unlink",
		(int, char *), (fd, name))
}

int
vxfs_nattr_link(
	int		fd,
	char		*src,
	char		*dst)
{
	BODY1("_vxfs_nattr_link",
		(int, char *, char *), (fd, src, dst))
}

int
vxfs_nattr_rename(
	int		fd,
	char		*src,
	char		*dst)
{
	BODY1("_vxfs_nattr_rename",
		(int, char *, char *), (fd, src, dst))
}

int
vxfs_nattr_utimes(
	int	fd,
	char	*attr,
	struct timeval	*timep)
{
	BODY1("_vxfs_nattr_utimes",
		(int, char *, struct timeval *), (fd, attr, timep))
}

int
vxfs_setcache(
	int		fd,
	int		iargs)
{
	BODY("_vxfs_setcache",
		(int, int), (fd, iargs))
}

int
vxfs_getcache(
	int		fd,
	int		*iargs)
{
	BODY("_vxfs_getcache",
		(int, int *), (fd, iargs))
}

struct fsap_info2 *
vxfs_ap_alloc2(
	uint32_t		ncomp,
	uint32_t		flags)
{
	struct fsap_info2	*(*func)(uint32_t, uint32_t);

	fsutil_setup("_vxfs_ap_alloc2", (void *)&func);
	if (func == NULL) {
		return NULL;
	}
	return (*func)(ncomp, flags);
}

void
vxfs_ap_free2(
	struct fsap_info2	*fsap)
{
	void	(*func)(struct fsap_info2 *);

	fsutil_setup("_vxfs_ap_free2", (void *)&func);
	if (func == NULL) {
		return;
	}
	(*func)(fsap);
	return;
}

int
vxfs_ap_define2(
	int			fd,
	struct fsap_info2	*fsap,
	uint32_t		flags)
{
	BODY("_vxfs_ap_define2",
	     (int, struct fsap_info2 *, uint32_t),
	     (fd, fsap, flags))
}

int
vxfs_ap_query2(
	int			fd,
	char			*policyname,
	struct fsap_info2	**fsapp)
{
	BODY("_vxfs_ap_query2",
	     (int, char *, struct fsap_info2 **),
	     (fd, policyname, fsapp))
}

int
vxfs_ap_enumerate2(
	int			fd,
	uint32_t		*count,
	struct fsap_info2	**fsaplist)
{
	BODY("_vxfs_ap_enumerate2",
	     (int, uint32_t *, struct fsap_info2 **),
	     (fd, count, fsaplist))
}

int
vxfs_fiostats_set(
	int		fd,
	int		action,
	uint64_t	offset,
	uint64_t	len,
	uint32_t	nrange)
{
	BODY("_vxfs_fiostats_set",
		(int, int, uint64_t, uint64_t, uint32_t),
		(fd, action, offset, len, nrange))
}

int
vxfs_fiostats_getconfig(
	int		fd,
	int		*state,
	uint64_t	*offset,
	uint64_t	*len,
	uint32_t	*nrange)
{
	BODY("_vxfs_fiostats_getconfig",
		(int, int *, uint64_t *, uint64_t *, uint32_t *),
		(fd, state, offset, len, nrange))
}

int
vxfs_fiostats_dump(
	int			fd,
	int			reset,
	struct vx_fileiostats	*filebufp,
	struct vx_rangeiostats	*rangebufp,
	uint32_t		*nrange)
{
	BODY("_vxfs_fiostats_dump",
		(int, int , struct vx_fileiostats *,
		struct vx_rangeiostats *, uint32_t *),
		(fd, reset, filebufp, rangebufp, nrange))
}

int
vxfs_ap_assign_file_pat(
	int				fd,
	struct fsap_pattern_table	*fsap_pat,
	uint32_t			flags)
{
	BODY("_vxfs_ap_assign_file_pattern",
	     (int, struct fsap_pattern_table *, uint32_t),
	     (fd, fsap_pat, flags))
}

int
vxfs_ap_query_file_pat(
	int				fd,
	struct fsap_pattern_table	**fsap_pat,
	uint32_t			flags)
{
	BODY("_vxfs_ap_query_file_pattern",
	     (int, struct fsap_pattern_table **, uint32_t),
	     (fd, fsap_pat, flags))
}

int
vxfs_ap_assign_fs_pat(
	int				fd,
	struct fsap_pattern_table	*fsap_pat,
	uint32_t			flags)
{
	BODY("_vxfs_ap_assign_fs_pattern",
	     (int, struct fsap_pattern_table *, uint32_t),
	     (fd, fsap_pat, flags))
}

int
vxfs_ap_query_fs_pat(
	int				fd,
	struct fsap_pattern_table	**fsap_pat,
	uint32_t			flags)
{
	BODY("_vxfs_ap_query_fs_pattern",
	     (int, struct fsap_pattern_table **, uint32_t),
	     (fd, fsap_pat, flags))
}

int
vxfacl(
	int		fd,
	int		cmd,
	int		nentries,
	void		*aclbufp)
{

	BODY1("_vxfacl",
		(int, int, int, void *), (fd, cmd, nentries, aclbufp))
}

int
vxfs_meta_backup(
		int		fd,
		int		cmd,
		void		**buffer,
		uint64_t	*buflen,
		void		**token)
{
	BODY("_vxfs_meta_backup",
		(int, int, void**, uint64_t*, void**), (fd, cmd, buffer,
		buflen, token));
}

int
vxfs_meta_restore(
		int		fd,
		int		cmd,
		void		*buffer,
		uint64_t	buflen,
		void		**token)
{
	BODY("_vxfs_meta_restore",
		(int, int, void*, uint64_t, void**), (fd, cmd, buffer, buflen,
		token));
}

int
vxfs_inostat(
	int		fd,
	uint64_t	ino,
	struct vxstat	*stat)
{
	BODY("_vxfs_inostat",
	     (int, uint64_t , struct vxstat *),
	     (fd, ino, stat))
}

int
vxfs_inotofd(
	int		fd,
	uint64_t	ino,
	int		mode,
	int		*newfd)
{
	BODY("_vxfs_inotofd",
	     (int, uint64_t, int, int *),
	     (fd, ino, mode, newfd))
}

int
vxfs_inostat2(
	int		fd,
	char		*ckptname,
	uint64_t	iltype,
	uint64_t	ino,
	struct vxstat	*stat)
{
	BODY("_vxfs_inostat",
	     (int, char *, uint32_t, uint64_t , struct vxstat *),
	     (fd, ckptname, iltype, ino, stat))
}

int
vxfs_inotofd2(
	int		fd,
	char		*ckptname,
	uint64_t	iltype,
	uint64_t	ino,
	int		mode,
	int		*newfd)
{
	BODY("_vxfs_inotofd",
	     (int, char*, uint32_t, uint64_t, int, int *),
	     (fd, ckptname, iltype, ino, mode, newfd))
}

int
vxfs_ap_assign_ckptchain(
	int	fd,
	char	*datapol,
	char	*indirpol)
{
	BODY("_vxfs_ap_assign_ckptchain",
	     (int, char *, char *),
	     (fd, datapol, indirpol))
}

int
vxfs_ap_enforce_ckptchain(
	int					fd,
	char					*datapol,
	char					*indirpol,
	uint64_t				flags)
{
	BODY("_vxfs_ap_enforce_ckptchain",
	    (int, char *, char *, uint64_t),
	    (fd, datapol, indirpol, flags))
}

int
vxfs_ap_enforce_ckpt(
	int					fd,
	char					*ckptname,
	char					*datapol,
	char					*indirpol,
	uint64_t				flags)
{
	BODY("_vxfs_ap_enforce_ckpt_name",
	    (int, char *, char *, char *, uint64_t),
	    (fd, ckptname, datapol, indirpol, flags))
}

int
vxfs_ts_reclaim(
	char		*mountpoint,
	uint64_t	offset,
	uint64_t	length,
	int32_t		volindex,
	uint64_t	unit_size,
	uint64_t	*bytes_reclaimed,
	uint32_t	flag)
{
	BODY("_vxfs_ts_reclaim",
	    (char *,uint64_t, uint64_t, int32_t, uint64_t, uint64_t *,
	    uint32_t), (mountpoint, offset, length, volindex, unit_size,
	    bytes_reclaimed, flag))
}
