/* $Id: fcltest.c,v 4.12.14.1 2007/09/07 11:08:38 ckrishna Exp $  */
/* #ident "@(#)vxfs:$RCSfile: fcltest.c,v $     $Revision: 4.12.14.1 $" */

/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

/*
 * FCL Demo:
 * 	Menu-based program that illustrates the features of FCL
 */

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <stdio.h>
#include <limits.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <strings.h>

#ifdef __linux__
#include <stdint.h>
#endif

#include <fcl.h>
#include <vxfsutil.h>

char *fcltype[] = {
	"No Change",
	"Info_Change",
	"Create",
	"Add_Link",
	"Unlink",
	"Rename",
	"Undelete",
	"Extend",
	"Overwrite",
	"Truncate",
	"Ext_Attr_Chg",
	"Hole_Punch",
	"Header_Rec",
	"Sym_Link",
	"ExtnAttrChg",
	"ReserveChg",
	"ModeChg",
	"OwnerChg",
	"GroupChg",
	"ItimeChg",
	"MtimeChg",
	"AccessInfo",
	"FileOpen",
	"FileClose",
	"FileStats",
	"FclEvntMask",
	NULL
};

int ismntpt = 0;
char arg[PATH_MAX];
char mntpt[PATH_MAX];
char filename[PATH_MAX];
uint64_t eventmask = FCL_ALL_V4_EVENTS;
int maxentries = -1;

static int
is_fcl_on(
	void)
{
	int error = 0;
	int retcode = 0;
	struct fcl_info fclinfo;
	void *fh;

	error = vxfs_fcl_open(filename, 0, &fh);
	if (error) {
		errno = error;
		perror("Open Failed");
		exit(-1);
	}

	error = vxfs_fcl_getinfo(fh, &fclinfo);
	if (error) {
		perror("vxfs_fcl_getinfo failed");
		exit(-1);
	}

	if (fclinfo.fi_state == VX_FCLS_ON)
		retcode = 1;

	vxfs_fcl_close(fh);
	return retcode;
}

static void
vx_path_lookup(
	char *mntpt,
	uint64_t inode,
	uint32_t gen,
	char *buf)
{
	char **ino_buf;
	int nentries;
	int i;
	int error = 0;

	error = vxfs_inotopath_gen(mntpt, inode, gen, 0, &ino_buf, &nentries);
	if (error == 0) {
		strcpy(buf, ino_buf[0]);
		for (i = 0; i < nentries; i++) {
			free(ino_buf[i]);
		}
		free(ino_buf);
	} else {

		/*  check for older routine on vxfs 4.0 */
		if (error == ENOTSUP) {
			if ((error = vxfs_inotopath(mntpt, inode, 0, &ino_buf,
							&nentries)) == 0) {
				strcpy(buf, ino_buf[0]);
				for (i = 0; i < nentries; i++) {
					free(ino_buf[i]);
				}
				free(ino_buf);
				return;
			} else if (error == ENOTSUP) {
				printf("\r\nvxfs_inotpath Error, %s",
						strerror(error));
				exit(1);
			}
		}
		strcpy(buf, strerror(error));
	}
}

static char*
getctime(
	time_t *tm)
{
	static char tbuf[100];
	char *ptr = ctime(tm);
	strcpy(tbuf, ptr);
	return tbuf;
}

/*
* process the fcl entry
*/
static void
process_fcl_entry(
	struct fcl_record *fr)
{
	struct fcl_iostats *fiop;
	char buf[PATH_MAX];

	printf("Type of record:%s\n", fcltype[fr->fr_op]);
	printf("Inode number:%llu\t\tInode gen count:%llu\n",
		(unsigned long long)fr->fr_inonum,
		(unsigned long long)fr->fr_inogen);
	printf("Time:%s\n", getctime((time_t *)&fr->fr_time.tv_sec));

	if(fr->fr_op != VX_FCL_UNLINK && ismntpt) {
		vx_path_lookup(mntpt, fr->fr_inonum, fr->fr_inogen, buf);
		printf("Path to the file:%s\n", buf);
	}

	if (fr->fr_op == VX_FCL_UNLINK || fr->fr_op == VX_FCL_RENAME ||
		fr->fr_op == VX_FCL_LINK) {
		printf("Parent inode number:%llu\t\tParent gen count:%llu\n",
			(unsigned long long)fr->fr_pinonum,
			(unsigned long long)fr->fr_pinogen);
		printf("Filename:%s", fr->fr_filename);

		if (fr->fr_op == VX_FCL_RENAME && fr->fr_newnmvalid) {
			printf("\t\tNew filename:%s\n", fr->fr_newfilename);
		} else {
			printf("\n");
		}
	} else if (fr->fr_op == VX_FCL_FILEOPEN) {
		printf("Command name:%s\n", fr->fr_cmdname);
	} else if (fr->fr_op == VX_FCL_FILESTATS) {
		fiop = fr->fr_stats;

		printf("No. of bytes read:%llu\t\t",
			(unsigned long long)fiop->nbytesread);
		printf("No. of bytes written:%llu\n",
			(unsigned long long)fiop->nbyteswrite);
		printf("Read time:%llu\t\t",
			(unsigned long long)fiop->readtime);
		printf("Write time:%llu\n",
			(unsigned long long)fiop->writetime);
		printf("No. of reads:%llu\t\t",
			(unsigned long long)fiop->nreads);
		printf("No. of writes:%llu\n",
			(unsigned long long)fiop->nwrites);
		printf("Last reset time=%s\n",
			getctime((time_t *) &fiop->lastreset.tv_sec));
	} else if (fr->fr_op == VX_FCL_EVNTMSK_CHG) {
		printf("Old event mask:0x%llx\t\t",
			(unsigned long long)fr->fr_oldmask);
		printf("New event mask:0x%llx\n",
			(unsigned long long)fr->fr_newmask);
	}

	if (fr->fr_acsinfovalid) {
		printf("Effective UID:%u\t\t", fr->fr_acsinfo->fa_euid);
		printf("Effective GID:%u\n", fr->fr_acsinfo->fa_egid);
		printf("Real UID:%u\t\t", fr->fr_acsinfo->fa_ruid);
		printf("Real GID:%u\n", fr->fr_acsinfo->fa_rgid);
		printf("Process ID:%u\n", fr->fr_acsinfo->fa_pid);
		printf("Node ID:%u\n", fr->fr_acsinfo->fa_nodeid);
	}

	printf("\n");
}

/*
* determine the fcl filename based upon whether arg is a mountpoint
* or a filename.
* store the filename in the global 'filename' and also return the same.
*/
static char*
get_fcl_filename(
	char *arg)
{
	struct stat st;
	int i;
	int error = 0;

	error = stat(arg, &st);
	if (error) {
		perror("Stat failed");
		exit(error);
	}

	bzero(filename, PATH_MAX);

        /* if directory, construct the path to the fcl file */
	if (S_ISDIR(st.st_mode)) {
		bzero(filename, PATH_MAX);
		strcpy(filename, arg);
		i = strlen(filename);
		if (filename[i-1] != '/')
			filename[i]='/';
		strcat(filename, VX_FCL_DIR);
		strcat(filename, "/");
		strcat(filename, VX_FCL_NAME);

		bzero(mntpt, PATH_MAX);
		strcpy(mntpt, arg);
		ismntpt = 1;
	} else if (S_ISREG(st.st_mode)) {
		bzero(filename, PATH_MAX);
		strcpy(filename, arg);

		bzero(mntpt, PATH_MAX);
		ismntpt = 0;
	} else {
		printf("get_fcl_filename: %s not a file or directory\n", arg);
		exit(-1);
	}

	/* verify if the file exists */
	error = stat(filename, &st);
	if (error) {
		perror("get_fcl_filename: stat of file failed");
		exit(error);
	}

	return &filename[0];
}

static void
print_menu(
	void)
{
	printf("\n\n");
	if (strlen(filename) != 0) {
		printf("Selected FCL file:%s\n\n", filename);
	} else {
		printf("No FCL file selected\n\n");
	}
	printf(" 0 - Select mountpoint or FCL file\n");
	printf(" 1 - Print info\n");
	printf(" 2 - Sync FCL file\n");
	printf(" 3 - Print all FCL Records\n");
	printf(" 4 - Print specified number of FCL Records\n");
	printf(" 5 - FCL Record Search Filter\n");
	printf(" q - Quit Program\n");
	printf("\nEnter Choice == > ");
}

/* determine the fcl filename */
static void
det_fcl_filename(
	void)
{
	unsigned int choice;
	int i;

	bzero(arg, PATH_MAX);

	while ((choice = getchar()) != '\n')
		;
	i = 0;
	printf("\nEnter mount point or file name == >");
	while ((arg[i++] = getchar()) != '\n')
		;
	arg[i-1] = 0;

	get_fcl_filename(arg);
}

/* print information about the fcl */
static void
print_info(
	void)
{
	int error = 0;
	void *fh;
	struct fcl_info fclinfo;
	uint64_t evmask;

	if (strlen(filename) == 0)
		return;

	error = vxfs_fcl_open(filename, 0, &fh);
	if (error) {
		errno = error;
		perror("Open Failed");
		exit(-1);
	}

	error = vxfs_fcl_getinfo(fh, &fclinfo);
	if (error) {
		perror("vxfs_fcl_getinfo failed");
		exit(-1);
	}
	printf("FCL version:%lld\n", (unsigned long long)fclinfo.fi_version);
	printf("FCL state:%lld\n", (unsigned long long)fclinfo.fi_state);

	error = vxfs_fcl_geteventmask(fh, &evmask);
	if (error) {
		perror("vxfs_fcl_geteventmask failed");
		return;
	}
	printf("FCL event mask:%llx\n", (unsigned long long)evmask);

	vxfs_fcl_close(fh);
}

/* sync the FCL file */
static void
sync_fcl_file(
	void)
{
	int error = 0;
	uint64_t loffset;

	if (strlen(mntpt) == 0)
		return;

	if (!is_fcl_on()) {
		printf("fcl isnt activated, cannot sync fcl file\n");
		return;
	}

	if (!ismntpt) {
		printf("\nNot a mount point,"
			"unable to sync");
		return;
	}

	error = vxfs_fcl_sync(mntpt, &loffset);
	if (error) {
		perror("sync failed");
		return;
	}

	printf("\nSynced the fcl file, offset: %lld\n",
		(unsigned long long)loffset);
}

/* read and print the fcl records */
static void
print_fcl_records(
	void)
{
	void *fh;
	char *bufptr;
	char buf[8192];
	size_t bufsize = 8192;

	uint32_t nentries = 0;
	uint32_t totentries = 0;
	struct fcl_record *fclrec;
	int error = 0;
	struct fcl_info fclinfo;

	if (maxentries == 0) {
		return;
	}

	if (strlen(filename) == 0)
		return;

	error = vxfs_fcl_open(filename, 0, &fh);
	if (error) {
		errno = error;
		perror("Open Failed");
		exit(-1);
	}

	error = vxfs_fcl_getinfo(fh, &fclinfo);
	if (error) {
		perror("vxfs_fcl_getinfo failed");
		exit(-1);
	}

	if (fclinfo.fi_state == VX_FCLS_OFF) {
		printf("fcl has been deactivated, not printing any records\n");
		return;
	}

	for (;;) {
		/*
		 * read the entries from fcl
		 */
		nentries = 0;
		bufptr = buf;
		error = vxfs_fcl_read(fh, bufptr, &bufsize,
				eventmask, &nentries);
		if (error) {
			errno = error;
			perror("Read Failed");
			goto err;
		}
		printf("\nNumber of entries read: %d\n", nentries);
		if (nentries == 0) {
			break;
		}

		totentries += nentries;

		/*
		 * process the fcl entries
		 */
		while (nentries--) {
			fclrec = (struct fcl_record *)bufptr;
			process_fcl_entry(fclrec);
			bufptr += fclrec->fr_reclen;
			if (maxentries != -1) {
				maxentries--;
				if (maxentries == 0)
					goto err;
			}
		}
	}
	if (totentries < maxentries) {
		printf("\nOnly %lld records found\n",
			(unsigned long long)totentries);
	}

err:
	vxfs_fcl_close(fh);
	if (error) {
		exit(-1);
	}
}

/* read and print specified number of fcl records */
static void
print_fcl_records_num(
	void)
{
	void *fh;
	uint32_t nentries = 0;
	struct fcl_record *fclrec;
	int error = 0;

	char *bufptr;
	char buf[128];
	size_t bufsize = 128;

	/*
	 * a small bufsize is used intentionally so as to illustrate
	 * the behaviour of vxfs_fcl_read() when it cannot fit in
	 * the specified number of records in the buffer.
	 */

	printf("\nEnter the number of records to be printed:");
	scanf("%uld", &nentries);
	printf("\nNumber of records to be printed: %lld",
		(unsigned long long)nentries);
	if (nentries <= 0) {
		return;
	}

	if (strlen(filename) == 0)
		return;

	error = vxfs_fcl_open(filename, 0, &fh);
	if (error) {
		errno = error;
		perror("Open Failed");
		exit(-1);
	}

	/*
	 * try to read all the required number of entries from fcl at once.
	 * if vxfs_fcl_read() cannot read them all at once, it returns ENOSPC
	 * and indicates the buffer size required.
	 * if this is the case, we can either try to use a buffer of the
	 * indicated size, or, use a fixed size buffer and read as many
	 * records as we can in a loop.
	 */

	bufptr = buf;
	error = vxfs_fcl_read(fh, bufptr, &bufsize, eventmask, &nentries);
	if (error == FCL_ENOSPC) {
		printf("\nNot enough space in the buffer to read all"
			"the entries at once...");
		printf("\nBuffer space required:%lld",
			(unsigned long long)bufsize);
		printf("\nInstead printing them in a loop.\n");
		maxentries = nentries;
		error = 0;
		goto err;
	} else if (error) {
		errno = error;
		perror("Read Failed");
		goto err;
	}
	printf("\nNumber of entries read: %d\n", nentries);

	/*
	 * process the fcl entries
	 */

	while (nentries--) {
		fclrec = (struct fcl_record *)bufptr;
		process_fcl_entry(fclrec);
		bufptr += fclrec->fr_reclen;
	}

err:
	vxfs_fcl_close(fh);
	if (error) {
		exit(-1);
	}
	if (maxentries != -1) {
		print_fcl_records();
	}
	maxentries = -1;
}

/* set the fcl record selection filter */
static void
set_fcl_record_filter(
	void)
{
	int filter;
	printf("\nCreate = 2, Unlink = 4, Rename = 5, Open = 23,\
		R/W stats = 24, default = -1");
	printf("\nEnter FCL Record Type == >");
	scanf("%d", &filter);

	if ((filter < 0) || (filter >= VX_FCL_MAX)) {
		filter = -1;
	}

	if (filter == 2) {
		eventmask = (uint64_t)VX_FCL_CREATE_MASK;
	} else if (filter == 4) {
		eventmask = (uint64_t)VX_FCL_UNLINK_MASK;
	} else if (filter == 5) {
		eventmask = (uint64_t)VX_FCL_RENAME_MASK;
	} else if (filter == 23) {
		eventmask = (uint64_t)VX_FCL_FILEOPEN_MASK;
	} else if (filter == 24) {
		eventmask = (uint64_t)VX_FCL_FILESTATS_MASK;
	} else {
		eventmask = (uint64_t)FCL_ALL_V4_EVENTS;
	}
}

int
main(
	int argc,
	char **argv)
{
	unsigned int choice;
	int error = 0;
	maxentries = -1;

	bzero(mntpt, PATH_MAX);
	bzero(filename, PATH_MAX);

	for (;;) {
		print_menu();
		choice = getchar();
		while (choice == '\n' || choice == '\t' || choice == ' ')
			choice = getchar();

		switch(choice) {
			case '0':
				det_fcl_filename();
				break;

			case '1':
				print_info();
				break;

			case '2':
				sync_fcl_file();
				break;

			case '3':
				print_fcl_records();
				break;

			case '4':
				print_fcl_records_num();
				break;

			case '5':
				set_fcl_record_filter();
				break;

			case 'q':
				exit(0);

			default:
				printf("\nInvalid choice\n");
				error = 1;
				break;
		}
		if(error)
			break;
	}
	return 0;
}

