/* $Id: rnl.c,v 4.6.14.1 2007/09/14 03:50:41 amukher2 Exp $ */
/* #ident "@(#)vxfs:$RCSfile: rnl.c,v $	$Revision: 4.6.14.1 $" */

/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

/*
 * EXAMPLES
 * The following C source code is similar to the vxlsino
 * command, which used vxfs_inotopath() to obtain path names.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <errno.h>
#include <sys/types.h>
#include <time.h>

#ifdef __linux__
#include <stdint.h>
#include <string.h>
#endif

#include <vxfsutil.h>

static void usage(void);

char		*cmd;
extern int	optind;
extern char	*optarg;

int
main(
	int	argc,
	char	**argv)
{
	uint64_t	inode;
	uint32_t	gen = 0;
	int		error, nentries, all = 0, i, c;
	char		**bufp, *mountpath;

	cmd = argv[0];
	while ((c = getopt(argc, argv, "ag:")) != -1) {
		switch (c) {
		case 'a':
			all = 1;
			break;

		case 'g':
			if (sscanf(optarg, "%d", &gen) != 1)
				usage();
			break;

		default:
			break;
		}
	}
	if ((argc - optind) != 2) {
		usage();
	}
	argc -= optind;
	argv = &argv[optind];
	sscanf(argv[0], "%llu", (unsigned long long *) &inode);
	mountpath = argv[1];

	if ((error =
	     vxfs_inotopath_gen(mountpath, inode, gen, all, &bufp,
				&nentries)) == 0) {
		for (i = 0; i < nentries; i++)
			printf("%s\n", bufp[i]);
	} else {

		/*
		 * check for older routine on vxfs 4.0
		 */

		if (error == ENOTSUP) {
			if ((error = vxfs_inotopath(mountpath, inode, all,
						    &bufp, &nentries)) == 0) {
				for (i = 0; i < nentries; i++)
					printf("%s\n", bufp[i]);
				goto out;
			}
		}
		fprintf(stderr, "vxfs_inotopath: %s\n", strerror(error));
		exit(1);
	}

      out:
	for (i = 0; i < nentries; i++) {
		free(bufp[i]);
	}

	free(bufp);
	exit(0);
}

void
usage(void)
{
	fprintf(stderr,
		"usage: %s [-a] [-g <generation #>] <inode#> <mountpoint>\n",
		cmd);
	exit(1);
}
