/* $Id: common_subs.c,v 4.4.14.3 2007/09/19 21:52:26 tko Exp $ */
/* #ident "@(#)vxfs:$RCSfile: common_subs.c,v $	$Revision: 4.4.14.3 $" */

/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/unistd.h>
#include <errno.h>
#include <dirent.h>
#include <sys/param.h>
#include <mntent.h>

#include <time.h>
#include <string.h>
#include <stdint.h>
#include <vxfsutil.h>

#include <linux/types.h>

#ifndef _syscall3
#define _syscall3(type,name,type1,arg1,type2,arg2,type3,arg3)	\
type name(type1 arg1, type2 arg2, type3 arg3)			\
{								\
								\
	return syscall(__NR_##name, arg1, arg2, arg3);		\
}
#endif

_syscall3(int, getdents, uint, fd, struct dirent *, dirp, uint, count);

#include "common_subs.h"

/* global flag */
int	func_flag = QUERY;
char	*gl_data_policy = NULL, *gl_meta_policy = NULL;

static char    *vol_order[] = {
	"FSAP_ORDER_ASGIVEN",
	"FSAP_ORDER_LEASTFULL",
	"FSAP_ORDER_ROUNDROBIN",
};

int
create_policy(int fd,
	      char *policy_name,
	      char *vol_name[],
	      int count)
{
	struct fsap_info ap;
	int		error = 0, i;

	strcpy((char *) ap.ap_name, policy_name);
	ap.ap_flags = FSAP_INHERIT;
	ap.ap_order = FSAP_ORDER_ASGIVEN;
	ap.ap_ndevs = count;

	for (i = 0; i < count; i++)
		strcpy(ap.ap_devs[i], vol_name[i]);

	if ((error = vxfs_ap_define(fd, &ap, 0)) != 0) {
		printf(" \nError  defining policy, %s", strerror(error));
	   exit(error);

	}
}

int
assign_policy_fs(int fd,
		 char *dpolicy,
		 char *mpolicy)
{

	int	error = 0;

	if ((error = vxfs_ap_assign_fs(fd, dpolicy, mpolicy)) != 0) {
		printf(" \nError  assigning policy, %s", strerror(error));
	   exit(error);
	}
}

int
assign_policy_file(int fd,
		   char *dpolicy,
		   char *mpolicy,
		   int flags)
{

	int	error = 0;

	if ((error = vxfs_ap_assign_file(fd, dpolicy, mpolicy, flags)) != 0) {
		printf(" \nError  assigning policy, %s", strerror(error));
	   exit(error);
	}
}

int
query_policy(int fd,
	     char *policy_name)
{

	struct fsap_info ap;
	int		i, error = 0;

	printf("\nAllocation Policy ==>  %s", policy_name);

	strcpy((char *) ap.ap_name, policy_name);

	if ((error = vxfs_ap_query(fd, &ap)) != 0) {
		printf("\nError querying policy %s, %s", policy_name,
		       strerror(error));
		exit(error);
	}
	printf("\n\tflags: 0x%x", ap.ap_flags);
	printf("\n\torder: %s", vol_order[ap.ap_order]);
	printf("\n\tndevs: %d", ap.ap_ndevs);
	printf("\n\tVolumes:");
	for (i = 0; i < ap.ap_ndevs; i++) {
		printf("  %s", ap.ap_devs[i]);
	}
	printf("\n");
	return error;
}

int
query_file(char *fname)
{
	int		fd, error = 0;
	uint32_t	flags;
	char		datapolicy[FSAP_NAMESZ + 1];
	char		metapolicy[FSAP_NAMESZ + 1];

	if ((fd = open(fname, O_RDONLY)) < 0) {
		printf(" \nError  opening file %s, %s", fname, strerror(errno));
		return error;
	}
	if (vxfs_ap_query_file(fd, datapolicy, metapolicy, &flags) != 0) {
		close(fd);
		exit(error);
	}
	printf("\n File %s has Data Policy(%s) and Meta Policy(%s)",
	       fname, datapolicy, metapolicy);
	close(fd);
	return error;
}

void
print_dir_contents(char *path,
		   struct dirent *dir_buf,
		   int size)
{
	struct dirent  *dptr = dir_buf;
	int		count = 0, error, fd;
	long		tmp = (long) dir_buf;
	struct stat	statb;
	char		pathname[MAXPATHLEN + 1];

	while (count < size) {
		strcpy(pathname, path);
		strcat(pathname, "/");
		strcat(pathname, dptr->d_name);

		stat(pathname, &statb);
		switch (statb.st_mode & S_IFMT) {
		case S_IFDIR:
			if (!strcmp(dptr->d_name, ".") ||
			    !strcmp(dptr->d_name, "..")

/* comment out lost & found, because the FCL log was not being
 * reallocated. This caused the volumes to appear empty. But when the
 * volume_????? programs were run the volumes were busy and programs
 * would fail
 */
			    /* ||
			    !strcmp(dptr->d_name, "lost+found") */  )
				break;
/* query, assign */
			if (func_flag == QUERY)
				query_file(pathname);
			search_dir(pathname);
			break;

		case S_IFREG:

/* query, assign or enforce */
			if (func_flag == QUERY)
				query_file(pathname);
			else {
				if ((fd = open(pathname, O_RDONLY)) < 0)
					break;

				assign_policy_file(fd, gl_data_policy,
						   gl_meta_policy, 0);

				if ((error =
				     vxfs_ap_enforce_file(fd, gl_data_policy,
							  gl_meta_policy)) != 0)
					printf(" \nError enforcing policy %s",
					       strerror(error));

				close(fd);
			}
			break;
		}

		count += dptr->d_reclen;
		tmp = tmp + dptr->d_reclen;
		dptr = (struct dirent *) tmp;
	}
}

void
search_dir(char *file_name)
{
	int		fd, dir_size;
	struct dirent	dir_buf[512];
	char		error_buf[256];

	/*
	 * search all directory entries.
	 */
	if ((fd = open(file_name, O_RDONLY)) < 0)
		return;

	while (1) {
		if ((dir_size = getdents(fd, dir_buf, sizeof(dir_buf))) > 0) {

			print_dir_contents(file_name, dir_buf, dir_size);

		} else if (dir_size == -1) {
			sprintf(error_buf,
				" getdents() error on directory %s", file_name);
			perror(error_buf);
			break;
		} else {	/* EOF */
			break;
		}
	}
	close(fd);
}

int
check_volumes(int fd,
	      char *fsname)
{

	int		i, error = 0, count;
	struct fsdev_info *infop;

	count = FSAP_MAXDEVS;
	if ((infop = (struct fsdev_info *) malloc(sizeof(struct fsdev_info)
						  * count)) == NULL) {
		printf("\r fatal error , unable to malloc() ");
		return 1;
	}

	memset(infop, 0, sizeof(struct fsdev_info) * count);
	if ((error = vxfs_vol_enumerate(fd, &count, infop)) != 0) {

		printf("Error finding volumes %s", strerror(error));
		exit(error);
	} else {
		printf("\nFiles system Name  -- %s ", fsname);

		printf("\nVolume Name                 Id     Size     "
		       "Free     Avail");
		for (i = 0; i < count; i++) {
			printf("\n %25s   %d  %lld   %lld   %lld",
			       infop[i].dev_name, infop[i].dev_id,
			       infop[i].dev_size / 1024,
			       infop[i].dev_free / 1024,
			       infop[i].dev_avail / 1024);
		}

	}
	printf("\n");
	free(infop);
	return error;
}

static int
verify_volume(int fd,
	      char *fsname,
	      char *vol_name)
{

	int		i, error = 1, err,  count;
	struct fsdev_info *infop;

	count = FSAP_MAXDEVS;
	if ((infop = (struct fsdev_info *) malloc(sizeof(struct fsdev_info)
						  * count)) == NULL) {
		printf("\r fatal error , unable to malloc() ");
		return 1;
	}

	memset(infop, 0, sizeof(struct fsdev_info) * count);
	if ((err = vxfs_vol_enumerate(fd, &count, infop)) != 0) {

		printf("\nError finding volumes, %s", strerror(err));
		error = err;
	} else {
		for (i = 0; i < count; i++) {
			if (!strcmp(infop[i].dev_name, vol_name)) {
				error = 0;
				break;
			}
		}
	}

	free(infop);
	return error;
}

int
enter_names(char **fsname,
	    char *vols[],
	    int nvols)
{
	struct mntent	*mp;
	FILE		*fpmnt = NULL;
	char		choice, buf[256];
	int		i, fd = -1, num_vols;
	int		entry_found = 0;

	while (*fsname == NULL) {
		/*
		 * select mount point
		 */

	      reselect:
		fpmnt = setmntent(MNTTAB, "r");
		if (!fpmnt) {
			printf("\nUnable to open mount table");
			exit(1);
		}

		do {
			i = 0;
			printf("\nEnter Mount Point ==>");

			while ((buf[i++] = getchar()) != '\n');
			buf[i - 1] = 0;

		} while (i <= 1);

		while ((mp = getmntent(fpmnt)) != NULL) {
			if ((strcmp (mp->mnt_dir, buf) == 0) && \
			    (strcmp (mp->mnt_type, "vxfs") == 0)) {
				entry_found = 1;
			}
		}
		endmntent(fpmnt);

		if (entry_found) {
			if ((fd = open(mp->mnt_dir, O_RDONLY)) < 0) {
				printf("\nNot a VxFS File System");
				continue;
			} else
				*fsname = strdup(buf);
			close(fd);
			break;
		} else
			printf("\nNot a VxFS File System");
		entry_found = 0;
	}

	if ((fd = open(*fsname, O_RDONLY)) < 0) {
		printf(" \nError  opening file %s, %s", *fsname,
		       strerror(errno));
		exit(1);
	}

	/*
	 * get MDS volume name
	 */
	num_vols = 0;
	while (num_vols < nvols) {
		/*
		 * check return code, is the FS mounted on a file set
		 */
		/*
		 * Inapproriate Ioctl for device
		 */
		if (check_volumes(fd, *fsname) == ENOTTY)
			goto reselect;

		i = 0;
		printf("\nEnter MDS Volume name ==>");

		while ((buf[i++] = getchar()) != '\n');
		buf[i - 1] = 0;

		if (strlen(buf) == 0)
			break;

		if (verify_volume(fd, *fsname, buf) == 1) {
			printf("\nNot a valid volume name");
			continue;
		} else
			vols[num_vols++] = strdup(buf);
	}

	close(fd);
	return num_vols;
}
