/* $Id: policy_create.c,v 4.8 2006/03/17 15:20:33 phils Exp $  */
/* #ident "@(#)vxfs:$RCSfile: policy_create.c,v $	$Revision: 4.8 $" */
/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#ifdef __linux__
#include <string.h>
#include <stdint.h>
#endif

#include <time.h>
#include <vxfsutil.h>

#include "common_subs.h"

int
main(
	int	argc,
	char	**argv)
{

	char		*path;
	int		i, count, fs, dir1, dir2, error, nvols;
	char		datapolicy[FSAP_NAMESZ + 1];
	char		metapolicy[FSAP_NAMESZ + 1];
	char		*fsname = NULL;
	char		home_dir1[256];
	char		home_dir2[256];
	char		*fs_metapolicy = "sdk_metapolicy";
	char		*fs_datapolicy = "sdk_datapolicy0";
	char		*dir1_datapolicy = "sdk_datapolicy1";
	char		*dir2_datapolicy = "sdk_datapolicy2";
	char		*vols[10];

	printf("\nSelect File Sytem and volume for FS meta policy\n");
	nvols = enter_names(&fsname, vols, 1);

	strcpy(home_dir1, fsname);
	strcat(home_dir1, "/sdk_user1");

	strcpy(home_dir2, fsname);
	strcat(home_dir2, "/sdk_user2");

	fs = open(fsname, O_RDONLY);
	if (fs < 0) {
		printf("\nError opening file, %s", strerror(errno));
		exit(1);
	}

	create_policy(fs, fs_metapolicy, vols, 1);

	/*
	 * create directories, which will inherit the FS policies
	 */

	dir1 = mkdir(home_dir1, 0x177);
	if ((dir1 < 0) &&  (errno != EEXIST)){
		printf("\nError creating, %s", strerror(errno));
		exit(1);
	}
	dir1 = open(home_dir1, O_RDONLY);
	if (dir1 < 0) {
		printf("\nError opening file, %s", strerror(errno));
		exit(1);
	}

	dir2 = mkdir(home_dir2, 0x177);
	if ((dir2 < 0) &&  (errno != EEXIST)){
		printf("\nError creating file, %s", strerror(errno));
		exit(1);
	}
	dir2 = open(home_dir2, O_RDONLY);
	if (dir2 < 0) {
		printf("\nError opening file, %s", strerror(errno));
		exit(1);
	}

	/*
	 * create and assign allocation policies to directories
	 */

	printf("\nSelect volume for directory %s data policy\n", home_dir1);
	nvols = enter_names(&fsname, vols, 1);
	create_policy(fs, dir1_datapolicy, vols, 1);

	printf("\nSelect volume for directory %s data policy\n", home_dir2);
	nvols = enter_names(&fsname, vols, 1);
	create_policy(fs, dir2_datapolicy, vols, 1);

	/*
	 * assign allocation policies to directories
	 */

	assign_policy_file(dir1, dir1_datapolicy, dir1_datapolicy, 0);
	assign_policy_file(dir2, dir2_datapolicy, dir2_datapolicy, 0);

	/*
	 * create and assign some allocation policies for file system
	 */

	printf("\nSelect volume for FS data policy\n");
	nvols = enter_names(&fsname, vols, 1);

	create_policy(fs, fs_datapolicy, vols, 1);
	assign_policy_fs(fs, fs_datapolicy, fs_metapolicy);

	/*
	 * query for current policy definitions
	 */

	printf("\nPolicies assigned to File System %s", fsname);
	query_policy(fs, fs_datapolicy);
	query_policy(fs, fs_metapolicy);

	printf("\nPolicies assigned to Directory %s", home_dir1);
	query_policy(fs, dir1_datapolicy);

	printf("\nPolicies assigned to Directory %s", home_dir2);
	query_policy(fs, dir2_datapolicy);

	close(fs);
	close(dir1);
	close(dir2);
	exit(0);
}
