/* $Id: volume_deencapsulate.c,v 4.7 2006/03/17 15:20:34 phils Exp $  */
/* #ident "@(#)vxfs:$RCSfile: volume_deencapsulate.c,v $	$Revision: 4.7 $" */
/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#ifdef __linux__
#include <string.h>
#include <stdint.h>
#endif

#include <time.h>
#include <vxfsutil.h>

#include "common_subs.h"

int
main(
	int	argc,
	char	**argv)
{
	int			i, fd, error, nvols;
	char			*fsname = NULL, *new_vol_name[1];
	struct fsdev_info	before_remove_vol, after_remove_vol;
	char			encapsulate_name[500];

	printf("\nEnter name of File System (mounted on a Volume set)"
	       " and volume to deencapsulate");
	printf("\nProgram restores volume from volume_encapsulate program");
	printf("\nVolume will be deencapsulated first, "
	       "then added to the volume set");

	nvols = enter_names(&fsname, new_vol_name, 1);
	strcpy(encapsulate_name, fsname);
	strcat(encapsulate_name, "/sdk_encap");

	if ((fd = open(fsname, O_RDONLY)) < 0) {
		printf(" \nError  opening file, %s", strerror(errno));
		exit(1);
	}
	printf("\nVolume set sizes for File System %s", fsname);
	check_volumes(fd, fsname);

	/*
	 * Stat volume, before we deecapsulate, so we can readd to volume set
	 */

	if ((error =
	     vxfs_vol_stat(fd, new_vol_name[0], &before_remove_vol)) == 0) {

		/*
		 * deencapsulate the volume
		 */

		printf("\n\nDeencapsulating File %s", encapsulate_name);
		if ((error = vxfs_vol_deencapsulate(encapsulate_name)) != 0) {
			printf("\nError, %s,  Deencapsulating File %s",
			       strerror(error), encapsulate_name);
			printf("\n\nVolume set sizes for File System %s",
			       fsname);
			check_volumes(fd, fsname);
			close(fd);
			exit(1);
		} else {

			/*
			 * restore the volume
			 */

			check_volumes(fd, fsname);

			printf("\nRestoring Volume %s to File system %s",
			       new_vol_name[0], fsname);

			if ((error = vxfs_vol_add(fd, new_vol_name[0],
						  before_remove_vol.
						  dev_size)) != 0) {
				printf("\nError, %s, adding Volume(%s) on "
					"File System " "%s",strerror(error),
					new_vol_name[0], fsname);
				close(fd);
				exit(1);
			} else {
				printf("\n\nVolume set sizes for File"
				       " System %s", fsname);
				check_volumes(fd, fsname);
			}
		}
	} else {
		printf("\nError, %s,  stat'ing Volume, %s, on File System %s",
		       strerror(error), new_vol_name[0], fsname);

		printf("\nWill not attempt to deencapsulate the volume\n");
		close(fd);
		exit(1);
	}
	printf("\n");
	close(fd);
	exit(0);
}
