/* $Id: policy_enforce.c,v 4.7 2006/03/17 15:20:38 phils Exp $   */
/* #ident "@(#)vxfs:$RCSfile: policy_enforce.c,v $	$Revision: 4.7 $" */

/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#ifdef __linux__
#include <string.h>
#include <stdint.h>
#endif

#include <time.h>
#include <vxfsutil.h>

#include "common_subs.h"

/*
 * 0) On existing files
 * 1) create some files in a directory
 * 2) create some data/meta policy
 * 3) assign to a file
 * 4) enforce the policy
 * 5) watch the file move volumes, though in same directory
 */

int
main(
	int	argc,
	char	**argv)
{
	int		fd, error, nvols;
	char		*datapolicy = "sdk_datapool";
	char		*metapolicy = "sdk_metapool";
	char		datapolicy2[FSAP_NAMESZ + 1];
	char		metapolicy2[FSAP_NAMESZ + 1];
	char		*fsname = NULL;
	char		*vols[10];

	printf("\nProgram will enforce reallocation of files"
	       " using allocations policies.\n");

	printf("\nEnter name of File System and destination volume "
	       "for Data Policy.\n");
	nvols = enter_names(&fsname, vols, 1);

	if ((fd = open(fsname, O_RDONLY)) < 0) {
		printf("\nError opening mount point, %s", strerror(errno));
		exit(1);
	}
	create_policy(fd, datapolicy, vols, nvols);

	printf("\nEnter name of destination volume for Meta Policy.\n");
	nvols = enter_names(&fsname, vols, 1);

	create_policy(fd, metapolicy, vols, nvols);

	printf("\n Volume Usage before reallocation");
	check_volumes(fd, fsname);

	func_flag = ENFORCE;
	gl_meta_policy = metapolicy;
	gl_data_policy = datapolicy;

	printf("\n Enforcing Policy, this may take some time");
	search_dir(fsname);

	printf("\n Volume Usage after reallocation");
	check_volumes(fd, fsname);

	close(fd);
	exit(0);
}
