/* $Id: volume_resize.c,v 4.6.14.1 2007/09/14 04:18:27 amukher2 Exp $  */
/* #ident "@(#)vxfs:$RCSfile: volume_resize.c,v $	$Revision: 4.6.14.1 $" */
/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#ifdef __linux__
#include <string.h>
#include <stdint.h>
#endif

#include <time.h>
#include <vxfsutil.h>

#include "common_subs.h"

int
main(
	int	argc,
	char	**argv)
{
	int			fd, error, nvols;
	char			*fsname = NULL, *resize_vol_name[1];
	struct fsdev_info	before_resize_vol, after_resize_vol;

	printf("\nEnter name of File System (mounted on a Volume set)"
	       " and volume to resize");
	printf("\n\nVolume Must have no files allocated on it for this "
	       "program to be successful");
	printf("\nTry using policy_enforce program to reallocate files first");

	nvols  = enter_names(&fsname, resize_vol_name, 1);

	if ((fd = open(fsname, O_RDONLY)) < 0) {
		printf("\nError  opening file, %s", strerror(errno));
		exit(1);
	}

	printf("\nVolume set sizes for File System %s", fsname);
	check_volumes(fd, fsname);

	if ((error = vxfs_vol_stat(fd, resize_vol_name[0],
				   &before_resize_vol)) != 0) {
		printf("\nError, %s,  stat'ing Volume on File System %s",
		       strerror(error), fsname);
		close(fd);
		exit(1);
	}

	printf("\n\n Volume size before resizing");
	printf("\n %25s   %d  %llu   %llu   %llu",
	       before_resize_vol.dev_name, before_resize_vol.dev_id,
	       (unsigned long long)before_resize_vol.dev_size / 1024,
	       (unsigned long long)before_resize_vol.dev_free / 1024,
	       (unsigned long long)before_resize_vol.dev_avail / 1024);

	/*
	 * shrink the volume
	 */

	if ((error =
	     vxfs_vol_resize(fd, resize_vol_name[0],
			     before_resize_vol.dev_size / 2)) != 0) {

		printf("\nError, %s,  resizing Volume(%s) on File System %s",
		       strerror(error), before_resize_vol.dev_name, fsname);
		close(fd);
		exit(1);
	}

	if ((error =
	     vxfs_vol_stat(fd, resize_vol_name[0],
			   &after_resize_vol)) != 0) {

		printf("\nError, %s,  stat'ing Volume on File System %s",
		       strerror(error), fsname);
		close(fd);
		exit(1);
	}

	printf("\n\n Volume size after shrinking");
	printf("\n %25s   %d  %llu   %llu   %llu",
	       after_resize_vol.dev_name, after_resize_vol.dev_id,
	       (unsigned long long)after_resize_vol.dev_size / 1024,
	       (unsigned long long)after_resize_vol.dev_free / 1024,
	       (unsigned long long)after_resize_vol.dev_avail / 1024);

	/*
	 * grow the volume
	 */

	if ((error = vxfs_vol_resize(fd, resize_vol_name[0],
				     before_resize_vol.dev_size)) != 0) {

		printf("\nError, %s,  resizing Volume(%s) on File System %s",
		       strerror(error), resize_vol_name[0], fsname);
		close(fd);
		exit(1);
	}

	if ((error = vxfs_vol_stat(fd, resize_vol_name[0],
				   &after_resize_vol)) != 0) {
		printf("\nError, %s,  stat'ing Volume on File System %s",
		       strerror(error), fsname);
		exit(1);
	}

	printf("\n\n Volume size after growing");
	printf("\n %25s   %d  %llu   %llu   %llu",
	       after_resize_vol.dev_name, after_resize_vol.dev_id,
	       (unsigned long long)after_resize_vol.dev_size / 1024,
	       (unsigned long long)after_resize_vol.dev_free / 1024,
	       (unsigned long long)after_resize_vol.dev_avail / 1024);

	printf("\n\nVolume set sizes for File System %s", fsname);
	check_volumes(fd, fsname);

	printf("\n");
	close(fd);
	exit(0);
}
