/* $Id: common_subs.c,v 4.2.14.1 2007/09/20 14:22:04 steveta Exp $ */
/* #ident "@(#)vxfs:$RCSfile: common_subs.c,v $	$Revision: 4.2.14.1 $" */
/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <grp.h>
#include <pwd.h>
#include <dirent.h>
#include <time.h>
#include <errno.h>

#ifdef __linux__
#include <stdint.h>
#include <string.h>
#endif

#include <vxfsutil.h>
#include "common_subs.h"

/*
 * Routine to print the file stats for the Attribute File
 * directory entries.
 */

static void
print_dir_stats(
	int fd,
	char *filename,
	struct dirent *dptr)
{
	char		error_buf[256];
	struct stat	sb;
	int		nfd;
	mode_t		mode = 0x777;
	struct passwd	*u;
	struct group	*g;
	char		time[26];

	/*
	 * open named attribute file
	 */

	if ((nfd =
	     vxfs_nattr_open(fd, dptr->d_name, O_RDONLY, mode)) < 0) {
		sprintf(error_buf,
			"\nError Opening Attribute File %s/./%s ",
			filename, dptr->d_name);
		perror(error_buf);
		return;
	}

	if (fstat(nfd, &sb) < 0) {
		sprintf(error_buf,
			"\nError Stat'ing Attribute File %s/./%s ",
			filename, dptr->d_name);
		perror(error_buf);
		close(nfd);
		return;
	}

	u = getpwuid(sb.st_uid);
	g = getgrgid(sb.st_gid);
	strcpy(time, ctime(&sb.st_ctime));
	time[24] = '\0';	/* get rid of the newline */
	printf("\n0x%x %s %s %lldK %s %s",
	       sb.st_mode & ~S_IFMT, u->pw_name, g->gr_name,
	       sb.st_size / 1024, time, dptr->d_name);
	close(nfd);
	return;
}

int
print_nattr_data(
	int		fd,
	char		*filename,
	mode_t		mode,
	int		stats)
{
	int		dfd;
	char		error_buf[256];
	DIR		*dirp = NULL;
	struct dirent	entry, *result;
	int		err = 0;

	/*
	 * read the file attribute directory
	 */

	if ((dfd = vxfs_nattr_open(fd, ".", O_RDONLY, mode)) < 0) {
		sprintf(error_buf,
			"Error Opening Attribute directory"
			" %s/. ", filename);
		perror(error_buf);
		return 1;
	}

	fchdir(dfd);
	if ((dirp = opendir(".")) == NULL) {
		sprintf(error_buf, "opendir() error on Attribute "
				   "directory %s/./ ", filename);
		perror(error_buf);
		err = errno;
		goto out;
	}

	printf("\r\nAttribute Directory contents for %s \n", filename);

	while ((err = readdir_r(dirp, (struct dirent *) &entry,
				&result)) == 0) {
		if (result == NULL) {
			break;
		}
		if (stats) {
			print_dir_stats(fd, filename, result);
		} else {
			printf("\r\nAttribute File => %s", result->d_name);
		}
	}
	closedir(dirp);

out:
	close(dfd);
	return err;
}
