/* $Id: nattr_create.c,v 4.9 2006/03/17 15:20:47 phils Exp $ */
/* #ident "@(#)vxfs:$RCSfile: nattr_create.c,v $	$Revision: 4.9 $" */
/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <errno.h>
#include <fcntl.h>
#include <time.h>

#ifdef __linux__
#include <stdint.h>
#include <string.h>
#include <linux/types.h>
#endif

#include <vxfsutil.h>
#include "common_subs.h"

static void
usage(
	char **argv)
{
	printf("\n");
	printf("Usage:\n");
	printf("%s  /mntpnt/....../filename\n", argv[0]);
	printf("\n");
	printf("\n File [mntpt] must be a mounted vxfs 4.0"
			"(or later) filesystem.");
	printf("\n Program will create a file[filename], then");
	printf("\n create several named attributes, then will");
	printf("\n query the attributes directory and print the results.");
	printf("\n");
	exit(1);
}

/*
 * This program is intended to demonstate the Named Attribute feature
 * of VXFS 4.0. The library libvxfsutil provides API calls for manipulating
 * Named attributes.  This program utilizes this API interface and demonstates
 * how to use the API calls to create attribute files.
 *
 * This sample program will create a file, then create several associated
 * named attribute files.
 *
 * nattr /vxfstest1/nattrfile
 *
 * NOTE:
 * After running the program, use the ls command to look at the file size
 * of the created file. Even though 20 attribute files are created, each
 * of size 2K( 20 * 2K = 40K) the file size from the ls command is only
 * 1K( the 1K is from the 1K write to the regular file).
 */

int
main(
	int	argc,
	char	**argv)
{
	int		fd;
	int		nfd;
	int		i;
	char		*filename;
	char		attrname[256];
	char		dst[256];
	mode_t		mode = 0x777;
	int		attr = 0;
	char		c;
	char		buf[2048];
	char		error_buf[256];
	int		err = 0;

	if (argc <= 1 || argc > 2) {
		usage(argv);
	}

	filename = argv[1];

	/*
	 * create/truncate and open a file
	 */

	if ((fd = open(filename, O_RDWR | O_CREAT | O_TRUNC, mode)) < 0) {
		sprintf(error_buf, "%s, Error Opening File %s ", argv[0],
			filename);
		perror(error_buf);
		exit(-1);
	}
	printf("\nCreated file %s", filename);

	/*
	 * you can write to the regular file as usual
	 */

	memset(buf, 0x61, 1024);
	write(fd, buf, 1024);

	/*
	 * create several  named attributes
	 */

	for (i = 0; i < 20; i++) {
		sprintf(attrname, "%s%d", "attribute", i);
		nfd = vxfs_nattr_open(fd, attrname, O_WRONLY | O_CREAT, mode);
		if (nfd < 0) {
			sprintf(error_buf,
				"%s, Error Opening Attribute file %s/./%s ",
				argv[0], filename, attrname);
			perror(error_buf);
			close(fd);
			exit(-1);
		}

		/*
		 * write some data to the  atribute file
		 */

		memset(buf, 0x41 + i, 2048);
		write(nfd, buf, 2048);
		close(nfd);
	}

	printf("\nCreated %d attribute files for %s", i, filename);

	/*
	 * note that the fd file size stays the same but the df blocks are being
	 * used up with the attribute files
	 */

	/*
	 * open the Attributes files directory and get all the
	 * directory entries
	 */

	err = print_nattr_data(fd, filename, mode, 0);

	close(fd);
	exit(0);
}
