/* $Id: nattr_display.c,v 4.12 2006/03/17 15:20:49 phils Exp $ */
/* #ident "@(#)vxfs:$RCSfile: nattr_display.c,v $	$Revision: 4.12 $" */
/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <errno.h>
#include <fcntl.h>

#ifdef __linux__
#include <string.h>
#include <stdint.h>
#include <linux/types.h>
#endif

#include <vxfsutil.h>
#include "common_subs.h"

static int err_rtn = 0;

static void
usage(
	char	**argv)
{
	printf("\n");
	printf("Usage:\n");
	printf("%s  [/mntpnt/....../filename]\n", argv[0]);
	printf("\n");
	printf("\n File [mntpt] must be a mounted vxfs 4.0"
			"(or later) filesystem.");
	printf("\n Program will use [filename] as initial value.");
	exit(1);
}

static void
print_menu(
	char *filename,
	char *attrname)
{

	printf("\n Named Attribute Display Program ");
	printf("\n 0 - Select New File Name (Current = %s)", filename);
	printf("\n 1 - Show Attrribute Files");
	printf("\n 2 - Select Attribute File (Current = %s)", attrname);
	printf("\n 3 - Print Attribute File Contents");
	printf("\n 4 - quit");
	printf("\n\nEnter choice ==> ");
}

static void
read_contents(
	char *filename,
	char *attrname)
{
	int		fd, nfd;
	int		i, j, count;
	char		error_buf[256];
	mode_t		mode = 0x777;
	char		buf[2048];

	/*
	 * open a file
	 */

	if ((fd = open(filename, O_RDONLY, mode)) < 0) {

		sprintf(error_buf, "Error Opening File %s ", filename);
		perror(error_buf);
		return;
	}

	/*
	 * read named attribute file
	 */

	nfd = vxfs_nattr_open(fd, attrname, O_RDONLY, mode);
	if (nfd < 0) {
		sprintf(error_buf,
			"Error Opening Attribute file %s/./%s ",
			filename, attrname);
		perror(error_buf);
		close(fd);
		err_rtn = errno;
		return;
	}

	/*
	 * read some data to the  atribute file
	 */

	printf("\nData from Attribute file %s/./%s\n", filename, attrname);
	count = read(nfd, buf, 2048);
	for (i = 0; i < count; i += 80) {
		for (j = i; j < i + 80; j++)
			printf("%c", buf[i]);
		printf("\n");
	}
	close(nfd);
	close(fd);
}

static void
show_dir(
	char *filename)
{

	int		fd;
	char		error_buf[256];
	mode_t		mode = 0x777;

	/*
	 * open a file
	 */

	if ((fd = open(filename, O_RDONLY, mode)) < 0) {
		sprintf(error_buf, "Error Opening File %s ", filename);
		perror(error_buf);
		return;
	}

	/*
	 * open the Attributes files directory and get all the
	 * directory entries
	 */

	err_rtn = print_nattr_data(fd, filename, mode, 0);

	close(fd);
	return;
}

/*
 * This program is intended to demonstate the Named Attribute feature
 * of VXFS 4.0. The library libvxfsutil provides API calls for manipulating
 * Named attributes.  This program can be used to view Attribute files.
 *
 * This sample program can be used to open a file, view the attribute files
 * associated with it and display the dats in the file( assumes printable data).
 *
 * nattr_display /vxfs_mnpt/nattrfile
 *
 */

int
main(
	int	argc,
	char	**argv)
{
	int		fd;
	int		err = 0;
	int		i;
	char		filename[256];
	char		attrname[256];
	int		choice;
	char		buf[2048];
	struct dirent	dir_buf[512];
	char		error_buf[256];

	memset(attrname, 0, 256);
	memset(filename, 0, 256);

	if (argc > 2) {
		usage(argv);
	} else if (argc == 2) {
		strcpy(filename, argv[1]);
	}

	while (1) {

		print_menu(filename, attrname);
		choice = getchar();
		switch (choice) {

		case '0':
			/*
			 * select regular file
			 */

			while ((choice = getchar()) != '\n');
			i = 0;
			printf("\nEnter File Name ==>");
			while ((filename[i++] = getchar()) != '\n');
			filename[i - 1] = 0;
			continue;
			break;

		case '1':
			/*
			 * show directory
			 */

			show_dir(filename);
			break;

		case '2':
			/*
			 * select attribute file
			 */

			while ((choice = getchar()) != '\n');
			i = 0;
			printf("\nEnter Attribute File Name ==>");
			while ((attrname[i++] = getchar()) != '\n');
			attrname[i - 1] = 0;
			printf("\n%s", attrname);
			continue;
			break;

		case '3':
			/*
			 * print file contents
			 */

			read_contents(filename, attrname);
			break;

		case '4':
			/*
			 * quit
			 */

			exit(err_rtn);
			break;

		default:
			break;

		}
		while ((choice = getchar()) != '\n');
	}
}
