/* $Id: nattr_menu_cmds.c,v 4.9 2006/03/17 15:20:50 phils Exp $ */
/* #ident "@(#)vxfs:$RCSfile: nattr_menu_cmds.c,v $	$Revision: 4.9 $" */
/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <errno.h>
#include <fcntl.h>
#include <time.h>

#ifdef __linux__
#include <string.h>
#include <stdint.h>
#include <linux/types.h>
#endif

#include <vxfsutil.h>
#include "common_subs.h"

extern int	optind;
extern char	*optarg;

static void
print_menu(
	char	*filename,
	char	*attrname,
	char	*new_attrname)
{
	printf("\nMenu choices\n\n");

	printf(" 0 - Select File Name          (filename = %s)\n", filename);
	printf(" 1 - Select Attribute Name     (attrname = %s)\n", attrname);
	printf(" 2 - Select New Attribute Name (new attrname = %s)\n\n",
	       new_attrname);

	printf(" c : creates attribute for filename\n");
	printf(" d : lists attribute directory for filename\n");
	printf(" u : unlinks attribute from filename\n");
	printf(" l : links attribute for filename\n");
	printf(" R : reads from attribute for filename\n");
	printf(" W : writes to attribute for filename\n");
	printf(" r : renames attribute for filename\n");
	printf(" s : lists stats for each attribute file\n");
	printf(" t : Set attribute access/modify time stamp for filename\n");
	printf(" q : Quit Program\n");
	printf("\n    Enter Choice ==> ");
}

/*
 * This program is intended to demonstate the usage of the Named Attribute
 * feature of the VxFs 4.0 file system.  The libvxfsutil library provides
 * an application interface for Named Attributes. This program utilizes these
 * API calls to create, delete, and list Named Attributes. Take a look at each
 * branch of the case statement below in the main program for an example of
 * how to use the API calls.
 *
 * About this program
 * This program can be used to create delete and list Named attributes
 * attached to a file. See the "usage" routine above for the command
 * option description.
 */

/*
 * Main program for parsing command options and using
 * libvxfsutil API library calls for Named Attribute Feature.
 *
 */

int
main(
	int	argc,
	char	**argv)
{

	int		fd, nfd;
	int		err = 0;
	char		filename[256];
	char		attrname[256];
	char		attr_new[256];
	int		choice;
	char		buf[2048];
	int		mode = 0x777;
	int		attr = 0;
	char		error_buf[256];
	int		stats = 0;
	int		i, j, count;

	memset(attrname, 0, 256);
	memset(attr_new, 0, 256);
	memset(filename, 0, 256);

	while (1) {
		print_menu(filename, attrname, attr_new);
		choice = getchar();
		switch (choice) {

		case '0':
			/*
			 * select regular file
			 */

			while ((choice = getchar()) != '\n');
			i = 0;
			printf("\nEnter File Name ==>");
			while ((filename[i++] = getchar()) != '\n');
			filename[i - 1] = 0;
			continue;
			break;

		case '1':
			/*
			 * select attribute file
			 */

			while ((choice = getchar()) != '\n');
			i = 0;
			printf("\nEnter Attribute File Name ==>");
			while ((attrname[i++] = getchar()) != '\n');
			attrname[i - 1] = 0;
			printf("\n%s", attrname);
			continue;
			break;

		case '2':
			/*
			 * select attribute file
			 */

			while ((choice = getchar()) != '\n');
			i = 0;
			printf("\nEnter New Attribute File Name ==>");
			while ((attr_new[i++] = getchar()) != '\n');
			attr_new[i - 1] = 0;
			printf("\n%s", attr_new);
			continue;
			break;

		case 'c':
			/*
			 * create an attribute
			 */

			/*
			 * open regular file, create is necessary.
			 */

			if ((fd = open(filename, O_RDWR | O_CREAT, mode)) < 0) {
				sprintf(error_buf, "%s, Error Opening File %s ",
					argv[0], filename);
				perror(error_buf);
				break;
			}

			/*
			 * create named attribute file
			 */

			if ((nfd =
			     vxfs_nattr_open(fd, attrname, O_CREAT,
					     mode)) < 0) {
				sprintf(error_buf,
				  "%s, Error Opening Attribute File %s/./%s ",
				  argv[0], filename, attrname);
				perror(error_buf);
			}

			close(nfd);
			close(fd);

			break;

		case 'u':
			/*
			 * unlink an attribute
			 */

			/*
			 * open file
			 */

			if ((fd = open(filename, O_RDWR)) < 0) {
				perror("open filename Error:");
				sprintf(error_buf, "%s, Error Opening File %s ",
					argv[0], filename);
				perror(error_buf);
				break;
			}

			/*
			 * unlink  attribute
			 */

			/*
			 * ??? seems like unlink returns a value of 2 when
			 * attrname is null. Man page says it returns
			 * -1 or 0. 2 is the error code for no suchfile or
			 * directory. must be a bug .
			 */

			if ((err = vxfs_nattr_unlink(fd, attrname)) != 0) {
				sprintf(error_buf,
				  "%s, Error Unlinking Attribute File %s/./%s ",
				  argv[0], filename, attrname);
				perror(error_buf);
			}

			close(fd);
			break;

		case 'W':
			/*
			 * write data to attribute file
			 */

			/*
			 * open file
			 */

			if ((fd = open(filename, O_RDWR | O_CREAT, mode)) < 0) {
				sprintf(error_buf, "%s, Error Opening File %s ",
					argv[0], filename);
				perror(error_buf);
				break;
			}

			/*
			 * create named attribute file
			 */

			if ((nfd = vxfs_nattr_open(fd, attrname,
						   O_WRONLY | O_CREAT,
						   mode)) < 0) {
				sprintf(error_buf,
				   "%s, Error Opening Attribute File %s/./%s ",
				   argv[0], filename, attrname);
				perror(error_buf);
				close(fd);
				break;
			}

			/*
			 * write some data to the  atribute file
			 */

			memset(buf, 0, 1024);
			memset(buf, '?', 1023);
			write(nfd, buf, 1024);

			close(nfd);
			close(fd);
			break;

		case 'R':
			/*
			 * read data from attribute file
			 */

			/*
			 * open file
			 */

			if ((fd = open(filename, O_RDONLY)) < 0) {
				sprintf(error_buf, "%s, Error Opening File %s ",
					argv[0], filename);
				perror(error_buf);
				break;
			}

			/*
			 * open named attribute file
			 */

			if ((nfd =
			     vxfs_nattr_open(fd, attrname, O_RDONLY,
					     mode)) < 0) {

				sprintf(error_buf,
				    "%s, Error Opening Attribute File %s/./%s ",
				    argv[0], filename, attrname);
				perror(error_buf);
				close(fd);
				break;
			}

			/*
			 * read some data from the  atribute file
			 */

			printf("\nData from Attribute file %s/./%s\n", filename,
			       attrname);
			count = read(nfd, buf, 2048);
			for (i = 0; i < count; i += 80) {
				for (j = i; j < i + 80; j++)
					printf("%c", buf[i]);
				printf("\n");
			}

			close(nfd);
			close(fd);
			break;

		case 's':
			/*
			 * display stats for each attribute file
			 */

			stats = 1;

		case 'd':
			/*
			 * read the attribute directory
			 */

			/*
			 * open file
			 */

			if ((fd = open(filename, O_RDONLY)) < 0) {
				sprintf(error_buf, "%s, Error Opening File %s ",
					argv[0], filename);
				perror(error_buf);
				stats = 0;
				break;
			}

			/*
			 * open the Attributes files directory and get all the
			 * directory entries
			 */

			err = print_nattr_data(fd, filename, mode, stats);

			stats = 0;
			close(fd);
			break;

		case 'r':
			/*
			 * rename an attribute
			 */

			/*
			 * open file
			 */

			if ((fd = open(filename, O_RDWR)) < 0) {
				sprintf(error_buf, "%s, Error Opening File %s ",
					argv[0], filename);
				perror(error_buf);
				stats = 0;
				break;
			}

			/*
			 * rename attribute
			 */

			if ((err =
			     vxfs_nattr_rename(fd, attrname, attr_new)) != 0) {

				sprintf(error_buf,
				   "%s, Error renaming Attribute File %s/./%s ",
				   argv[0], filename, attrname);
				perror(error_buf);

			}
			close(fd);
			break;

		case 'l':
			/*
			 * link an attribute file
			 */

			/*
			 * open file
			 */

			if ((fd = open(filename, O_RDWR)) < 0) {
				sprintf(error_buf, "%s, Error Opening File %s ",
					argv[0], filename);
				perror(error_buf);
				break;
			}

			/*
			 * link attribute
			 */

			if ((err =
			     vxfs_nattr_link(fd, attrname, attr_new)) != 0) {
				sprintf(error_buf,
				   "%s, Error linking Attribute File %s/./%s ",
				   argv[0], filename, attrname);
				perror(error_buf);

			}
			close(fd);
			break;

		case 't':
			/*
			 * set atime/mtime  for attribute file
			 */

			/*
			 * open file
			 */

			if ((fd = open(filename, O_RDWR)) < 0) {
				sprintf(error_buf, "%s, Error Opening File %s ",
					argv[0], filename);
				perror(error_buf);
				break;
			}

			/*
			 * set attribute atime/mtime to current TOD
			 * NOTE: Using NULL for the 3rd arguement will
			 * cause the timestamp to be the current time.
			 */

			if ((err =vxfs_nattr_utimes(fd, attrname, NULL)) != 0) {

				/*
				 * An alternate way to modify the time stamp,
				 * is to pass a timeval structure as the 3rd
				 *  arguement
				 * struct timeval time;
				 *  gettimeofday(&time, NULL);
				 *
				 * if ( (err =
				 *   vxfs_nattr_utimes(fd, attrname, time)) ) {
				 */

				sprintf(error_buf, "%s, Error setting timestamp"
					" on Attribute File %s/./%s ",
					argv[0], filename, attrname);
				perror(error_buf);

			}
			close(fd);
			break;

		case 'q':
			exit(0);
			break;

		default:
			break;
		}
		while ((choice = getchar()) != '\n');
	}
}
