/* $Id: cache_advisory.c,v 4.6.14.2 2007/09/13 16:40:27 amukher2 Exp $ */
/* #ident "@(#)vxfs:$RCSfile: cache_advisory.c,v $	$Revision: 4.6.14.2 $"*/

/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

/*
 *	Cache Advisory tests
 */

#include "cache_advisory.h"

#define STARTID		'a'
#define	PAGEOFF		PAGEOFFSET
#define	BUFSIZE		(8*1024)
#define BUFOFF		(BUFSIZE - 1)
#define SECTORSIZE	DEV_BSIZE
#define NSPB		(BUFSIZE/SECTORSIZE)	/* Number of Sectors/Buf */
#define FILESIZE	(BUFSIZE * 100)	/* Default File Size */

typedef struct io_mem {
	int	im_size;
	int	im_align;
	char	*im_buf;
	char	*im_fbuf;
} io_mem_t;

extern int	optind, opterr;
extern char	*optarg;

static void
usage(void)
{
	printf("\nUsage: cache_advisory [-cdrsuq] <file>");
	printf("\n               [-c ] ==> concurrent I/O"
	       "\n               [-d ] ==> direct I/O"
	       "\n               [-r ] ==> random "
	       "\n               [-s ] ==> syncronous"
	       "\n               [-u ] ==> unbuffered"
	       "\n               [-q ] ==> sequnetial\n");
	exit(1);
}

/*
 * vx_get_mem
 */

int
vx_get_mem(
	io_mem_t * iomp)
{
	char	*p;

	if ((iomp->im_fbuf = malloc(iomp->im_size + (PAGESIZE * 2))) == NULL) {
		return ENOMEM;
	}

	p = (char *) (((long) iomp->im_fbuf + (long)PAGEOFF) & ~PAGEOFF);
	if (iomp->im_align == 0)
		p += 4;

	iomp->im_buf = p;
	printf("\n\t\tMemory buffer size %d", iomp->im_size);
	printf("\n\t\tRaw buffer adr = 0x%lx, "
	       "\n\t\tUsing buffer aligned on page boundary = 0x%lx",
	       (unsigned long) iomp->im_buf, (unsigned long) iomp->im_fbuf);
	return 0;
}

/*
 * vx_rel_mem
 */

void
vx_rel_mem(
	io_mem_t * iomp)
{
	free(iomp->im_fbuf);
}

/*
 * Write file using Direct I/O
 *	Align memory buffer on page boundary
 *      Write data algined on Sector boundary
 */

int
dio_write_sector(
	char *fname,
	int filesz,
	int flag,
	off_t offset)
{
	int		fd, i, j, id, error = 0;
	char		*tp;
	io_mem_t	iom;

	printf("\n\n\tDIO write sector with buffer aligned");

	if ((fd = open(fname, O_CREAT | O_TRUNC | O_RDWR | O_SYNC, 0644)) < 0) {
		return errno;
	}
	printf("\nSeeking to %lld", offset);
	lseek(fd, offset, SEEK_SET);

	iom.im_size = BUFSIZE;
	iom.im_align = 1;
	if (error = vx_get_mem(&iom)) {
		printf("\n\t\tMemory allocation FAILED\n");
		return error;
	}

	if (flag == VX_DIRECT) {
		printf("\n\t\tSet Direct I/O");
	} else if (flag == VX_UNBUFFERED) {
		printf("\n\t\tSet UNBUFFERED I/O");
	} else {
		printf("\n\t\tSet CONCURRENT I/O");
	}

	if (ioctl(fd, VX_SETCACHE, flag) < 0) {
		error = errno;
		goto out;
	}

	id = STARTID + 1;
	for (i = 0; i < filesz; i += BUFSIZE, id++) {
		memset(iom.im_buf, '0' + (id % 10), BUFSIZE);
		tp = iom.im_buf;
		for (j = 0; j < NSPB; j++) {
			if (write(fd, tp, SECTORSIZE) != SECTORSIZE) {
				error = errno;
				goto out;
			}
			tp += SECTORSIZE;
		}
	}

      out:
	vx_rel_mem(&iom);

	/*
	 * unset DIO
	 */

	if (ioctl(fd, VX_SETCACHE, 0) < 0) {
		error = errno;
	}

	close(fd);
	return error;
}

int
write_test(
	char *fname,
	int filesz,
	int flag)
{
	int	fd, i, id, error = 0;
	char	*mem_buf;

	printf("\n\n\tWrite data to file  using buffer caching");

	if ((fd = open(fname, O_CREAT | O_TRUNC | O_RDWR, 0644)) < 0) {
		return errno;
	}

	if ((mem_buf = malloc(BUFSIZE)) == NULL) {
		return ENOMEM;
	}

	if (flag == VX_SEQ) {
		printf("\n\t\tSet SEQUENTIAL I/O");
	} else if (flag == VX_RANDOM) {
		printf("\n\t\tSet RANDOM I/O");
	} else if (flag == VX_DSYNC) {
		printf("\n\t\tSet SYNCRONOUS I/O");
	} else {
		return EINVAL;
	}

	if (ioctl(fd, VX_SETCACHE, flag) < 0) {
		error = errno;
		goto out2;
	}

	id = STARTID + 1;
	for (i = 0; i < filesz; i += BUFSIZE, id++) {
		memset(mem_buf, '0' + (id % 10), BUFSIZE);
		if (write(fd, mem_buf, BUFSIZE) < 0) {
			error = errno;
			goto out2;
		}
	}

      out2:
	free(mem_buf);

	/*
	 * unset  Caching Advisory
	 */

	if (ioctl(fd, VX_SETCACHE, 0) < 0) {
		error = errno;
	}

	close(fd);
	return error;
}

int
main(
	int	argc,
	char	**argv)
{
	int	error = 0;
	int	flag = 0;
	int	c;
	char	*filename;
	pid_t	child;

	while ((c = getopt(argc, argv, "cdsrqu")) != EOF) {
		switch (c) {

		case 'c':
			/*
			 * specify cache advisory
			 */

			flag = VX_CONCURRENT;
			break;

		case 'd':
			/*
			 * specify cache advisory
			 */

			flag = VX_DIRECT;
			break;

		case 's':
			/*
			 * specify cache advisory
			 */

			flag = VX_DSYNC;
			break;

		case 'r':
			/*
			 * specify cache advisory
			 */

			flag = VX_RANDOM;
			break;

		case 'u':
			/*
			 * specify cache advisory
			 */

			flag = VX_UNBUFFERED;
			break;

		case 'q':
			/*
			 * specify cache advisory
			 */

			flag = VX_SEQ;
			break;

		case '?':
			usage();
			break;
		}
	}

	if (optind != argc - 1) {
		usage();
	}
	filename = argv[optind];

	if ((flag == VX_DIRECT) || (flag == VX_UNBUFFERED)
	    || (flag == VX_CONCURRENT)) {

		if ((child = fork()) == 0) {
			printf("\nStarting child");

			/*
			 * child to write at top of file
			 */

			error = dio_write_sector(filename, FILESIZE, flag,
						 FILESIZE);

		} else if (child == -1)
			exit(1);
		else {
			/*
			 * parent to write a start of file
			 */

			error = dio_write_sector(filename, FILESIZE, flag, 0);
		}

	} else {
		error = write_test(filename, FILESIZE, flag);
	}

	printf("\n");

	if (error != 0) {
		printf("\nError, %s", strerror(error));
	}

	exit(error);
}
