/* $Id: extent.c,v 4.2 2006/03/17 15:26:29 phils Exp $ */
/* #ident "@(#)vxfs:$RCSfile: extent.c,v $	$Revision: 4.2 $" */

/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

/*
 * set extent attributes
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/fs/vx_ioctl.h>
#include <sys/statvfs.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <string.h>
#include <sys/vfs.h>

extern int	optind, opterr;
extern char	*optarg;

char		*earg = "2000";

static void
usage(void)
{
	printf("Usage: extend [-e extent size] <file>\n");
	exit(1);
}

/*
 * Convert a size spec to file system blocks
 */

static int
strtofsblks(
	int	bsize,
	char	*arg,
	off_t	*blks)
{
	int	arg_blks;

	sscanf(arg, "%d", &arg_blks);
	*blks = (arg_blks + bsize - 1) / bsize;
	return 0;
}

#define  DB_EXTSIZE_MULTIPLE	2

int
main(
	int	argc,
	char	*argv[])
{
	struct statvfs	statvfsbuf;
	struct statfs	statfsbuf;
	char		*filename;
	void		*buf;
	int		c, fd, i;
	struct stat	sb;
	struct vx_ext	extbuf;

	memset(&extbuf, '\0', sizeof(extbuf));
	while ((c = getopt(argc, argv, "e:")) != EOF) {
		switch (c) {
		case 'e':
			/*
			 * specify extent size
			 */

			earg = optarg;
			break;

		case '?':
			usage();
			break;
		}
	}

	if (optind != argc - 1) {
		usage();
	}
	filename = argv[optind];
	if ((fd = open(filename, O_RDWR | O_CREAT, 0x777)) < 0) {
		printf("cannot open %s: %s\n", filename, strerror(errno));
		exit(1);
	}
	if (fstatfs(fd, &statfsbuf)) {
		printf("cannot statfs %s: %s\n", filename, strerror(errno));
		exit(1);
	}
	if (statfsbuf.f_type == 0xa501fcf5) {
		printf("%s is not on a vxfs file system\n", filename);
		exit(1);
	}

	if (fstatvfs(fd, &statvfsbuf)) {
		printf("cannot statvfs %s: %s\n", filename, strerror(errno));
		exit(1);
	}
	if (strtofsblks(statvfsbuf.f_frsize, earg, &extbuf.ext_size)) {
		printf("%s is not a valid extent size\n", earg);
		exit(1);
	}

	extbuf.ext_size *= DB_EXTSIZE_MULTIPLE;

	printf("\n Setting an extent size of %lld FS blocks for file %s\n",
	       extbuf.ext_size, filename);

	if (ioctl(fd, VX_SETEXT, &extbuf)) {
		printf("cannot set attributes for %s: %s\n",
		       filename, strerror(errno));
		printf("\nUnable to set extent For File %s", filename);
		printf("\n Extent Size = %lld   \n FS block size = %lu "
		       "\n Reservation size = %lld",
		       extbuf.ext_size, statvfsbuf.f_frsize, extbuf.reserve);

		exit(1);
	}

	if (ioctl(fd, VX_GETEXT, &extbuf)) {
		printf("cannot set attributes for %s: %s\n",
		       filename, strerror(errno));
		exit(1);
	}

	printf("\nFor File %s \n Extent Size = %lld  "
	       "blocks \n FS block size = %lu "
	       "\n Reservation size = %lld  blocks", filename,
	       extbuf.ext_size, statvfsbuf.f_frsize, extbuf.reserve);

	buf = malloc(extbuf.ext_size * statvfsbuf.f_frsize);

	/*
	 * Writing Data in blocks of size ext_size to optimize
	 * FS extent usage and allocation.
	 */

	for (i = 0; i < 4; i++) {
		write(fd, buf, extbuf.ext_size * statvfsbuf.f_frsize);
		printf("\nWrote  %lld KB to file %s",
		       extbuf.ext_size * statvfsbuf.f_frsize / 1024, filename);
	}

	/*
	 * file stat's on created file
	 */

	printf("\nChecking file stats on %s\n", filename);
	if (stat(filename, &sb) != 0) {
		printf("\n Error, %s, while trying to stat file %s\n",
		       strerror(errno), filename);
	} else {
		printf("\nFile stat for %s ,  size = %lld KB\n",
		       filename, sb.st_size / 1024ULL);
	}

	free(buf);
	close(fd);
	exit(0);
}
