/* $Id: freeze_all.c,v 4.2 2006/03/17 15:26:31 phils Exp $ */
/* #ident "@(#)vxfs:$RCSfile: freeze_all.c,v $	$Revision: 4.2 $" */

/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/fs/vx_ioctl.h>
#include <mntent.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <string.h>

#define MNTAB "/etc/mtab"

/* Global Variables */
int	vxfs_fd[20];
int	success_flag = 0;
pid_t	parent = 0;
int	num_mntpt = 0;
char	*mntpt[20];

/* Signal Handlers */

/*
 * parent's handlers
 */

void
success(
	int i)
{
	printf("\r\nChild process has successfully Frozen file systems\n");
	success_flag = 1;
}

void
failure(
	int i)
{
	printf("Child Process failed to Freeze File systems\n");
	exit(1);
}

 /*
  * used by parent or child to thaw file system
  *
  * After Freezing multiple File Systems,
  * By default the files systems will Thaw after the timeout
  * period has expired. You can Thaw the file systems early,
  * before the timeout value has expired by Thaw'ing each
  * file system individually.
  */

void
thaw(
	int i)
{
	int	error = 0;

	for (i = 0; i < num_mntpt; i++) {
		if (ioctl(vxfs_fd[i], VX_THAW, NULL)) {
			printf("\nFile system thaw failed,%s,  for file "
			       "system %s", strerror(errno), mntpt[i]);
			close(vxfs_fd[i]);
			error = 1;
		} else {
			printf("\r\n%s process has successfully"
			       " Thaw'ed file system %s\n",
			       (getpid() == parent) ? "Parent" : "Child",
			       mntpt[i]);
			close(vxfs_fd[i]);
		}
	}
	exit(error);
}

/* Usage function */
void
usage(
	char *command)
{
	fprintf(stderr, "USAGE: %s [ -t timeout ] <mount_point> "
		"<mount_point> <mount_point> ...\n", command);
	exit(1);
}

static void
print_menu(void)
{
	printf("\nMenu choices\n");

	printf("\n t  - Early Thaw - Thaw File system");
	printf("\n q  - Wait for Child to complete Freeze/Thaw File system");

	printf("\n    Enter Choice ==> ");
}

int
main(
	int	argc,
	char	**argv)
{
	long		timeout = 30;
	uid_t		iam = geteuid();
	pid_t		gppid = getpid();
	int		opt, status;
	struct mntent	*mnt_ent;
	FILE		*mnttabfile;
	int		i, j, fd;
	char		choice, buf[100];
	pid_t		child;
	struct vx_freezeall freeze_info;
	int		entry_found = 0;

	/*
	 * Make sure they entered enough arguments, otherwise print usage
	 */

	if ((argc != 1) && (argc < 4)) {
		usage(argv[0]);
	}

	/*
	 * Make sure that we are root
	 */

	if (iam != 0) {
		fprintf(stderr, "ERROR: %s must be run setuid/as root.\n",
			argv[0]);
		exit(1);
	}

	/*
	 * check for any command line mount point arguments
	 */

	if (argc > 1) {

		/*
		 * Parse the args
		 */

		while ((opt = getopt(argc, argv, "t:")) != EOF) {
			switch (opt) {
			case 't':
				timeout = atol(optarg);
				break;

			case '?':
				usage(argv[0]);
				break;
			}
		}

		/*
		 * Check the timeout
		 */

		if ((timeout < 10) || (timeout > 60)) {
			fprintf(stderr,
				"ERROR: Timeout must be between 10 and 60 "
				"seconds (default 30).\n");
			exit(1);
		}

		/*
		 * Take Mount Points from command line
		 */

		num_mntpt = argc - 3;

		for (i = 0, j = 3; i < num_mntpt; i++, j++) {
			mntpt[i] = argv[j];

			/*
			 * Verify that the mount point is an absolute path
			 */

			if (strncmp(mntpt[i], "/", 1)) {
				fprintf(stderr,
					"ERROR: File system mount point must"
					" begin with /, %s.\n", mntpt[i]);
				exit(1);
			}

			/*
			 * Get the mnttab entry
			 */

			mnttabfile = setmntent(MNTTAB, "r");
			if (!mnttabfile) {
				perror("ERROR: Unable to open the MNTTAB");
				exit(1);
			}

			while ((mnt_ent = getmntent(mnttabfile)) != NULL) {
				if ((strcmp(mnt_ent->mnt_dir, mntpt[i]) == 0) \
				 && (strcmp(mnt_ent->mnt_type, "vxfs") == 0)) {
					entry_found = 1;
				}
			}
			endmntent(mnttabfile);
			if (!entry_found) {
				fprintf(stderr,
					"ERROR: file system %s not found"
					" in MNTTAB or is not VXFS.\n",
					mntpt[i]);
				exit(1);
			}
			entry_found = 0;
		}

	} else {

		/*
		 * select mount points
		 */

		printf("\nProgram will simultaneaously Freeze All"
		       " Selected Files Systems");
		while (1) {
			i = 0;
			printf("\nEnter Mount Point of FS to Freeze ==>");

			while ((buf[i++] = getchar()) != '\n');
			buf[i - 1] = 0;

			if (strlen(buf) == 0)
				break;	/* last entry */

			mnttabfile = setmntent(MNTTAB, "r");
			if (!mnttabfile) {
				perror("ERROR: Unable to open the MNTTAB");
				exit(1);
			}

			while ((mnt_ent = getmntent(mnttabfile)) != NULL) {
				if ((strcmp(mnt_ent->mnt_dir, buf) == 0) && \
				    (strcmp(mnt_ent->mnt_type, "vxfs") == 0)) {
					entry_found = 1;
				}
			}
			endmntent(mnttabfile);

			if (entry_found) {
				if ((fd =
				     open(mnt_ent->mnt_dir, O_RDONLY)) < 0) {
					printf("\nUnable to open mount point");
					continue;
				} else {
					mntpt[num_mntpt++] =
						strdup(mnt_ent->mnt_dir);
					close(fd);
				}
			} else {
				printf("\nNot in Mount Table or "
				       "Not a VxFS file system\n");
			}
			entry_found = 0;
		}
	}

	if (num_mntpt == 0) {
		printf("\nNo Valid mount points");
		exit(1);
	}

	for (i = 0; i < num_mntpt; i++)
		printf("\nWill attempt to freeze File System %s", mntpt[i]);

	/*
	 * Child does the work, Parent waits for signal to return status
	 */

	/*
	 * if open is done here, then the parent & child can THAW
	 * since child inherits fd
	 */

	parent = getpid();
	if ((child = fork()) == 0) {

		/*
		 * Child, setup the signal handlers
		 */

		signal(SIGUSR2, thaw);

		/*
		 * Freeze filesystem, signal parent to return status
		 */

		/*
		 * if open is done here, only the child can THAW
		 */

		for (i = 0; i < num_mntpt; i++) {
			vxfs_fd[i] = open(mntpt[i], O_RDONLY);
			if (vxfs_fd[i] < 0) {
				perror("ERROR: Failed to open device file");
				exit(1);
			}
		}

		/*
		 * Format the FREEZE_ALL data structure
		 */

		freeze_info.num = num_mntpt;
		freeze_info.timeout = timeout;
		freeze_info.fds = &vxfs_fd[0];

		/*
		 * Initiate FREEZE_ALL using IOCTL interface
		 */

		if (ioctl(vxfs_fd[0], VX_FREEZE_ALL, &freeze_info)) {
			perror("ERROR: File system freeze failed");
			for (i = 0; i < num_mntpt; i++)
				close(vxfs_fd[i]);
			kill(gppid, SIGUSR2);
			exit(1);
		} else {
			kill(gppid, SIGUSR1);

			/*
			 * Print our pid, then wait for later thaw signal
			 */

			/*
			 * fclose(stdout);
			 */

			sleep(timeout);
			for (i = 0; i < num_mntpt; i++)
				close(vxfs_fd[i]);
			exit(0);
		}
	} else {

		/*
		 * Parent
		 */

		/*
		 * setup parent keyboard handlers
		 */

		signal(SIGHUP, thaw);
		signal(SIGINT, thaw);
		signal(SIGTERM, thaw);

		/*
		 * Setup to receive success/failure signal from child
		 */

		signal(SIGUSR1, success);
		signal(SIGUSR2, failure);

		/*
		 * wait for child to freeze
		 */

		printf("\nWaiting for Child to Freeze File systems\n");
		while (!success_flag);
		while (1) {
			print_menu();
			choice = getchar();
			switch (choice) {

			case 't':
				/*
				 * send a signal thaw the file system
				 * before the timeout period
				 */

				kill(child, SIGUSR2);

			case 'q':
				goto out;
				break;

			default:
				break;
			}

			while ((choice = getchar()) != '\n');
		}

	      out:
		wait(&status);
		exit(0);
	}
}
