/* $Id: reserve.c,v 4.2 2006/03/17 15:26:32 phils Exp $ */
/* #ident "@(#)vxfs:$RCSfile: reserve.c,v $	$Revision: 4.2 $" */

/*
 * $Copyright: Copyright (c) 2009 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

/*
 * set extent attributes
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/fs/vx_ioctl.h>
#include <sys/statvfs.h>
#include <sys/statfs.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <string.h>

extern int	optind, opterr;
extern char	*optarg;

char	*rarg = "100000";

static void
usage(void)
{
	printf("Usage: [-r reservation] <file>\n");
	exit(1);
}

/*
 * Convert a size spec to file system blocks
 */

static int
strtofsblks(
	int	bsize,
	char	*arg,
	off_t	*blks)
{
	int	arg_blks;

	sscanf(arg, "%d", &arg_blks);
	*blks = (arg_blks + bsize - 1) / bsize;
	return 0;
}

#define  EXTENT_SIZE	10	/* 10 FS Blocks */

int
main(
	int	argc,
	char	**argv)
{
	struct statvfs	statvfsbuf;
	struct statfs	statfsbuf;
	char		*filename;
	void		*buf;
	int		c, fd, i;
	off_t		reservation;
	struct stat	sb;
	struct vx_ext	extbuf;

	memset(&extbuf, '\0', sizeof(extbuf));
	while ((c = getopt(argc, argv, "r:")) != EOF) {
		switch (c) {

		case 'r':
			/*
			 * specify reservation
			 */

			rarg = optarg;
			break;

		case '?':
			usage();
			break;
		}
	}
	if (optind != argc - 1) {
		usage();
	}

	filename = argv[optind];
	if ((fd = open(filename, O_RDWR | O_CREAT, 0x777)) < 0) {
		printf("cannot open %s: %s\n", filename, strerror(errno));
		exit(1);
	}

	if (fstatfs(fd, &statfsbuf)) {
		printf("cannot statfs %s: %s\n", filename, strerror(errno));
		exit(1);
	}
	if (0xa501fcf5 == statfsbuf.f_type) {
		printf("%s is not on a vxfs file system\n", filename);
		exit(1);
	}

	if (fstatvfs(fd, &statvfsbuf)) {
		printf("cannot statvfs %s: %s\n", filename, strerror(errno));
		exit(1);
	}
	if (strtofsblks(statvfsbuf.f_frsize, rarg, &extbuf.reserve)) {
		printf("%s is not a valid reservation size\n", rarg);
		exit(1);
	}

	extbuf.ext_size = EXTENT_SIZE;
	extbuf.a_flags = VX_CHGSIZE;

	printf("\n Setting a reservation of %lld FS blocks for file %s\n",
	       extbuf.reserve, filename);

	if (ioctl(fd, VX_SETEXT, &extbuf)) {
		printf("cannot set attributes for %s: %s\n",
		       filename, strerror(errno));
		printf("\nAttempted to Set Reservation For File %s "
		       "\n Extent Size = %lld  "
		       "\n FS block size = %lu "
		       "\n Reservation size = %lld ", filename,
		       extbuf.ext_size, statvfsbuf.f_frsize, extbuf.reserve);

		exit(1);
	}

	if (ioctl(fd, VX_GETEXT, &extbuf)) {
		printf("cannot get attributes for %s: %s\n",
		       filename, strerror(errno));
		exit(1);
	}

	printf("\nSet extent values for %s", filename);
	printf(" \n Extent Size = %lld  blocks  \n FS block size = %lu "
	       "\n Reservation size = %lld  blocks",
	       extbuf.ext_size, statvfsbuf.f_frsize, extbuf.reserve);

	close(fd);
	printf("\nNo data has been written to file, but note that the "
	       "\nsize of the file is the size of the reservation");

	/*
	 * file stat's on created file
	 */

	printf("\n\nChecking file stats on %s\n", filename);
	if (stat(filename, &sb) != 0) {

		printf("\n Error, %s, while trying to stat file %s\n",
		       strerror(errno), filename);
	} else {
		printf("\nFile stat for %s ,  size = %lld KB\n",
		       filename, sb.st_size / 1024ULL);
	}

	exit(0);
}
