Usage()
{
    echo "cfgsecurity -a <agent> -h <Host> -d <DC> -p <Password> -b [AB] -s [Sec
_password] -g [vea_guest passwd]"
    echo "   agent_name   : Name of the agent."
    echo "   Host         : Name of the host where the package is installed."
    echo "   DC           : Name of the Domain controller host."
    echo "   Password     : Password for the private account for the agents."
    echo "   AB           : Name of the Authentication Broker host."
    echo "   Sec_password : security administrators password."
    echo "   vea_guest passwd : password for vea_guest. (Default is vea_guest)"
    exit 1
}

#The key Mode has to be present in the registry.
PAL_VERSION=33
REGCTL=$PKG_INSTALL_ROOT/opt/VRTSobc/pal${PAL_VERSION}/bin/vxregctl
REGFILE=$PKG_INSTALL_ROOT/etc/vx/isis/Registry
VEACONFIG=$PKG_INSTALL_ROOT/opt/VRTSobc/pal${PAL_VERSION}/bin/veaconfig 

while getopts :a:h:d:p:b:s:g: name
do
    case $name in
        a) agent=$OPTARG;;

        h) LocalHost=$OPTARG;;

        d) DC=$OPTARG;;

        p) password=$OPTARG;;

        s) secpwd=$OPTARG;;

        b) AB=$OPTARG;;
        
	g) guest_pwd=$OPTARG;;

    esac
done


if [ "$agent" = "" ]
then
  Usage
fi

#Get vxsvc version
vxsvc_ver=`$REGCTL $REGFILE queryvalue Software/VERITAS/VxSvc/CurrentVersion Version`
if [ $? -eq 0 ]
then
   vxsvc_ver=`echo $vxsvc_ver |sed -e 's/^.*: //g'`
   base_ver=`echo $vxsvc_ver | awk -F. '{print $2}'`;
   if [ "$base_ver" -lt "3" ] && [ "$agent" = "root" ]
   then
        exit 0
   fi
fi

#If vxsvc is to configured in a hybrid mode, the default's Mode key may not be present in the registry so ignore
#it .For all other cases absense of this key would mean that there is something wrong with the setup.
if [ "$DC" != "HB" ]
then
   mode=`$REGCTL $REGFILE queryvalue  Software/VERITAS/VRTSobc/pal${PAL_VERSION}/Agents/__defaults Mode`
   if [ $? != 0 ]
   then
     exit 1
   fi
fi


mode=`echo $mode |sed -e 's/^.*: //g'`

if [ "$agent" = "root" ] 
then

 # If local vxsvc is to be configured such that it has agents on
 # the same m/c part of a central domain (i.e this is a hybrid configuration)
 # then set the Mode for vxsvc as 1 else Mode for vxsvc is the Mode set in Agents/__defaults
 if [ "$DC" = "HB" ]
 then
    mode=1;
 fi

 `$REGCTL $REGFILE setvalue Software/VERITAS/VxSvc/CurrentVersion Mode REG_INT $mode`;

else

#The Mode for agents is always the Mode set in Agents/__defaults in Registry
`$REGCTL $REGFILE setvalue Software/VERITAS/VRTSobc/pal${PAL_VERSION}/Agents/$agent Mode REG_INT $mode`;
fi



#If VRTSobc was installed in host controller mode create csf_resolv.conf and
#set agent security to high

if [ "$mode" = "1" ]
then

#Set agents security to high
  $VEACONFIG -c security -s high -a $agent -b $PKG_INSTALL_ROOT/
  rc=$?
  if [ "$rc" != "0" ];
  then
      exit $rc
   fi

  #Create csf_resolv.conf if not already present only if
  #vxsvc is not configured locally in a hybrid mode 
  if [ "$DC" != "HB" ]
  then
  
      #Prepare csf_resolv.conf with domaintype = "localhost"

      if [ ! -d $PKG_INSTALL_ROOT/etc/default ]
      then
          mkdir $PKG_INSTALL_ROOT/etc/default
      fi

      RESOLVE_CONF_FILE=$PKG_INSTALL_ROOT/etc/default/csf_resolv.conf

      if [ ! -f $RESOLVE_CONF_FILE ]
      then
           echo "[SECURITY]" > $RESOLVE_CONF_FILE
           echo "PROTOCOL_TYPE = \"high\";" >> $RESOLVE_CONF_FILE
           echo "USER = \"\";" >> $RESOLVE_CONF_FILE
           echo "DOMAIN_NAME = \"\";" >> $RESOLVE_CONF_FILE
           echo "DOMAIN_TYPE = \"localhost\";" >> $RESOLVE_CONF_FILE
           echo "BROKER_HOST = \"$AB\";" >> $RESOLVE_CONF_FILE
           echo "BROKER_PORT = \"\";" >> $RESOLVE_CONF_FILE
           echo "[DOMAIN_CONTROLLER]" >> $RESOLVE_CONF_FILE
           if [ "$DC" != "__NO_DC" ]
           then
           if [ "X$DC" = "X" ]
           then 
              DC="localhost"
           fi
              echo "domain = \"$DC\";" >> $RESOLVE_CONF_FILE
           fi
     fi 
  fi
   exit 0
fi


if [ "$LocalHost" = "" -o "$DC" = "" -o "$password" = "" ]; then
    Usage
fi

if [ "$secpwd" = "" ]
then
    secpwd="Vxadmin"
    echo "Default password being used, please change it later";
fi

if [ "$AB" = "" ];
then
    if [ "$DC" = "__NO_DC" ]
    then
        AB="$LocalHost"
    else
    AB="$DC"
    fi
fi

OS=`uname -s`

private_dom=vea_domain
security_admin=$AB:admin@broker.vx
agent_id=$AB:vea_agent@vea_domain.vx
guest=$AB:vea_guest@vea_domain.vx 
if [ "$guest_pwd" = "" ];
then
	guest_pwd=vea_guest
fi

#prepare the csf_resolv.conf
     
if [ ! -d $PKG_INSTALL_ROOT/etc/default ]
then
       mkdir $PKG_INSTALL_ROOT/etc/default
fi
 
RESOLVE_CONF_FILE=$PKG_INSTALL_ROOT/etc/default/csf_resolv.conf

#if [ ! -f $RESOLVE_CONF_FILE ]
#  then
        echo "[SECURITY]" > $RESOLVE_CONF_FILE
        echo "PROTOCOL_TYPE = \"high\";" >> $RESOLVE_CONF_FILE
        echo "USER = \"vea_guest\";" >> $RESOLVE_CONF_FILE
        echo "DOMAIN_NAME = \"vea_domain\";" >> $RESOLVE_CONF_FILE
        echo "DOMAIN_TYPE = \"vx\";" >> $RESOLVE_CONF_FILE
        echo "BROKER_HOST = \"$AB\";" >> $RESOLVE_CONF_FILE
        echo "BROKER_PORT = \"1556\";" >> $RESOLVE_CONF_FILE
        echo "[DOMAIN_CONTROLLER]" >> $RESOLVE_CONF_FILE
        if [ "$DC" != "__NO_DC" ]
        then 
           if [ "X$DC" = "X" ]
           then
              DC="localhost"
           fi
 
           echo "domain = \"$DC\";" >> $RESOLVE_CONF_FILE
        fi
#fi

# Set up trust
#echo /opt/VRTSobc/pal33/bin/veaconfig -c setuptrust -l $AB
$VEACONFIG -c setuptrust -l $AB -b $PKG_INSTALL_ROOT/

rc=$?
if [ "$rc" != "0" ];
then
   exit $rc
fi
if [ "$agent" = "Root" ];
then 
      agent="root"
fi      
# Create domain and account if agent is $agent 
if [ "$agent" = "root" ];
then 
    #echo  /opt/VRTSobc/pal33/bin/veaconfig -c createpd -d $private_dom -l $AB -u $security_admin -p $secpwd 
    $VEACONFIG -c createpd -d $private_dom -l $AB -u $security_admin -p $secpwd -b $PKG_INSTALL_ROOT/
    rc=$?
    if [ "$rc" != "0" ];
    then
        exit $rc
    fi
    #echo  /opt/VRTSobc/pal33/bin/veaconfig -c addprpl -u $security_admin -p $secpwd -n $agent_id -q $password 
    $VEACONFIG -c addprpl -u $security_admin -p $secpwd -n $agent_id -q $password -b $PKG_INSTALL_ROOT/
    rc=$?
    if [ "$rc" != "0" ];
    then
        exit $rc
    fi
    $VEACONFIG -c addprpl -u $security_admin -p $secpwd -n $guest -q $guest_pwd -b $PKG_INSTALL_ROOT/
    rc=$?
    if [ "$rc" != "0" ];
    then
        exit $rc
    fi
fi

# Save long term credential 
$VEACONFIG -c deletecred -u $agent_id -b $PKG_INSTALL_ROOT/
$VEACONFIG -c deletecred -u $guest -b $PKG_INSTALL_ROOT/
tagent_id=$AB:vea_agent@vea_domain@$AB.vx
$VEACONFIG -c deletecred -u $tagent_id -b $PKG_INSTALL_ROOT/
#echo /opt/VRTSobc/pal33/bin/veaconfig -c authenticate -u  $agent_id -p $password
$VEACONFIG -c authenticate -u  $agent_id -p $password -b $PKG_INSTALL_ROOT/
rc=$?
if [ "$rc" != "0" ];
then
    exit $rc
fi
$VEACONFIG -c authenticate -u  $guest -p $guest_pwd -b $PKG_INSTALL_ROOT/
rc=$?
if [ "$rc" != "0" ];
then
    exit $rc
fi
    
# Set Agent's and default security Id
#echo /opt/VRTSobc/pal33/bin/veaconfig -c setagentid  -n  $agent_id -a $agent
$VEACONFIG -c setagentid  -n  $agent_id -a $agent -b $PKG_INSTALL_ROOT/
    rc=$?
    if [ "$rc" != "0" ];
    then
        exit $rc
    fi
#/opt/VRTSobc/pal33/bin/veaconfig -c setagentid -d $private_dom -l $AB -n  $agent_id -a default
#    rc=$?
#    if [ "$rc" != "0" ];
#    then
#        exit $rc
#    fi
#echo /opt/VRTSobc/pal33/bin/veaconfig -c security -s high -a $agent 
$VEACONFIG -c security -s high -a $agent -b $PKG_INSTALL_ROOT/
    rc=$?
    if [ "$rc" != "0" ];
    then
        exit $rc
    fi
#/opt/VRTSobc/pal33/bin/veaconfig -c security high -a default 
#    rc=$?
#    if [ "$rc" != "0" ];
#    then
#        exit $rc
#    fi

if [ "$agent" = "root" ];
then 
    #echo /opt/VRTSobc/pal33/bin/veaconfig -c setab -u $security_admin -p $secpwd 
    $VEACONFIG -c setab -u $security_admin -p $secpwd -b $PKG_INSTALL_ROOT/
    rc=$?
    if [ "$rc" != "0" ];
    then
        exit $rc
    fi
else
    if [ "$DC" != "__NO_DC" ]
    then
    $VEACONFIG -c joindc -l $DC  -u  $agent_id -p $password -a $agent@$LocalHost -b $PKG_INSTALL_ROOT/
    rc=$?
    if [ "$rc" != "0" ];
    then
        exit $rc
    fi

    #If The agent is configured in hybrid mode then join the agent to
    #the local domain if there is one configured in HC mode.

    ldc_mode=`$REGCTL $REGFILE queryvalue  Software/VERITAS/VxSvc/CurrentVersion Mode 2> /dev/null`
    rc=$?
    if [ "$rc" = "0" ]
    then
       ldc_mode=`echo $ldc_mode |sed -e 's/^.*: //g'`
       if [ "$ldc_mode" = "1" ]
       then
           $VEACONFIG -c joindc -a $agent@$LocalHost -l localhost -b $PKG_INSTALL_ROOT/
           rc=$?
           if [ "$rc" != "0" ];
           then
               exit $rc
           fi

       fi 
    fi 
 fi
fi
   
# delete security admin certificate
$VEACONFIG -c deletecred -u $security_admin  -b $PKG_INSTALL_ROOT/ 

secpwd=""; export secpwd
DC=""; export DC
password=""; export password
AB=""; export AB
exit 0
