#!/bin/ksh

#set -x
RESOLVE_CONF_FILE=$PKG_INSTALL_ROOT/etc/default/csf_resolv.conf
# PAL_VERSION=33
PAL_VERSION=33
VEACONFIG=$PKG_INSTALL_ROOT/opt/VRTSobc/pal${PAL_VERSION}/bin/veaconfig
VEAINFO=$PKG_INSTALL_ROOT/opt/VRTSobc/pal${PAL_VERSION}/bin/veainfo
REGCTL=$PKG_INSTALL_ROOT/opt/VRTSobc/pal${PAL_VERSION}/bin/vxregctl
REGFILE=$PKG_INSTALL_ROOT/etc/vx/isis/Registry

#
# creates private domain if not present
#
create_pd()
{
  PD=$1
  AB=$2
  SADM=$3
  SPWD=$4
  
   # create private domain and account
  $VEACONFIG -c createpd -d $PD -l $AB -u $SADM -p $SPWD   
  rc=$?
  if [ "$rc" != "0" ];
  then
    exit $rc
  fi
}
add_prpl()
{
  SADM=$1
  SPWD=$2
  agent_id=$3
  PWD=$4
  
  $VEACONFIG -c addprpl -u $SADM -p $SPWD -n $agent_id -q $PWD 
  rc=$?
  if [ "$rc" != "0" ];
  then
     exit $rc
  fi
}

setuptrust ()
{
# set up trust
$VEACONFIG -c setuptrust -l $1
rc=$?
if [ "$rc" != "0" ];
then
  exit $rc
fi
}

authenticate()
{
echo $VEACONFIG -c authenticate -u  $1 -p $2
$VEACONFIG -c authenticate -u  $1 -p $2
rc=$?
if [ "$rc" != "0" ];
then
  exit $rc
fi
}

update_security()
{
# set up trust
  setuptrust $AB
  
if [ "$type" = "CS" ];
then 
   create_pd $PD $AB $SADM $SPWD
   add_prpl $SADM $SPWD $agent_id $PWD
fi    

# Delete old credentials if present and  save new long term credential 
$VEACONFIG -c deletecred -u $agent_id
tagent_id=$AB:vea_agent@vea_domain@$AB.vx
$VEACONFIG -c deletecred -u $tagent_id

authenticate $agent_id  $PWD
return
}

Usage()
{
    echo "veacfg -k <package> [ -m <mode>  -h [Host] -d [DC] -p [Password] -b [AB] -s [Sec_password] ]]"
    echo "   package      : OB or OBC."
    echo "   mode         : SA for standalone; MH for managed host; CS for central server."
    echo "   Host         : FQHN of the host where the package is installed."
    echo "   DC           : FQHN of the Domain controller host."
    echo "   Password     : Password for the private account for the agents."
    echo "   AB           : FQHN of the Authentication Broker host."
    echo "   Sec_password : security administrator's password."
    echo " "
    echo " When configuring OBC as stand alone invoke"
    echo "      veacfg -k OBC -m SA -h <host_name>" 
    echo " When configuring OBC as central server invoke"
    echo "      veacfg -k OBC -m CS -h <host_name> -p password -b broker -s sec_password" 
    echo " When configuring OBC as managed host invoke"
    echo "      veacfg -k OBC -m MH -h <host_name> -d <central_server> -p password" 
    echo "          (Defaults to the first AB returned by veainfo)"  
    echo " When configuring OBC as managed host along with specifying the AB , invoke"
    echo "      veacfg -k OBC -m MH -h <host_name> -d <central_server> -p password -b broker" 
    echo " "
    echo " When configuring OB. invoke  veacfg -k OB" 
    exit 1
}
SetDefaults()
{
`$REGCTL $REGFILE setvalue Software/VERITAS/VRTSobc/pal33/Agents/__defaults LocalHost REG_SZ $LH`;
`$REGCTL $REGFILE deletekey Software/VERITAS/VRTSobc/pal33/Agents/__defaults/Network/Params/Security `;
`$REGCTL $REGFILE createkey Software/VERITAS/VRTSobc/pal33/Agents/__defaults/Network/Params/Security `;

}
update_csf_resolv()
{
#Create csf_resolv.conf if not already present only if
#Prepare csf_resolv.conf with domaintype = "localhost"
if [ "$mode" = "1" ];
then
  DT=localhost
  DC=$LH
else
  BH=$1
  DC=$2  
fi 
# create /etc/defaut if not present 
if [ ! -d $PKG_INSTALL_ROOT/etc/default ]
then
    mkdir $PKG_INSTALL_ROOT/etc/default
fi

echo "[SECURITY]" > $RESOLVE_CONF_FILE
echo "PROTOCOL_TYPE = \"high\";" >> $RESOLVE_CONF_FILE
echo "SECURITY = \"high\";" >> $RESOLVE_CONF_FILE
echo "USER = \"\";" >> $RESOLVE_CONF_FILE
echo "DOMAIN_NAME = \"\";" >> $RESOLVE_CONF_FILE
echo "DOMAIN_TYPE = \"$DT\";" >> $RESOLVE_CONF_FILE
echo "BROKER_HOST = \"$AB\";" >> $RESOLVE_CONF_FILE
echo "[DOMAIN_CONTROLLER]" >> $RESOLVE_CONF_FILE
echo "domain = \"$DC\";" >> $RESOLVE_CONF_FILE

}

while getopts :h:d:p:b:s:k:m: name
do
    case $name in
        k) pkg=$OPTARG;;
        
        m) type=$OPTARG;;
        
        h) LH=$OPTARG;;
        
        d) DC=$OPTARG;;

        p) PWD=$OPTARG;;

        s) SPWD=$OPTARG;;

        b) AB=$OPTARG;;

    esac
done

if [ "$type" = "" ];
then
   type=SA;
fi
       
if [ "$pkg" = "" ]
then
  Usage
fi


if [ "$pkg" = "OBC" ] ;
then
    if [ "$type" = "SA" ];
    then
       mode=1;
    else
        if [ "$type" = "MH" ];
        then
            mode=2;
        else
            if [ "$type" = "CS" ];
            then
                mode=3;
                DC=$LH
            else    
                Usage
            fi
        fi
    fi
    if  [ "$LH" = "" ];
    then
         Usage
    fi 
         
    `$REGCTL $REGFILE setvalue Software/VERITAS/VRTSobc/pal33/Agents/__defaults Mode REG_INT $mode`;
     rc=$?
    if [ "$rc" != "0" ];
    then
        exit $rc
    fi
    SetDefaults 
    if [ "$mode" = "2" ];
    then
        if  [ "$DC" = "" ];
        then
                Usage
        fi 
        if [ "$AB" = "" ];
        then
            AB=`$VEAINFO  auth_brokers -d $DC | sed -n '1p;1q'`
        fi
        rc=$?
        if [ "$rc" != "0" ];
        then
            exit $rc
        fi       
    fi
    if [ "$mode" != "1" ];
    then         
        if [ "$AB" = "" ];
        then
            Usage
        fi
        PD=vea_domain
        SADM=$AB:admin@broker.vx
        agent_id=$AB:vea_agent@vea_domain.vx
        update_security
    fi
    update_csf_resolv $AB $DC
fi

if [ "$pkg" = "OB" ] 
then

    mode=`$REGCTL $REGFILE queryvalue  Software/VERITAS/VRTSobc/pal${PAL_VERSION}/Agents/__defaults Mode`
    if [ $? != 0 ]
    then
    echo Default mode entry missing in registry 
    exit 1
    fi
    mode=`echo $mode |sed -e 's/^.*: //g'`
    
    
    $VEACONFIG -c security -s high -a root 
    rc=$?
    if [ "$rc" != "0" ];
    then
        exit $rc
    fi
# if central server set the agent if and populate domain list.    
    if [ $mode = 3 ];
    then
        AB=`/opt/VRTSobc/pal33/bin/vxini -f /etc/default/csf_resolv.conf -s SECURITY -k BROKER_HOST -q`
        if [ $? != 0 ]
        then
            echo broker host missing in csf_resolv.conf >> /var/vx/isis/cfg_error.log
            exit 2
        fi
        agent_id=$AB:vea_agent@vea_domain.vx
        SADM=$AB:admin@broker.vx
        $VEACONFIG -c setagentinfo  -a root -n $agent_id -m Domain 
        rc=$?
        if [ "$rc" != "0" ];
        then
            exit $rc
        fi
        $VEACONFIG -c setab -u $agent_id  
        rc=$?
        if [ "$rc" != "0" ];
        then
            exit $rc
        fi
    else
        $VEACONFIG -c setagentinfo  -a root  -m standalone
    fi
fi

# delete security admin certificate
if [ "$type" = "CS" ];
then
 $VEACONFIG -c deletecred -u $SADM   
fi

secpwd=""; export secpwd
DC=""; export DC
PWD=""; export PWD
AB=""; export AB
exit 0

